﻿// 2004/05/31
// dmd v0.91対応
// uuid.libが必要
// D3DXはLoadLibrary, GetProcAddressでロードすれば使える
// または
// import hdk.dll;
// DLL dll = new DLL(d3dx9d.dll);
// dll.load((void**)&D3DXCreateTextureFromFileW, "D3DXCreateTextureFromFileW");

module d3d9;

private import std.c.windows.windows;
private import std.c.windows.com;
private import std.math;
private import std.string;


const float D3DX_DEFAULT_FLOAT = float.max;
const float D3DX_PI = 3.141592654f;
const float D3DX_1BYPI = 0.318309886f;
const float D3DX_16F_DIG = 3;
const float D3DX_16F_EPSILON = 4.8875809e-4f;
const float D3DX_16F_MANT_DIG = 11;
const float D3DX_16F_MAX = 6.550400e+004;
const float D3DX_16F_MAX_10_EXP = 4;
const float D3DX_16F_MAX_EXP = 15;
const float D3DX_16F_MIN = 6.1035156e-5f;
const float D3DX_16F_MIN_10_EXP = (-4);
const float D3DX_16F_MIN_EXP = (-12);
const float D3DX_16F_RADIX = 2;
const float D3DX_16F_ROUNDS = 1;

enum : uint
{
	_FACDD = 0x876,
	D3DX_DEFAULT = uint.max,
	D3DX_DEFAULT_NONPOW2 = cast(uint)-2,
	UNUSED16 = (0xffff),
	UNUSED32 = (0xffffffff),
	LF_FACESIZE = (32),
	D3DX_VERSION = 0x0900,
	D3DX_SDK_VERSION = 9,
	D3DXFX_DONOTSAVESTATE = (1 << 0),
	D3DXFX_DONOTSAVESHADERSTATE = (1 << 1),
	D3DX_PARAMETER_SHARED = (1 << 0),
	D3DX_PARAMETER_LITERAL = (1 << 1),
	D3DX_PARAMETER_ANNOTATION = (1 << 2),
	D3DXSHADER_DEBUG = (1 << 0),
	D3DXSHADER_SKIPVALIDATION = (1 << 2),
	D3DXSHADER_SKIPOPTIMIZATION = (1 << 3),
	D3DXSHADER_PACKMATRIX_ROWMAJOR = (1 << 4),
	D3DXSHADER_PACKMATRIX_COLUMNMAJOR = (1 << 5),
	D3DX_FILTER_NONE = (1 << 0),
	D3DX_FILTER_POINT = (2 << 0),
	D3DX_FILTER_LINEAR = (3 << 0),
	D3DX_FILTER_TRIANGLE = (4 << 0),
	D3DX_FILTER_BOX = (5 << 0),
	D3DX_FILTER_MIRROR_U = (1 << 16),
	D3DX_FILTER_MIRROR_V = (2 << 16),
	D3DX_FILTER_MIRROR_W = (4 << 16),
	D3DX_FILTER_MIRROR = (7 << 16),
	D3DX_FILTER_DITHER = (1 << 19),
	D3DX_FILTER_SRGB_IN = (1 << 20),
	D3DX_FILTER_SRGB_OUT = (2 << 20),
	D3DX_FILTER_SRGB = (3 << 20),
	D3DX_NORMALMAP_MIRROR_U = (1 << 16),
	D3DX_NORMALMAP_MIRROR_V = (2 << 16),
	D3DX_NORMALMAP_MIRROR = (3 << 16),
	D3DX_NORMALMAP_INVERTSIGN = (8 << 16),
	D3DX_NORMALMAP_COMPUTE_OCCLUSION = (16 << 16),
	D3DX_CHANNEL_RED = (1 << 0),
	D3DX_CHANNEL_BLUE = (1 << 1),
	D3DX_CHANNEL_GREEN = (1 << 2),
	D3DX_CHANNEL_ALPHA = (1 << 3),
	D3DX_CHANNEL_LUMINANCE = (1 << 4),
	DIRECT3D_VERSION = 0x0900,
	D3D_SDK_VERSION = 31,
	DXFILEFORMAT_BINARY = 0,
	DXFILEFORMAT_TEXT = 1,
	DXFILEFORMAT_COMPRESSED = 2,
	DXFILELOAD_FROMFILE = 0x00,
	DXFILELOAD_FROMRESOURCE = 0x01,
	DXFILELOAD_FROMMEMORY = 0x02,
	DXFILELOAD_FROMSTREAM = 0x04,
	DXFILELOAD_FROMURL = 0x08,
	DXFILE_OK = 0,
	DXFILEERR_BADOBJECT = (1 << 31) | (_FACDD << 16) | (850),
	DXFILEERR_BADVALUE = (1 << 31) | (_FACDD << 16) | (851),
	DXFILEERR_BADTYPE = (1 << 31) | (_FACDD << 16) | (852),
	DXFILEERR_BADSTREAMHANDLE = (1 << 31) | (_FACDD << 16) | (853),
	DXFILEERR_BADALLOC = (1 << 31) | (_FACDD << 16) | (854),
	DXFILEERR_NOTFOUND = (1 << 31) | (_FACDD << 16) | (855),
	DXFILEERR_NOTDONEYET = (1 << 31) | (_FACDD << 16) | (856),
	DXFILEERR_FILENOTFOUND = (1 << 31) | (_FACDD << 16) | (857),
	DXFILEERR_RESOURCENOTFOUND = (1 << 31) | (_FACDD << 16) | (858),
	DXFILEERR_URLNOTFOUND = (1 << 31) | (_FACDD << 16) | (859),
	DXFILEERR_BADRESOURCE = (1 << 31) | (_FACDD << 16) | (860),
	DXFILEERR_BADFILETYPE = (1 << 31) | (_FACDD << 16) | (861),
	DXFILEERR_BADFILEVERSION = (1 << 31) | (_FACDD << 16) | (862),
	DXFILEERR_BADFILEFLOATSIZE = (1 << 31) | (_FACDD << 16) | (863),
	DXFILEERR_BADFILECOMPRESSIONTYPE = (1 << 31) | (_FACDD << 16) | (864),
	DXFILEERR_BADFILE = (1 << 31) | (_FACDD << 16) | (865),
	DXFILEERR_PARSEERROR = (1 << 31) | (_FACDD << 16) | (866),
	DXFILEERR_NOTEMPLATE = (1 << 31) | (_FACDD << 16) | (867),
	DXFILEERR_BADARRAYSIZE = (1 << 31) | (_FACDD << 16) | (868),
	DXFILEERR_BADDATAREFERENCE = (1 << 31) | (_FACDD << 16) | (869),
	DXFILEERR_INTERNALERROR = (1 << 31) | (_FACDD << 16) | (870),
	DXFILEERR_NOMOREOBJECTS = (1 << 31) | (_FACDD << 16) | (871),
	DXFILEERR_BADINTRINSICS = (1 << 31) | (_FACDD << 16) | (872),
	DXFILEERR_NOMORESTREAMHANDLES = (1 << 31) | (_FACDD << 16) | (873),
	DXFILEERR_NOMOREDATA = (1 << 31) | (_FACDD << 16) | (874),
	DXFILEERR_BADCACHEFILE = (1 << 31) | (_FACDD << 16) | (875),
	DXFILEERR_NOINTERNET = (1 << 31) | (_FACDD << 16) | (876),
	E_ACCESSDENIED = 485843,
	MMIO_CREATE = 0x00001000 ,
	MMIO_PARSE = 0x00000100 ,
	MMIO_DELETE = 0x00000200 ,
	MMIO_EXIST = 0x00004000 ,
	MMIO_ALLOCBUF = 0x00010000 ,
	MMIO_GETTEMP = 0x00020000 ,
	MMIO_DIRTY = 0x10000000 ,
	MMIO_READ = 0x00000000 ,
	MMIO_WRITE = 0x00000001 ,
	MMIO_READWRITE = 0x00000002 ,
	MMIO_COMPAT = 0x00000000 ,
	MMIO_EXCLUSIVE = 0x00000010 ,
	MMIO_DENYWRITE = 0x00000020 ,
	MMIO_DENYREAD = 0x00000030 ,
	MMIO_DENYNONE = 0x00000040 ,
	MMIO_FHOPEN = 0x0010 ,
	MMIO_EMPTYBUF = 0x0010 ,
	MMIO_TOUPPER = 0x0010 ,
	MMIO_INSTALLPROC = 0x00010000 ,
	MMIO_GLOBALPROC = 0x10000000 ,
	MMIO_REMOVEPROC = 0x00020000 ,
	MMIO_UNICODEPROC = 0x01000000 ,
	MMIO_FINDPROC = 0x00040000 ,
	MMIO_FINDCHUNK = 0x0010 ,
	MMIO_FINDRIFF = 0x0020 ,
	MMIO_FINDLIST = 0x0040 ,
	MMIO_CREATERIFF = 0x0020 ,
	MMIO_CREATELIST = 0x0040 ,
	MMIOM_READ = MMIO_READ ,
	MMIOM_WRITE = MMIO_WRITE ,
	MMIOM_SEEK = 2 ,
	MMIOM_OPEN = 3 ,
	MMIOM_CLOSE = 4 ,
	MMIOM_WRITEFLUSH = 5 ,
	WAVE_FORMAT_PCM = 1,
	RDH_RECTANGLES = 1,
	SEVERITY_ERROR = 1,
	FACILITY_WIN32 = 7,
	ERROR_OLD_WIN_VERSION = cast(int) 1150,
	ERROR_RMODE_APP = cast(int) 1153,
	ERROR_BAD_DRIVER_LEVEL = cast(int) 119,
	ERROR_NOT_READY = cast(int) 21,
	ERROR_ALREADY_INITIALIZED = cast(int) 1247,
	ERROR_READ_FAULT = cast(int) 30,
	ERROR_BUSY = cast(int) 170,
	ERROR_INVALID_ACCESS = cast(int) 12,
	REGDB_E_CLASSNOTREG = 0,
	D3DSPD_IUNKNOWN = 0x00000001L,
	D3DCREATE_FPU_PRESERVE = 0x00000002L,
	D3DCREATE_MULTITHREADED = 0x00000004L,
	D3DCREATE_PUREDEVICE = 0x00000010L,
	D3DCREATE_SOFTWARE_VERTEXPROCESSING = 0x00000020L,
	D3DCREATE_HARDWARE_VERTEXPROCESSING = 0x00000040L,
	D3DCREATE_MIXED_VERTEXPROCESSING = 0x00000080L,
	D3DCREATE_DISABLE_DRIVER_MANAGEMENT = 0x00000100L,
	D3DCREATE_ADAPTERGROUP_DEVICE = 0x00000200L,
	D3DADAPTER_DEFAULT = 0,
	D3DENUM_WHQL_LEVEL = 0x00000002L,
	D3DPRESENT_BACK_BUFFERS_MAX = 3L,
	D3DSGR_NO_CALIBRATION = 0x00000000L,
	D3DSGR_CALIBRATE = 0x00000001L,
	D3DCURSOR_IMMEDIATE_UPDATE = 0x00000001L,
	D3DPRESENT_DONOTWAIT = 0x00000001L,
	D3DPRESENT_LINEAR_CONTENT = 0x00000002L,
	_FACD3D = 0x876,
	D3D_OK = 0,
	D3DERR_WRONGTEXTUREFORMAT = (1 << 31) | (0x876 << 16) | (2072),
	D3DERR_UNSUPPORTEDCOLOROPERATION = (1 << 31) | (0x876 << 16) | (2073),
	D3DERR_UNSUPPORTEDCOLORARG = (1 << 31) | (0x876 << 16) | (2074),
	D3DERR_UNSUPPORTEDALPHAOPERATION = (1 << 31) | (0x876 << 16) | (2075),
	D3DERR_UNSUPPORTEDALPHAARG = (1 << 31) | (0x876 << 16) | (2076),
	D3DERR_TOOMANYOPERATIONS = (1 << 31) | (0x876 << 16) | (2077),
	D3DERR_CONFLICTINGTEXTUREFILTER = (1 << 31) | (0x876 << 16) | (2078),
	D3DERR_UNSUPPORTEDFACTORVALUE = (1 << 31) | (0x876 << 16) | (2079),
	D3DERR_CONFLICTINGRENDERSTATE = (1 << 31) | (0x876 << 16) | (2081),
	D3DERR_UNSUPPORTEDTEXTUREFILTER = (1 << 31) | (0x876 << 16) | (2082),
	D3DERR_CONFLICTINGTEXTUREPALETTE = (1 << 31) | (0x876 << 16) | (2086),
	D3DERR_DRIVERINTERNALERROR = (1 << 31) | (0x876 << 16) | (2087),
	D3DERR_NOTFOUND = (1 << 31) | (0x876 << 16) | (2150),
	D3DERR_MOREDATA = (1 << 31) | (0x876 << 16) | (2151),
	D3DERR_DEVICELOST = (1 << 31) | (0x876 << 16) | (2152),
	D3DERR_DEVICENOTRESET = (1 << 31) | (0x876 << 16) | (2153),
	D3DERR_NOTAVAILABLE = (1 << 31) | (0x876 << 16) | (2154),
	D3DERR_OUTOFVIDEOMEMORY = (1 << 31) | (0x876 << 16) | (380),
	D3DERR_INVALIDDEVICE = (1 << 31) | (0x876 << 16) | (2155),
	D3DERR_INVALIDCALL = (1 << 31) | (0x876 << 16) | (2156),
	D3DERR_DRIVERINVALIDCALL = (1 << 31) | (0x876 << 16) | (2157),
	D3DERR_WASSTILLDRAWING = (1 << 31) | (0x876 << 16) | (540),
	D3DOK_NOAUTOGEN = (0 << 31) | (0x876 << 16) | (2159),
	D3DVS20CAPS_PREDICATION = (1<<0),
	D3DVS20_MAX_DYNAMICFLOWCONTROLDEPTH = 24,
	D3DVS20_MIN_DYNAMICFLOWCONTROLDEPTH = 0,
	D3DVS20_MAX_NUMTEMPS = 32,
	D3DVS20_MIN_NUMTEMPS = 12,
	D3DVS20_MAX_STATICFLOWCONTROLDEPTH = 4,
	D3DVS20_MIN_STATICFLOWCONTROLDEPTH = 1,
	D3DPS20CAPS_ARBITRARYSWIZZLE = (1<<0),
	D3DPS20CAPS_GRADIENTINSTRUCTIONS = (1<<1),
	D3DPS20CAPS_PREDICATION = (1<<2),
	D3DPS20CAPS_NODEPENDENTREADLIMIT = (1<<3),
	D3DPS20CAPS_NOTEXINSTRUCTIONLIMIT = (1<<4),
	D3DPS20_MAX_DYNAMICFLOWCONTROLDEPTH = 24,
	D3DPS20_MIN_DYNAMICFLOWCONTROLDEPTH = 0,
	D3DPS20_MAX_NUMTEMPS = 32,
	D3DPS20_MIN_NUMTEMPS = 12,
	D3DPS20_MAX_STATICFLOWCONTROLDEPTH = 4,
	D3DPS20_MIN_STATICFLOWCONTROLDEPTH = 0,
	D3DPS20_MAX_NUMINSTRUCTIONSLOTS = 512,
	D3DPS20_MIN_NUMINSTRUCTIONSLOTS = 96,
	D3DMIN30SHADERINSTRUCTIONS = 512,
	D3DMAX30SHADERINSTRUCTIONS = 32768,
	D3DCAPS_READ_SCANLINE = 0x00020000L,
	D3DCAPS2_FULLSCREENGAMMA = 0x00020000L,
	D3DCAPS2_CANCALIBRATEGAMMA = 0x00100000L,
	D3DCAPS2_RESERVED = 0x02000000L,
	D3DCAPS2_CANMANAGERESOURCE = 0x10000000L,
	D3DCAPS2_DYNAMICTEXTURES = 0x20000000L,
	D3DCAPS2_CANAUTOGENMIPMAP = 0x40000000L,
	D3DCAPS3_RESERVED = 0x8000001fL,
	D3DCAPS3_ALPHA_FULLSCREEN_FLIP_OR_DISCARD = 0x00000020L,
	D3DCAPS3_LINEAR_TO_SRGB_PRESENTATION = 0x00000080L,
	D3DCAPS3_COPY_TO_VIDMEM = 0x00000100L,
	D3DCAPS3_COPY_TO_SYSTEMMEM = 0x00000200L,
	D3DPRESENT_INTERVAL_DEFAULT = 0x00000000L,
	D3DPRESENT_INTERVAL_ONE = 0x00000001L,
	D3DPRESENT_INTERVAL_TWO = 0x00000002L,
	D3DPRESENT_INTERVAL_THREE = 0x00000004L,
	D3DPRESENT_INTERVAL_FOUR = 0x00000008L,
	D3DPRESENT_INTERVAL_IMMEDIATE = 0x80000000L,
	D3DCURSORCAPS_COLOR = 0x00000001L,
	D3DCURSORCAPS_LOWRES = 0x00000002L,
	D3DDEVCAPS_EXECUTESYSTEMMEMORY = 0x00000010L ,
	D3DDEVCAPS_EXECUTEVIDEOMEMORY = 0x00000020L ,
	D3DDEVCAPS_TLVERTEXSYSTEMMEMORY = 0x00000040L ,
	D3DDEVCAPS_TLVERTEXVIDEOMEMORY = 0x00000080L ,
	D3DDEVCAPS_TEXTURESYSTEMMEMORY = 0x00000100L ,
	D3DDEVCAPS_TEXTUREVIDEOMEMORY = 0x00000200L ,
	D3DDEVCAPS_DRAWPRIMTLVERTEX = 0x00000400L ,
	D3DDEVCAPS_CANRENDERAFTERFLIP = 0x00000800L ,
	D3DDEVCAPS_TEXTURENONLOCALVIDMEM = 0x00001000L ,
	D3DDEVCAPS_DRAWPRIMITIVES2 = 0x00002000L ,
	D3DDEVCAPS_SEPARATETEXTUREMEMORIES = 0x00004000L ,
	D3DDEVCAPS_DRAWPRIMITIVES2EX = 0x00008000L ,
	D3DDEVCAPS_HWTRANSFORMANDLIGHT = 0x00010000L ,
	D3DDEVCAPS_CANBLTSYSTONONLOCAL = 0x00020000L ,
	D3DDEVCAPS_HWRASTERIZATION = 0x00080000L ,
	D3DDEVCAPS_PUREDEVICE = 0x00100000L ,
	D3DDEVCAPS_QUINTICRTPATCHES = 0x00200000L ,
	D3DDEVCAPS_RTPATCHES = 0x00400000L ,
	D3DDEVCAPS_RTPATCHHANDLEZERO = 0x00800000L ,
	D3DDEVCAPS_NPATCHES = 0x01000000L ,
	D3DPMISCCAPS_MASKZ = 0x00000002L,
	D3DPMISCCAPS_CULLNONE = 0x00000010L,
	D3DPMISCCAPS_CULLCW = 0x00000020L,
	D3DPMISCCAPS_CULLCCW = 0x00000040L,
	D3DPMISCCAPS_COLORWRITEENABLE = 0x00000080L,
	D3DPMISCCAPS_CLIPPLANESCALEDPOINTS = 0x00000100L ,
	D3DPMISCCAPS_CLIPTLVERTS = 0x00000200L ,
	D3DPMISCCAPS_TSSARGTEMP = 0x00000400L ,
	D3DPMISCCAPS_BLENDOP = 0x00000800L ,
	D3DPMISCCAPS_NULLREFERENCE = 0x00001000L ,
	D3DPMISCCAPS_INDEPENDENTWRITEMASKS = 0x00004000L ,
	D3DPMISCCAPS_PERSTAGECONSTANT = 0x00008000L ,
	D3DPMISCCAPS_FOGANDSPECULARALPHA = 0x00010000L ,
	D3DPMISCCAPS_SEPARATEALPHABLEND = 0x00020000L ,
	D3DPMISCCAPS_MRTINDEPENDENTBITDEPTHS = 0x00040000L ,
	D3DPMISCCAPS_MRTPOSTPIXELSHADERBLENDING = 0x00080000L ,
	D3DPMISCCAPS_FOGVERTEXCLAMPED = 0x00100000L ,
	D3DLINECAPS_TEXTURE = 0x00000001L,
	D3DLINECAPS_ZTEST = 0x00000002L,
	D3DLINECAPS_BLEND = 0x00000004L,
	D3DLINECAPS_ALPHACMP = 0x00000008L,
	D3DLINECAPS_FOG = 0x00000010L,
	D3DLINECAPS_ANTIALIAS = 0x00000020L,
	D3DPRASTERCAPS_DITHER = 0x00000001L,
	D3DPRASTERCAPS_ZTEST = 0x00000010L,
	D3DPRASTERCAPS_FOGVERTEX = 0x00000080L,
	D3DPRASTERCAPS_FOGTABLE = 0x00000100L,
	D3DPRASTERCAPS_MIPMAPLODBIAS = 0x00002000L,
	D3DPRASTERCAPS_ZBUFFERLESSHSR = 0x00008000L,
	D3DPRASTERCAPS_FOGRANGE = 0x00010000L,
	D3DPRASTERCAPS_ANISOTROPY = 0x00020000L,
	D3DPRASTERCAPS_WBUFFER = 0x00040000L,
	D3DPRASTERCAPS_WFOG = 0x00100000L,
	D3DPRASTERCAPS_ZFOG = 0x00200000L,
	D3DPRASTERCAPS_COLORPERSPECTIVE = 0x00400000L,
	D3DPRASTERCAPS_SCISSORTEST = 0x01000000L,
	D3DPRASTERCAPS_SLOPESCALEDEPTHBIAS = 0x02000000L,
	D3DPRASTERCAPS_DEPTHBIAS = 0x04000000L,
	D3DPRASTERCAPS_MULTISAMPLE_TOGGLE = 0x08000000L,
	D3DPCMPCAPS_NEVER = 0x00000001L,
	D3DPCMPCAPS_LESS = 0x00000002L,
	D3DPCMPCAPS_EQUAL = 0x00000004L,
	D3DPCMPCAPS_LESSEQUAL = 0x00000008L,
	D3DPCMPCAPS_GREATER = 0x00000010L,
	D3DPCMPCAPS_NOTEQUAL = 0x00000020L,
	D3DPCMPCAPS_GREATEREQUAL = 0x00000040L,
	D3DPCMPCAPS_ALWAYS = 0x00000080L,
	D3DPBLENDCAPS_ZERO = 0x00000001L,
	D3DPBLENDCAPS_ONE = 0x00000002L,
	D3DPBLENDCAPS_SRCCOLOR = 0x00000004L,
	D3DPBLENDCAPS_INVSRCCOLOR = 0x00000008L,
	D3DPBLENDCAPS_SRCALPHA = 0x00000010L,
	D3DPBLENDCAPS_INVSRCALPHA = 0x00000020L,
	D3DPBLENDCAPS_DESTALPHA = 0x00000040L,
	D3DPBLENDCAPS_INVDESTALPHA = 0x00000080L,
	D3DPBLENDCAPS_DESTCOLOR = 0x00000100L,
	D3DPBLENDCAPS_INVDESTCOLOR = 0x00000200L,
	D3DPBLENDCAPS_SRCALPHASAT = 0x00000400L,
	D3DPBLENDCAPS_BOTHSRCALPHA = 0x00000800L,
	D3DPBLENDCAPS_BOTHINVSRCALPHA = 0x00001000L,
	D3DPBLENDCAPS_BLENDFACTOR = 0x00002000L,
	D3DPSHADECAPS_COLORGOURAUDRGB = 0x00000008L,
	D3DPSHADECAPS_SPECULARGOURAUDRGB = 0x00000200L,
	D3DPSHADECAPS_ALPHAGOURAUDBLEND = 0x00004000L,
	D3DPSHADECAPS_FOGGOURAUD = 0x00080000L,
	D3DPTEXTURECAPS_PERSPECTIVE = 0x00000001L ,
	D3DPTEXTURECAPS_POW2 = 0x00000002L ,
	D3DPTEXTURECAPS_ALPHA = 0x00000004L ,
	D3DPTEXTURECAPS_SQUAREONLY = 0x00000020L ,
	D3DPTEXTURECAPS_TEXREPEATNOTSCALEDBYSIZE = 0x00000040L ,
	D3DPTEXTURECAPS_ALPHAPALETTE = 0x00000080L ,
	D3DPTEXTURECAPS_NONPOW2CONDITIONAL = 0x00000100L,
	D3DPTEXTURECAPS_PROJECTED = 0x00000400L ,
	D3DPTEXTURECAPS_CUBEMAP = 0x00000800L ,
	D3DPTEXTURECAPS_VOLUMEMAP = 0x00002000L ,
	D3DPTEXTURECAPS_MIPMAP = 0x00004000L ,
	D3DPTEXTURECAPS_MIPVOLUMEMAP = 0x00008000L ,
	D3DPTEXTURECAPS_MIPCUBEMAP = 0x00010000L ,
	D3DPTEXTURECAPS_CUBEMAP_POW2 = 0x00020000L ,
	D3DPTEXTURECAPS_VOLUMEMAP_POW2 = 0x00040000L ,
	D3DPTEXTURECAPS_NOPROJECTEDBUMPENV = 0x00200000L ,
	D3DPTFILTERCAPS_MINFPOINT = 0x00000100L ,
	D3DPTFILTERCAPS_MINFLINEAR = 0x00000200L,
	D3DPTFILTERCAPS_MINFANISOTROPIC = 0x00000400L,
	D3DPTFILTERCAPS_MINFPYRAMIDALQUAD = 0x00000800L,
	D3DPTFILTERCAPS_MINFGAUSSIANQUAD = 0x00001000L,
	D3DPTFILTERCAPS_MIPFPOINT = 0x00010000L ,
	D3DPTFILTERCAPS_MIPFLINEAR = 0x00020000L,
	D3DPTFILTERCAPS_MAGFPOINT = 0x01000000L ,
	D3DPTFILTERCAPS_MAGFLINEAR = 0x02000000L,
	D3DPTFILTERCAPS_MAGFANISOTROPIC = 0x04000000L,
	D3DPTFILTERCAPS_MAGFPYRAMIDALQUAD = 0x08000000L,
	D3DPTFILTERCAPS_MAGFGAUSSIANQUAD = 0x10000000L,
	D3DPTADDRESSCAPS_WRAP = 0x00000001L,
	D3DPTADDRESSCAPS_MIRROR = 0x00000002L,
	D3DPTADDRESSCAPS_CLAMP = 0x00000004L,
	D3DPTADDRESSCAPS_BORDER = 0x00000008L,
	D3DPTADDRESSCAPS_INDEPENDENTUV = 0x00000010L,
	D3DPTADDRESSCAPS_MIRRORONCE = 0x00000020L,
	D3DSTENCILCAPS_KEEP = 0x00000001L,
	D3DSTENCILCAPS_ZERO = 0x00000002L,
	D3DSTENCILCAPS_REPLACE = 0x00000004L,
	D3DSTENCILCAPS_INCRSAT = 0x00000008L,
	D3DSTENCILCAPS_DECRSAT = 0x00000010L,
	D3DSTENCILCAPS_INVERT = 0x00000020L,
	D3DSTENCILCAPS_INCR = 0x00000040L,
	D3DSTENCILCAPS_DECR = 0x00000080L,
	D3DSTENCILCAPS_TWOSIDED = 0x00000100L,
	D3DTEXOPCAPS_DISABLE = 0x00000001L,
	D3DTEXOPCAPS_SELECTARG1 = 0x00000002L,
	D3DTEXOPCAPS_SELECTARG2 = 0x00000004L,
	D3DTEXOPCAPS_MODULATE = 0x00000008L,
	D3DTEXOPCAPS_MODULATE2X = 0x00000010L,
	D3DTEXOPCAPS_MODULATE4X = 0x00000020L,
	D3DTEXOPCAPS_ADD = 0x00000040L,
	D3DTEXOPCAPS_ADDSIGNED = 0x00000080L,
	D3DTEXOPCAPS_ADDSIGNED2X = 0x00000100L,
	D3DTEXOPCAPS_SUBTRACT = 0x00000200L,
	D3DTEXOPCAPS_ADDSMOOTH = 0x00000400L,
	D3DTEXOPCAPS_BLENDDIFFUSEALPHA = 0x00000800L,
	D3DTEXOPCAPS_BLENDTEXTUREALPHA = 0x00001000L,
	D3DTEXOPCAPS_BLENDFACTORALPHA = 0x00002000L,
	D3DTEXOPCAPS_BLENDTEXTUREALPHAPM = 0x00004000L,
	D3DTEXOPCAPS_BLENDCURRENTALPHA = 0x00008000L,
	D3DTEXOPCAPS_PREMODULATE = 0x00010000L,
	D3DTEXOPCAPS_MODULATEALPHA_ADDCOLOR = 0x00020000L,
	D3DTEXOPCAPS_MODULATECOLOR_ADDALPHA = 0x00040000L,
	D3DTEXOPCAPS_MODULATEINVALPHA_ADDCOLOR = 0x00080000L,
	D3DTEXOPCAPS_MODULATEINVCOLOR_ADDALPHA = 0x00100000L,
	D3DTEXOPCAPS_BUMPENVMAP = 0x00200000L,
	D3DTEXOPCAPS_BUMPENVMAPLUMINANCE = 0x00400000L,
	D3DTEXOPCAPS_DOTPRODUCT3 = 0x00800000L,
	D3DTEXOPCAPS_MULTIPLYADD = 0x01000000L,
	D3DTEXOPCAPS_LERP = 0x02000000L,
	D3DFVFCAPS_TEXCOORDCOUNTMASK = 0x0000ffffL ,
	D3DFVFCAPS_DONOTSTRIPELEMENTS = 0x00080000L ,
	D3DFVFCAPS_PSIZE = 0x00100000L ,
	D3DVTXPCAPS_TEXGEN = 0x00000001L ,
	D3DVTXPCAPS_MATERIALSOURCE7 = 0x00000002L ,
	D3DVTXPCAPS_DIRECTIONALLIGHTS = 0x00000008L ,
	D3DVTXPCAPS_POSITIONALLIGHTS = 0x00000010L ,
	D3DVTXPCAPS_LOCALVIEWER = 0x00000020L ,
	D3DVTXPCAPS_TWEENING = 0x00000040L ,
	D3DVTXPCAPS_TEXGEN_SPHEREMAP = 0x00000100L ,
	D3DVTXPCAPS_NO_TEXGEN_NONLOCALVIEWER = 0x00000200L ,
	D3DDEVCAPS2_STREAMOFFSET = 0x00000001L ,
	D3DDEVCAPS2_DMAPNPATCH = 0x00000002L ,
	D3DDEVCAPS2_ADAPTIVETESSRTPATCH = 0x00000004L ,
	D3DDEVCAPS2_ADAPTIVETESSNPATCH = 0x00000008L ,
	D3DDEVCAPS2_CAN_STRETCHRECT_FROM_TEXTURES = 0x00000010L ,
	D3DDEVCAPS2_PRESAMPLEDDMAPNPATCH = 0x00000020L ,
	D3DDEVCAPS2_VERTEXELEMENTSCANSHARESTREAMOFFSET = 0x00000040L ,
	D3DDTCAPS_UBYTE4 = 0x00000001L,
	D3DDTCAPS_UBYTE4N = 0x00000002L,
	D3DDTCAPS_SHORT2N = 0x00000004L,
	D3DDTCAPS_SHORT4N = 0x00000008L,
	D3DDTCAPS_USHORT2N = 0x00000010L,
	D3DDTCAPS_USHORT4N = 0x00000020L,
	D3DDTCAPS_UDEC3 = 0x00000040L,
	D3DDTCAPS_DEC3N = 0x00000080L,
	D3DDTCAPS_FLOAT16_2 = 0x00000100L,
	D3DDTCAPS_FLOAT16_4 = 0x00000200L,
	D3DMAXUSERCLIPPLANES = 32,
	D3DCLIPPLANE0 = (1 << 0),
	D3DCLIPPLANE1 = (1 << 1),
	D3DCLIPPLANE2 = (1 << 2),
	D3DCLIPPLANE3 = (1 << 3),
	D3DCLIPPLANE4 = (1 << 4),
	D3DCLIPPLANE5 = (1 << 5),
	D3DCS_LEFT = 0x00000001L,
	D3DCS_RIGHT = 0x00000002L,
	D3DCS_TOP = 0x00000004L,
	D3DCS_BOTTOM = 0x00000008L,
	D3DCS_FRONT = 0x00000010L,
	D3DCS_BACK = 0x00000020L,
	D3DCS_PLANE0 = 0x00000040L,
	D3DCS_PLANE1 = 0x00000080L,
	D3DCS_PLANE2 = 0x00000100L,
	D3DCS_PLANE3 = 0x00000200L,
	D3DCS_PLANE4 = 0x00000400L,
	D3DCS_PLANE5 = 0x00000800L,
	D3DCS_ALL = (D3DCS_LEFT | D3DCS_RIGHT | D3DCS_TOP | D3DCS_BOTTOM | D3DCS_FRONT | D3DCS_BACK
		| D3DCS_PLANE0 | D3DCS_PLANE1 | D3DCS_PLANE2 | D3DCS_PLANE3 | D3DCS_PLANE4 | D3DCS_PLANE5),
	D3DCLEAR_TARGET = 0x00000001l ,
	D3DCLEAR_ZBUFFER = 0x00000002l ,
	D3DCLEAR_STENCIL = 0x00000004l ,
	D3D_MAX_SIMULTANEOUS_RENDERTARGETS = 4,
	D3DRENDERSTATE_WRAPBIAS = 128UL,
	D3DWRAP_U = 0x00000001L,
	D3DWRAP_V = 0x00000002L,
	D3DWRAP_W = 0x00000004L,
	D3DWRAPCOORD_0 = 0x00000001L,
	D3DWRAPCOORD_1 = 0x00000002L,
	D3DWRAPCOORD_2 = 0x00000004L,
	D3DWRAPCOORD_3 = 0x00000008L,
	D3DCOLORWRITEENABLE_RED = (1L<<0),
	D3DCOLORWRITEENABLE_GREEN = (1L<<1),
	D3DCOLORWRITEENABLE_BLUE = (1L<<2),
	D3DCOLORWRITEENABLE_ALPHA = (1L<<3),
	D3DDMAPSAMPLER = 256,
	D3DVERTEXTEXTURESAMPLER0 = (D3DDMAPSAMPLER+1),
	D3DVERTEXTEXTURESAMPLER1 = (D3DDMAPSAMPLER+2),
	D3DVERTEXTEXTURESAMPLER2 = (D3DDMAPSAMPLER+3),
	D3DVERTEXTEXTURESAMPLER3 = (D3DDMAPSAMPLER+4),
	D3DTSS_TCI_PASSTHRU = 0x00000000,
	D3DTSS_TCI_CAMERASPACENORMAL = 0x00010000,
	D3DTSS_TCI_CAMERASPACEPOSITION = 0x00020000,
	D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR = 0x00030000,
	D3DTSS_TCI_SPHEREMAP = 0x00040000,
	D3DTA_SELECTMASK = 0x0000000f ,
	D3DTA_DIFFUSE = 0x00000000 ,
	D3DTA_CURRENT = 0x00000001 ,
	D3DTA_TEXTURE = 0x00000002 ,
	D3DTA_TFACTOR = 0x00000003 ,
	D3DTA_SPECULAR = 0x00000004 ,
	D3DTA_TEMP = 0x00000005 ,
	D3DTA_CONSTANT = 0x00000006 ,
	D3DTA_COMPLEMENT = 0x00000010 ,
	D3DTA_ALPHAREPLICATE = 0x00000020 ,
	D3DPV_DONOTCOPYDATA = (1 << 0),
	D3DFVF_RESERVED0 = 0x001,
	D3DFVF_POSITION_MASK = 0x400E,
	D3DFVF_XYZ = 0x002,
	D3DFVF_XYZRHW = 0x004,
	D3DFVF_XYZB1 = 0x006,
	D3DFVF_XYZB2 = 0x008,
	D3DFVF_XYZB3 = 0x00a,
	D3DFVF_XYZB4 = 0x00c,
	D3DFVF_XYZB5 = 0x00e,
	D3DFVF_XYZW = 0x4002,
	D3DFVF_NORMAL = 0x010,
	D3DFVF_PSIZE = 0x020,
	D3DFVF_DIFFUSE = 0x040,
	D3DFVF_SPECULAR = 0x080,
	D3DFVF_TEXCOUNT_MASK = 0xf00,
	D3DFVF_TEXCOUNT_SHIFT = 8,
	D3DFVF_TEX0 = 0x000,
	D3DFVF_TEX1 = 0x100,
	D3DFVF_TEX2 = 0x200,
	D3DFVF_TEX3 = 0x300,
	D3DFVF_TEX4 = 0x400,
	D3DFVF_TEX5 = 0x500,
	D3DFVF_TEX6 = 0x600,
	D3DFVF_TEX7 = 0x700,
	D3DFVF_TEX8 = 0x800,
	D3DFVF_LASTBETA_UBYTE4 = 0x1000,
	D3DFVF_LASTBETA_D3DCOLOR = 0x8000,
	D3DFVF_RESERVED2 = 0x6000,
	MAXD3DDECLUSAGEINDEX = 15,
	MAXD3DDECLLENGTH = 64,
	D3DDP_MAXTEXCOORD = 8,
	D3DSI_OPCODE_MASK = 0x0000FFFF,
	D3DSI_INSTLENGTH_MASK = 0x0F000000,
	D3DSI_INSTLENGTH_SHIFT = 24,
	D3DSI_COISSUE = 0x40000000,
	D3DSP_OPCODESPECIFICCONTROL_MASK = 0x00ff0000,
	D3DSP_OPCODESPECIFICCONTROL_SHIFT = 16,
	D3DSI_TEXLD_PROJECT = (0x01 << D3DSP_OPCODESPECIFICCONTROL_SHIFT),
	D3DSI_TEXLD_BIAS = (0x02 << D3DSP_OPCODESPECIFICCONTROL_SHIFT),
	D3DSHADER_COMPARISON_SHIFT = D3DSP_OPCODESPECIFICCONTROL_SHIFT,
	D3DSHADER_COMPARISON_MASK = (0x7<<D3DSHADER_COMPARISON_SHIFT),
	D3DSHADER_INSTRUCTION_PREDICATED = (0x1 << 28),
	D3DSP_DCL_USAGE_SHIFT = 0,
	D3DSP_DCL_USAGE_MASK = 0x0000000f,
	D3DSP_DCL_USAGEINDEX_SHIFT = 16,
	D3DSP_DCL_USAGEINDEX_MASK = 0x000f0000,
	D3DSP_TEXTURETYPE_SHIFT = 27,
	D3DSP_TEXTURETYPE_MASK = 0x78000000,
	D3DSP_REGNUM_MASK = 0x000007FF,
	D3DSP_WRITEMASK_0 = 0x00010000,
	D3DSP_WRITEMASK_1 = 0x00020000,
	D3DSP_WRITEMASK_2 = 0x00040000,
	D3DSP_WRITEMASK_3 = 0x00080000,
	D3DSP_WRITEMASK_ALL = 0x000F0000,
	D3DSP_DSTMOD_SHIFT = 20,
	D3DSP_DSTMOD_MASK = 0x00F00000,
	D3DSPDM_NONE = (0<<D3DSP_DSTMOD_SHIFT),
	D3DSPDM_SATURATE = (1<<D3DSP_DSTMOD_SHIFT),
	D3DSPDM_PARTIALPRECISION = (2<<D3DSP_DSTMOD_SHIFT),
	D3DSPDM_MSAMPCENTROID = (4<<D3DSP_DSTMOD_SHIFT),
	D3DSP_DSTSHIFT_SHIFT = 24,
	D3DSP_DSTSHIFT_MASK = 0x0F000000,
	D3DSP_REGTYPE_SHIFT = 28,
	D3DSP_REGTYPE_SHIFT2 = 8,
	D3DSP_REGTYPE_MASK = 0x70000000,
	D3DSP_REGTYPE_MASK2 = 0x00001800,
	D3DVS_ADDRESSMODE_SHIFT = 13,
	D3DVS_ADDRESSMODE_MASK = (1 << D3DVS_ADDRESSMODE_SHIFT),
	D3DSHADER_ADDRESSMODE_SHIFT = 13,
	D3DSHADER_ADDRESSMODE_MASK = (1 << D3DSHADER_ADDRESSMODE_SHIFT),
	D3DVS_SWIZZLE_SHIFT = 16,
	D3DVS_SWIZZLE_MASK = 0x00FF0000,
	D3DVS_X_X = (0 << D3DVS_SWIZZLE_SHIFT),
	D3DVS_X_Y = (1 << D3DVS_SWIZZLE_SHIFT),
	D3DVS_X_Z = (2 << D3DVS_SWIZZLE_SHIFT),
	D3DVS_X_W = (3 << D3DVS_SWIZZLE_SHIFT),
	D3DVS_Y_X = (0 << (D3DVS_SWIZZLE_SHIFT + 2)),
	D3DVS_Y_Y = (1 << (D3DVS_SWIZZLE_SHIFT + 2)),
	D3DVS_Y_Z = (2 << (D3DVS_SWIZZLE_SHIFT + 2)),
	D3DVS_Y_W = (3 << (D3DVS_SWIZZLE_SHIFT + 2)),
	D3DVS_Z_X = (0 << (D3DVS_SWIZZLE_SHIFT + 4)),
	D3DVS_Z_Y = (1 << (D3DVS_SWIZZLE_SHIFT + 4)),
	D3DVS_Z_Z = (2 << (D3DVS_SWIZZLE_SHIFT + 4)),
	D3DVS_Z_W = (3 << (D3DVS_SWIZZLE_SHIFT + 4)),
	D3DVS_W_X = (0 << (D3DVS_SWIZZLE_SHIFT + 6)),
	D3DVS_W_Y = (1 << (D3DVS_SWIZZLE_SHIFT + 6)),
	D3DVS_W_Z = (2 << (D3DVS_SWIZZLE_SHIFT + 6)),
	D3DVS_W_W = (3 << (D3DVS_SWIZZLE_SHIFT + 6)),
	D3DVS_NOSWIZZLE = (D3DVS_X_X | D3DVS_Y_Y | D3DVS_Z_Z | D3DVS_W_W),
	D3DSP_SWIZZLE_SHIFT = 16,
	D3DSP_SWIZZLE_MASK = 0x00FF0000,
	D3DSP_NOSWIZZLE = ( (0 << (D3DSP_SWIZZLE_SHIFT + 0)) | (1 << (D3DSP_SWIZZLE_SHIFT + 2)) | (2 << (D3DSP_SWIZZLE_SHIFT + 4)) | (3 << (D3DSP_SWIZZLE_SHIFT + 6)) ),
	D3DSP_REPLICATERED = ( (0 << (D3DSP_SWIZZLE_SHIFT + 0)) | (0 << (D3DSP_SWIZZLE_SHIFT + 2)) | (0 << (D3DSP_SWIZZLE_SHIFT + 4)) | (0 << (D3DSP_SWIZZLE_SHIFT + 6)) ),
	D3DSP_REPLICATEGREEN = ( (1 << (D3DSP_SWIZZLE_SHIFT + 0)) | (1 << (D3DSP_SWIZZLE_SHIFT + 2)) | (1 << (D3DSP_SWIZZLE_SHIFT + 4)) | (1 << (D3DSP_SWIZZLE_SHIFT + 6)) ),
	D3DSP_REPLICATEBLUE = ( (2 << (D3DSP_SWIZZLE_SHIFT + 0)) | (2 << (D3DSP_SWIZZLE_SHIFT + 2)) | (2 << (D3DSP_SWIZZLE_SHIFT + 4)) | (2 << (D3DSP_SWIZZLE_SHIFT + 6)) ),
	D3DSP_REPLICATEALPHA = ( (3 << (D3DSP_SWIZZLE_SHIFT + 0)) | (3 << (D3DSP_SWIZZLE_SHIFT + 2)) | (3 << (D3DSP_SWIZZLE_SHIFT + 4)) | (3 << (D3DSP_SWIZZLE_SHIFT + 6)) ),
	D3DSP_SRCMOD_SHIFT = 24,
	D3DSP_SRCMOD_MASK = 0x0F000000,
	D3DSI_COMMENTSIZE_SHIFT = 16,
	D3DSI_COMMENTSIZE_MASK = 0x7FFF0000,
	D3DFVF_TEXTUREFORMAT2 = 0,
	D3DFVF_TEXTUREFORMAT1 = 3,
	D3DFVF_TEXTUREFORMAT3 = 1,
	D3DFVF_TEXTUREFORMAT4 = 2,
	D3DPRESENT_RATE_DEFAULT = 0x00000000,
	D3DPRESENTFLAG_LOCKABLE_BACKBUFFER = 0x00000001,
	D3DPRESENTFLAG_DISCARD_DEPTHSTENCIL = 0x00000002,
	D3DPRESENTFLAG_DEVICECLIP = 0x00000004,
	D3DPRESENTFLAG_VIDEO = 0x00000010,
	D3DUSAGE_RENDERTARGET = (0x00000001L),
	D3DUSAGE_DEPTHSTENCIL = (0x00000002L),
	D3DUSAGE_DYNAMIC = (0x00000200L),
	D3DUSAGE_AUTOGENMIPMAP = (0x00000400L),
	D3DUSAGE_DMAP = (0x00004000L),
	D3DUSAGE_QUERY_LEGACYBUMPMAP = (0x00008000L),
	D3DUSAGE_QUERY_SRGBREAD = (0x00010000L),
	D3DUSAGE_QUERY_FILTER = (0x00020000L),
	D3DUSAGE_QUERY_SRGBWRITE = (0x00040000L),
	D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING = (0x00080000L),
	D3DUSAGE_QUERY_VERTEXTEXTURE = (0x00100000L),
	D3DUSAGE_WRITEONLY = (0x00000008L),
	D3DUSAGE_SOFTWAREPROCESSING = (0x00000010L),
	D3DUSAGE_DONOTCLIP = (0x00000020L),
	D3DUSAGE_POINTS = (0x00000040L),
	D3DUSAGE_RTPATCHES = (0x00000080L),
	D3DUSAGE_NPATCHES = (0x00000100L),
	D3DLOCK_READONLY = 0x00000010L,
	D3DLOCK_DISCARD = 0x00002000L,
	D3DLOCK_NOOVERWRITE = 0x00001000L,
	D3DLOCK_NOSYSLOCK = 0x00000800L,
	D3DLOCK_DONOTWAIT = 0x00004000L ,
	D3DLOCK_NO_DIRTY_UPDATE = 0x00008000L,
	MAX_DEVICE_IDENTIFIER_STRING = 512,
	D3DISSUE_END = (1 << 0),
	D3DISSUE_BEGIN = (1 << 1),
	D3DGETDATA_FLUSH = (1 << 0),
	D3DRTYPE_INDEXBUFFER = 7,
	D3DRTYPECOUNT = (D3DRTYPE_INDEXBUFFER+1),
}



































alias uint D3DLIGHTTYPE; enum : uint
{
    D3DLIGHT_POINT = 1,
    D3DLIGHT_SPOT = 2,
    D3DLIGHT_DIRECTIONAL = 3,
    D3DLIGHT_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DSHADEMODE; enum : uint
{
    D3DSHADE_FLAT = 1,
    D3DSHADE_GOURAUD = 2,
    D3DSHADE_PHONG = 3,
    D3DSHADE_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DFILLMODE; enum : uint
{
    D3DFILL_POINT = 1,
    D3DFILL_WIREFRAME = 2,
    D3DFILL_SOLID = 3,
    D3DFILL_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DBLEND; enum : uint
{
    D3DBLEND_ZERO = 1,
    D3DBLEND_ONE = 2,
    D3DBLEND_SRCCOLOR = 3,
    D3DBLEND_INVSRCCOLOR = 4,
    D3DBLEND_SRCALPHA = 5,
    D3DBLEND_INVSRCALPHA = 6,
    D3DBLEND_DESTALPHA = 7,
    D3DBLEND_INVDESTALPHA = 8,
    D3DBLEND_DESTCOLOR = 9,
    D3DBLEND_INVDESTCOLOR = 10,
    D3DBLEND_SRCALPHASAT = 11,
    D3DBLEND_BOTHSRCALPHA = 12,
    D3DBLEND_BOTHINVSRCALPHA = 13,
    D3DBLEND_BLENDFACTOR        = 14,
    D3DBLEND_INVBLENDFACTOR     = 15,
    D3DBLEND_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DBLENDOP; enum : uint
{
    D3DBLENDOP_ADD = 1,
    D3DBLENDOP_SUBTRACT = 2,
    D3DBLENDOP_REVSUBTRACT = 3,
    D3DBLENDOP_MIN = 4,
    D3DBLENDOP_MAX = 5,
    D3DBLENDOP_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DTEXTUREADDRESS; enum : uint
{
    D3DTADDRESS_WRAP = 1,
    D3DTADDRESS_MIRROR = 2,
    D3DTADDRESS_CLAMP = 3,
    D3DTADDRESS_BORDER = 4,
    D3DTADDRESS_MIRRORONCE = 5,
    D3DTADDRESS_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DCULL; enum : uint
{
    D3DCULL_NONE = 1,
    D3DCULL_CW = 2,
    D3DCULL_CCW = 3,
    D3DCULL_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DCMPFUNC; enum : uint
{
    D3DCMP_NEVER = 1,
    D3DCMP_LESS = 2,
    D3DCMP_EQUAL = 3,
    D3DCMP_LESSEQUAL = 4,
    D3DCMP_GREATER = 5,
    D3DCMP_NOTEQUAL = 6,
    D3DCMP_GREATEREQUAL = 7,
    D3DCMP_ALWAYS = 8,
    D3DCMP_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DSTENCILOP; enum : uint
{
    D3DSTENCILOP_KEEP           = 1,
    D3DSTENCILOP_ZERO           = 2,
    D3DSTENCILOP_REPLACE        = 3,
    D3DSTENCILOP_INCRSAT        = 4,
    D3DSTENCILOP_DECRSAT        = 5,
    D3DSTENCILOP_INVERT         = 6,
    D3DSTENCILOP_INCR           = 7,
    D3DSTENCILOP_DECR           = 8,
    D3DSTENCILOP_FORCE_DWORD    = 0x7fffffff,
}
alias uint D3DFOGMODE; enum : uint
{
    D3DFOG_NONE                 = 0,
    D3DFOG_EXP                  = 1,
    D3DFOG_EXP2                 = 2,
    D3DFOG_LINEAR               = 3,
    D3DFOG_FORCE_DWORD          = 0x7fffffff,
}
alias uint D3DZBUFFERTYPE; enum : uint
{
    D3DZB_FALSE                 = 0,
    D3DZB_TRUE                  = 1,
    D3DZB_USEW                  = 2,
    D3DZB_FORCE_DWORD           = 0x7fffffff,
}
alias uint D3DPRIMITIVETYPE; enum : uint
{
    D3DPT_POINTLIST             = 1,
    D3DPT_LINELIST              = 2,
    D3DPT_LINESTRIP             = 3,
    D3DPT_TRIANGLELIST          = 4,
    D3DPT_TRIANGLESTRIP         = 5,
    D3DPT_TRIANGLEFAN           = 6,
    D3DPT_FORCE_DWORD           = 0x7fffffff,
}
alias uint D3DTRANSFORMSTATETYPE; enum : uint
{
    D3DTS_VIEW          = 2,
    D3DTS_PROJECTION    = 3,
    D3DTS_TEXTURE0      = 16,
    D3DTS_TEXTURE1      = 17,
    D3DTS_TEXTURE2      = 18,
    D3DTS_TEXTURE3      = 19,
    D3DTS_TEXTURE4      = 20,
    D3DTS_TEXTURE5      = 21,
    D3DTS_TEXTURE6      = 22,
    D3DTS_TEXTURE7      = 23,
    D3DTS_FORCE_DWORD     = 0x7fffffff,
    D3DTS_WORLD =  (0 + 256),
	D3DTS_WORLD1 = (1 + 256),
	D3DTS_WORLD2 = (2 + 256),
	D3DTS_WORLD3 = (3 + 256)
}
alias uint D3DRENDERSTATETYPE; enum : uint
{
    D3DRS_ZENABLE                   = 7,
    D3DRS_FILLMODE                  = 8,
    D3DRS_SHADEMODE                 = 9,
    D3DRS_ZWRITEENABLE              = 14,
    D3DRS_ALPHATESTENABLE           = 15,
    D3DRS_LASTPIXEL                 = 16,
    D3DRS_SRCBLEND                  = 19,
    D3DRS_DESTBLEND                 = 20,
    D3DRS_CULLMODE                  = 22,
    D3DRS_ZFUNC                     = 23,
    D3DRS_ALPHAREF                  = 24,
    D3DRS_ALPHAFUNC                 = 25,
    D3DRS_DITHERENABLE              = 26,
    D3DRS_ALPHABLENDENABLE          = 27,
    D3DRS_FOGENABLE                 = 28,
    D3DRS_SPECULARENABLE            = 29,
    D3DRS_FOGCOLOR                  = 34,
    D3DRS_FOGTABLEMODE              = 35,
    D3DRS_FOGSTART                  = 36,
    D3DRS_FOGEND                    = 37,
    D3DRS_FOGDENSITY                = 38,
    D3DRS_RANGEFOGENABLE            = 48,
    D3DRS_STENCILENABLE             = 52,
    D3DRS_STENCILFAIL               = 53,
    D3DRS_STENCILZFAIL              = 54,
    D3DRS_STENCILPASS               = 55,
    D3DRS_STENCILFUNC               = 56,
    D3DRS_STENCILREF                = 57,
    D3DRS_STENCILMASK               = 58,
    D3DRS_STENCILWRITEMASK          = 59,
    D3DRS_TEXTUREFACTOR             = 60,
    D3DRS_WRAP0                     = 128,
    D3DRS_WRAP1                     = 129,
    D3DRS_WRAP2                     = 130,
    D3DRS_WRAP3                     = 131,
    D3DRS_WRAP4                     = 132,
    D3DRS_WRAP5                     = 133,
    D3DRS_WRAP6                     = 134,
    D3DRS_WRAP7                     = 135,
    D3DRS_CLIPPING                  = 136,
    D3DRS_LIGHTING                  = 137,
    D3DRS_AMBIENT                   = 139,
    D3DRS_FOGVERTEXMODE             = 140,
    D3DRS_COLORVERTEX               = 141,
    D3DRS_LOCALVIEWER               = 142,
    D3DRS_NORMALIZENORMALS          = 143,
    D3DRS_DIFFUSEMATERIALSOURCE     = 145,
    D3DRS_SPECULARMATERIALSOURCE    = 146,
    D3DRS_AMBIENTMATERIALSOURCE     = 147,
    D3DRS_EMISSIVEMATERIALSOURCE    = 148,
    D3DRS_VERTEXBLEND               = 151,
    D3DRS_CLIPPLANEENABLE           = 152,
    D3DRS_POINTSIZE                 = 154,
    D3DRS_POINTSIZE_MIN             = 155,
    D3DRS_POINTSPRITEENABLE         = 156,
    D3DRS_POINTSCALEENABLE          = 157,
    D3DRS_POINTSCALE_A              = 158,
    D3DRS_POINTSCALE_B              = 159,
    D3DRS_POINTSCALE_C              = 160,
    D3DRS_MULTISAMPLEANTIALIAS      = 161,
    D3DRS_MULTISAMPLEMASK           = 162,
    D3DRS_PATCHEDGESTYLE            = 163,
    D3DRS_DEBUGMONITORTOKEN         = 165,
    D3DRS_POINTSIZE_MAX             = 166,
    D3DRS_INDEXEDVERTEXBLENDENABLE  = 167,
    D3DRS_COLORWRITEENABLE          = 168,
    D3DRS_TWEENFACTOR               = 170,
    D3DRS_BLENDOP                   = 171,
    D3DRS_POSITIONDEGREE            = 172,
    D3DRS_NORMALDEGREE              = 173,
    D3DRS_SCISSORTESTENABLE         = 174,
    D3DRS_SLOPESCALEDEPTHBIAS       = 175,
    D3DRS_ANTIALIASEDLINEENABLE     = 176,
    D3DRS_MINTESSELLATIONLEVEL      = 178,
    D3DRS_MAXTESSELLATIONLEVEL      = 179,
    D3DRS_ADAPTIVETESS_X            = 180,
    D3DRS_ADAPTIVETESS_Y            = 181,
    D3DRS_ADAPTIVETESS_Z            = 182,
    D3DRS_ADAPTIVETESS_W            = 183,
    D3DRS_ENABLEADAPTIVETESSELLATION = 184,
    D3DRS_TWOSIDEDSTENCILMODE       = 185,
    D3DRS_CCW_STENCILFAIL           = 186,
    D3DRS_CCW_STENCILZFAIL          = 187,
    D3DRS_CCW_STENCILPASS           = 188,
    D3DRS_CCW_STENCILFUNC           = 189,
    D3DRS_COLORWRITEENABLE1         = 190,
    D3DRS_COLORWRITEENABLE2         = 191,
    D3DRS_COLORWRITEENABLE3         = 192,
    D3DRS_BLENDFACTOR               = 193,
    D3DRS_SRGBWRITEENABLE           = 194,
    D3DRS_DEPTHBIAS                 = 195,
    D3DRS_WRAP8                     = 198,
    D3DRS_WRAP9                     = 199,
    D3DRS_WRAP10                    = 200,
    D3DRS_WRAP11                    = 201,
    D3DRS_WRAP12                    = 202,
    D3DRS_WRAP13                    = 203,
    D3DRS_WRAP14                    = 204,
    D3DRS_WRAP15                    = 205,
    D3DRS_SEPARATEALPHABLENDENABLE  = 206,
    D3DRS_SRCBLENDALPHA             = 207,
    D3DRS_DESTBLENDALPHA            = 208,
    D3DRS_BLENDOPALPHA              = 209,
    D3DRS_FORCE_DWORD               = 0x7fffffff,
}
alias uint D3DMATERIALCOLORSOURCE; enum : uint
{
    D3DMCS_MATERIAL         = 0,
    D3DMCS_COLOR1           = 1,
    D3DMCS_COLOR2           = 2,
    D3DMCS_FORCE_DWORD      = 0x7fffffff,
}
alias uint D3DTEXTURESTAGESTATETYPE; enum : uint
{
    D3DTSS_COLOROP        =  1,
    D3DTSS_COLORARG1      =  2,
    D3DTSS_COLORARG2      =  3,
    D3DTSS_ALPHAOP        =  4,
    D3DTSS_ALPHAARG1      =  5,
    D3DTSS_ALPHAARG2      =  6,
    D3DTSS_BUMPENVMAT00   =  7,
    D3DTSS_BUMPENVMAT01   =  8,
    D3DTSS_BUMPENVMAT10   =  9,
    D3DTSS_BUMPENVMAT11   = 10,
    D3DTSS_TEXCOORDINDEX  = 11,
    D3DTSS_BUMPENVLSCALE  = 22,
    D3DTSS_BUMPENVLOFFSET = 23,
    D3DTSS_TEXTURETRANSFORMFLAGS = 24,
    D3DTSS_COLORARG0      = 26,
    D3DTSS_ALPHAARG0      = 27,
    D3DTSS_RESULTARG      = 28,
    D3DTSS_CONSTANT       = 32,
    D3DTSS_FORCE_DWORD   = 0x7fffffff,
}
alias uint D3DSAMPLERSTATETYPE; enum : uint
{
    D3DSAMP_ADDRESSU       = 1,
    D3DSAMP_ADDRESSV       = 2,
    D3DSAMP_ADDRESSW       = 3,
    D3DSAMP_BORDERCOLOR    = 4,
    D3DSAMP_MAGFILTER      = 5,
    D3DSAMP_MINFILTER      = 6,
    D3DSAMP_MIPFILTER      = 7,
    D3DSAMP_MIPMAPLODBIAS  = 8,
    D3DSAMP_MAXMIPLEVEL    = 9,
    D3DSAMP_MAXANISOTROPY  = 10,
    D3DSAMP_SRGBTEXTURE    = 11,
    D3DSAMP_ELEMENTINDEX   = 12,
    D3DSAMP_DMAPOFFSET     = 13,
    D3DSAMP_FORCE_DWORD   = 0x7fffffff,
}
alias uint D3DTEXTUREOP; enum  : uint
{
    D3DTOP_DISABLE              = 1,
    D3DTOP_SELECTARG1           = 2,
    D3DTOP_SELECTARG2           = 3,
    D3DTOP_MODULATE             = 4,
    D3DTOP_MODULATE2X           = 5,
    D3DTOP_MODULATE4X           = 6,
    D3DTOP_ADD                  =  7,
    D3DTOP_ADDSIGNED            =  8,
    D3DTOP_ADDSIGNED2X          =  9,
    D3DTOP_SUBTRACT             = 10,
    D3DTOP_ADDSMOOTH            = 11,
    D3DTOP_BLENDDIFFUSEALPHA    = 12,
    D3DTOP_BLENDTEXTUREALPHA    = 13,
    D3DTOP_BLENDFACTORALPHA     = 14,
    D3DTOP_BLENDTEXTUREALPHAPM  = 15,
    D3DTOP_BLENDCURRENTALPHA    = 16,
    D3DTOP_PREMODULATE            = 17,
    D3DTOP_MODULATEALPHA_ADDCOLOR = 18,
    D3DTOP_MODULATECOLOR_ADDALPHA = 19,
    D3DTOP_MODULATEINVALPHA_ADDCOLOR = 20,
    D3DTOP_MODULATEINVCOLOR_ADDALPHA = 21,
    D3DTOP_BUMPENVMAP           = 22,
    D3DTOP_BUMPENVMAPLUMINANCE  = 23,
    D3DTOP_DOTPRODUCT3          = 24,
    D3DTOP_MULTIPLYADD          = 25,
    D3DTOP_LERP                 = 26,
    D3DTOP_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DTEXTUREFILTERTYPE; enum : uint
{
    D3DTEXF_NONE            = 0,
    D3DTEXF_POINT           = 1,
    D3DTEXF_LINEAR          = 2,
    D3DTEXF_ANISOTROPIC     = 3,
    D3DTEXF_PYRAMIDALQUAD   = 6,
    D3DTEXF_GAUSSIANQUAD    = 7,
    D3DTEXF_FORCE_DWORD     = 0x7fffffff,
}
alias uint D3DDECLUSAGE; enum : uint
{
    D3DDECLUSAGE_POSITION = 0,
    D3DDECLUSAGE_BLENDWEIGHT,
    D3DDECLUSAGE_BLENDINDICES,
    D3DDECLUSAGE_NORMAL,
    D3DDECLUSAGE_PSIZE,
    D3DDECLUSAGE_TEXCOORD,
    D3DDECLUSAGE_TANGENT,
    D3DDECLUSAGE_BINORMAL,
    D3DDECLUSAGE_TESSFACTOR,
    D3DDECLUSAGE_POSITIONT,
    D3DDECLUSAGE_COLOR,
    D3DDECLUSAGE_FOG,
    D3DDECLUSAGE_DEPTH,
    D3DDECLUSAGE_SAMPLE,
}
alias D3DDECLUSAGE_SAMPLE MAXD3DDECLUSAGE;
alias uint D3DDECLMETHOD; enum : uint
{
    D3DDECLMETHOD_DEFAULT = 0,
    D3DDECLMETHOD_PARTIALU,
    D3DDECLMETHOD_PARTIALV,
    D3DDECLMETHOD_CROSSUV,
    D3DDECLMETHOD_UV,
    D3DDECLMETHOD_LOOKUP,
    D3DDECLMETHOD_LOOKUPPRESAMPLED,
}
alias D3DDECLMETHOD_LOOKUPPRESAMPLED MAXD3DDECLMETHOD;
alias uint D3DDECLTYPE; enum : uint
{
    D3DDECLTYPE_FLOAT1    =  0,
    D3DDECLTYPE_FLOAT2    =  1,
    D3DDECLTYPE_FLOAT3    =  2,
    D3DDECLTYPE_FLOAT4    =  3,
    D3DDECLTYPE_D3DCOLOR  =  4,
    D3DDECLTYPE_UBYTE4    =  5,
    D3DDECLTYPE_SHORT2    =  6,
    D3DDECLTYPE_SHORT4    =  7,
    D3DDECLTYPE_UBYTE4N   =  8,
    D3DDECLTYPE_SHORT2N   =  9,
    D3DDECLTYPE_SHORT4N   = 10,
    D3DDECLTYPE_USHORT2N  = 11,
    D3DDECLTYPE_USHORT4N  = 12,
    D3DDECLTYPE_UDEC3     = 13,
    D3DDECLTYPE_DEC3N     = 14,
    D3DDECLTYPE_FLOAT16_2 = 15,
    D3DDECLTYPE_FLOAT16_4 = 16,
    D3DDECLTYPE_UNUSED    = 17,
}
alias D3DDECLTYPE_UNUSED MAXD3DDECLTYPE;
alias uint D3DSHADER_INSTRUCTION_OPCODE_TYPE; enum : uint
{
    D3DSIO_NOP          = 0,
    D3DSIO_MOV          ,
    D3DSIO_ADD          ,
    D3DSIO_SUB          ,
    D3DSIO_MAD          ,
    D3DSIO_MUL          ,
    D3DSIO_RCP          ,
    D3DSIO_RSQ          ,
    D3DSIO_DP3          ,
    D3DSIO_DP4          ,
    D3DSIO_MIN          ,
    D3DSIO_MAX          ,
    D3DSIO_SLT          ,
    D3DSIO_SGE          ,
    D3DSIO_EXP          ,
    D3DSIO_LOG          ,
    D3DSIO_LIT          ,
    D3DSIO_DST          ,
    D3DSIO_LRP          ,
    D3DSIO_FRC          ,
    D3DSIO_M4x4         ,
    D3DSIO_M4x3         ,
    D3DSIO_M3x4         ,
    D3DSIO_M3x3         ,
    D3DSIO_M3x2         ,
    D3DSIO_CALL         ,
    D3DSIO_CALLNZ       ,
    D3DSIO_LOOP         ,
    D3DSIO_RET          ,
    D3DSIO_ENDLOOP      ,
    D3DSIO_LABEL        ,
    D3DSIO_DCL          ,
    D3DSIO_POW          ,
    D3DSIO_CRS          ,
    D3DSIO_SGN          ,
    D3DSIO_ABS          ,
    D3DSIO_NRM          ,
    D3DSIO_SINCOS       ,
    D3DSIO_REP          ,
    D3DSIO_ENDREP       ,
    D3DSIO_IF           ,
    D3DSIO_IFC          ,
    D3DSIO_ELSE         ,
    D3DSIO_ENDIF        ,
    D3DSIO_BREAK        ,
    D3DSIO_BREAKC       ,
    D3DSIO_MOVA         ,
    D3DSIO_DEFB         ,
    D3DSIO_DEFI         ,
    D3DSIO_TEXCOORD     = 64,
    D3DSIO_TEXKILL      ,
    D3DSIO_TEX          ,
    D3DSIO_TEXBEM       ,
    D3DSIO_TEXBEML      ,
    D3DSIO_TEXREG2AR    ,
    D3DSIO_TEXREG2GB    ,
    D3DSIO_TEXM3x2PAD   ,
    D3DSIO_TEXM3x2TEX   ,
    D3DSIO_TEXM3x3PAD   ,
    D3DSIO_TEXM3x3TEX   ,
    D3DSIO_RESERVED0    ,
    D3DSIO_TEXM3x3SPEC  ,
    D3DSIO_TEXM3x3VSPEC ,
    D3DSIO_EXPP         ,
    D3DSIO_LOGP         ,
    D3DSIO_CND          ,
    D3DSIO_DEF          ,
    D3DSIO_TEXREG2RGB   ,
    D3DSIO_TEXDP3TEX    ,
    D3DSIO_TEXM3x2DEPTH ,
    D3DSIO_TEXDP3       ,
    D3DSIO_TEXM3x3      ,
    D3DSIO_TEXDEPTH     ,
    D3DSIO_CMP          ,
    D3DSIO_BEM          ,
    D3DSIO_DP2ADD       ,
    D3DSIO_DSX          ,
    D3DSIO_DSY          ,
    D3DSIO_TEXLDD       ,
    D3DSIO_SETP         ,
    D3DSIO_TEXLDL       ,
    D3DSIO_BREAKP       ,
    D3DSIO_PHASE        = 0xFFFD,
    D3DSIO_COMMENT      = 0xFFFE,
    D3DSIO_END          = 0xFFFF,
    D3DSIO_FORCE_DWORD  = 0x7fffffff,
}
alias uint D3DSHADER_COMPARISON; enum : uint
{
    D3DSPC_RESERVED0= 0,
    D3DSPC_GT       = 1,
    D3DSPC_EQ       = 2,
    D3DSPC_GE       = 3,
    D3DSPC_LT       = 4,
    D3DSPC_NE       = 5,
    D3DSPC_LE       = 6,
    D3DSPC_RESERVED1= 7
}
alias uint D3DSAMPLER_TEXTURE_TYPE; enum : uint
{
    D3DSTT_UNKNOWN = 0,
    D3DSTT_2D      = 2<<D3DSP_TEXTURETYPE_SHIFT,
    D3DSTT_CUBE    = 3<<D3DSP_TEXTURETYPE_SHIFT,
    D3DSTT_VOLUME  = 4<<D3DSP_TEXTURETYPE_SHIFT,
    D3DSTT_FORCE_DWORD  = 0x7fffffff,
}
alias uint D3DSHADER_PARAM_REGISTER_TYPE; enum : uint
{
    D3DSPR_TEMP           =  0,
    D3DSPR_INPUT          =  1,
    D3DSPR_CONST          =  2,
    D3DSPR_ADDR           =  3,
    D3DSPR_TEXTURE        =  3,
    D3DSPR_RASTOUT        =  4,
    D3DSPR_ATTROUT        =  5,
    D3DSPR_TEXCRDOUT      =  6,
    D3DSPR_OUTPUT         =  6,
    D3DSPR_CONSTINT       =  7,
    D3DSPR_COLOROUT       =  8,
    D3DSPR_DEPTHOUT       =  9,
    D3DSPR_SAMPLER        = 10,
    D3DSPR_CONST2         = 11,
    D3DSPR_CONST3         = 12,
    D3DSPR_CONST4         = 13,
    D3DSPR_CONSTBOOL      = 14,
    D3DSPR_LOOP           = 15,
    D3DSPR_TEMPFLOAT16    = 16,
    D3DSPR_MISCTYPE       = 17,
    D3DSPR_LABEL          = 18,
    D3DSPR_PREDICATE      = 19,
    D3DSPR_FORCE_DWORD  = 0x7fffffff,
}
alias uint D3DSHADER_MISCTYPE_OFFSETS; enum : uint
{
    D3DSMO_POSITION   = 0,
    D3DSMO_FACE   = 1,
}
alias uint D3DVS_RASTOUT_OFFSETS; enum : uint
{
    D3DSRO_POSITION = 0,
    D3DSRO_FOG,
    D3DSRO_POINT_SIZE,
    D3DSRO_FORCE_DWORD  = 0x7fffffff,
}
alias uint D3DVS_ADDRESSMODE_TYPE; enum : uint
{
    D3DVS_ADDRMODE_ABSOLUTE  = (0 << D3DVS_ADDRESSMODE_SHIFT),
    D3DVS_ADDRMODE_RELATIVE  = (1 << D3DVS_ADDRESSMODE_SHIFT),
    D3DVS_ADDRMODE_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DSHADER_ADDRESSMODE_TYPE; enum : uint
{
    D3DSHADER_ADDRMODE_ABSOLUTE  = (0 << D3DSHADER_ADDRESSMODE_SHIFT),
    D3DSHADER_ADDRMODE_RELATIVE  = (1 << D3DSHADER_ADDRESSMODE_SHIFT),
    D3DSHADER_ADDRMODE_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DSHADER_PARAM_SRCMOD_TYPE; enum : uint
{
    D3DSPSM_NONE    = 0<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_NEG     = 1<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_BIAS    = 2<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_BIASNEG = 3<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_SIGN    = 4<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_SIGNNEG = 5<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_COMP    = 6<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_X2      = 7<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_X2NEG   = 8<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_DZ      = 9<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_DW      = 10<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_ABS     = 11<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_ABSNEG  = 12<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_NOT     = 13<<D3DSP_SRCMOD_SHIFT,
    D3DSPSM_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DBASISTYPE; enum : uint
{
   D3DBASIS_BEZIER      = 0,
   D3DBASIS_BSPLINE     = 1,
   D3DBASIS_CATMULL_ROM = 2,
   D3DBASIS_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DDEGREETYPE; enum : uint
{
   D3DDEGREE_LINEAR      = 1,
   D3DDEGREE_QUADRATIC   = 2,
   D3DDEGREE_CUBIC       = 3,
   D3DDEGREE_QUINTIC     = 5,
   D3DDEGREE_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DPATCHEDGESTYLE; enum : uint
{
   D3DPATCHEDGE_DISCRETE    = 0,
   D3DPATCHEDGE_CONTINUOUS  = 1,
   D3DPATCHEDGE_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DSTATEBLOCKTYPE; enum : uint
{
    D3DSBT_ALL           = 1,
    D3DSBT_PIXELSTATE    = 2,
    D3DSBT_VERTEXSTATE   = 3,
    D3DSBT_FORCE_DWORD   = 0x7fffffff,
}
alias uint D3DVERTEXBLENDFLAGS; enum : uint
{
    D3DVBF_DISABLE  = 0,
    D3DVBF_1WEIGHTS = 1,
    D3DVBF_2WEIGHTS = 2,
    D3DVBF_3WEIGHTS = 3,
    D3DVBF_TWEENING = 255,
    D3DVBF_0WEIGHTS = 256,
    D3DVBF_FORCE_DWORD = 0x7fffffff,
}
alias uint D3DTEXTURETRANSFORMFLAGS; enum : uint
{
    D3DTTFF_DISABLE         = 0,
    D3DTTFF_COUNT1          = 1,
    D3DTTFF_COUNT2          = 2,
    D3DTTFF_COUNT3          = 3,
    D3DTTFF_COUNT4          = 4,
    D3DTTFF_PROJECTED       = 256,
    D3DTTFF_FORCE_DWORD     = 0x7fffffff,
}
alias uint D3DDEVTYPE; enum : uint
{
    D3DDEVTYPE_HAL         = 1,
    D3DDEVTYPE_REF         = 2,
    D3DDEVTYPE_SW          = 3,
    D3DDEVTYPE_FORCE_DWORD  = 0x7fffffff
}
alias uint D3DMULTISAMPLE_TYPE; enum : uint
{
    D3DMULTISAMPLE_NONE            =  0,
    D3DMULTISAMPLE_NONMASKABLE     =  1,
    D3DMULTISAMPLE_2_SAMPLES       =  2,
    D3DMULTISAMPLE_3_SAMPLES       =  3,
    D3DMULTISAMPLE_4_SAMPLES       =  4,
    D3DMULTISAMPLE_5_SAMPLES       =  5,
    D3DMULTISAMPLE_6_SAMPLES       =  6,
    D3DMULTISAMPLE_7_SAMPLES       =  7,
    D3DMULTISAMPLE_8_SAMPLES       =  8,
    D3DMULTISAMPLE_9_SAMPLES       =  9,
    D3DMULTISAMPLE_10_SAMPLES      = 10,
    D3DMULTISAMPLE_11_SAMPLES      = 11,
    D3DMULTISAMPLE_12_SAMPLES      = 12,
    D3DMULTISAMPLE_13_SAMPLES      = 13,
    D3DMULTISAMPLE_14_SAMPLES      = 14,
    D3DMULTISAMPLE_15_SAMPLES      = 15,
    D3DMULTISAMPLE_16_SAMPLES      = 16,
    D3DMULTISAMPLE_FORCE_DWORD     = 0x7fffffff
}
alias uint D3DFORMAT; enum : uint
{
    D3DFMT_UNKNOWN              =  0,
    D3DFMT_R8G8B8               = 20,
    D3DFMT_A8R8G8B8             = 21,
    D3DFMT_X8R8G8B8             = 22,
    D3DFMT_R5G6B5               = 23,
    D3DFMT_X1R5G5B5             = 24,
    D3DFMT_A1R5G5B5             = 25,
    D3DFMT_A4R4G4B4             = 26,
    D3DFMT_R3G3B2               = 27,
    D3DFMT_A8                   = 28,
    D3DFMT_A8R3G3B2             = 29,
    D3DFMT_X4R4G4B4             = 30,
    D3DFMT_A2B10G10R10          = 31,
    D3DFMT_A8B8G8R8             = 32,
    D3DFMT_X8B8G8R8             = 33,
    D3DFMT_G16R16               = 34,
    D3DFMT_A2R10G10B10          = 35,
    D3DFMT_A16B16G16R16         = 36,
    D3DFMT_A8P8                 = 40,
    D3DFMT_P8                   = 41,
    D3DFMT_L8                   = 50,
    D3DFMT_A8L8                 = 51,
    D3DFMT_A4L4                 = 52,
    D3DFMT_V8U8                 = 60,
    D3DFMT_L6V5U5               = 61,
    D3DFMT_X8L8V8U8             = 62,
    D3DFMT_Q8W8V8U8             = 63,
    D3DFMT_V16U16               = 64,
    D3DFMT_A2W10V10U10          = 67,
    D3DFMT_UYVY = 0x59565955,
    D3DFMT_YUY2 = 0x32595559,
    D3DFMT_DXT1 = 0x31545844,
    D3DFMT_DXT2 = 0x32545844,
    D3DFMT_DXT3 = 0x33545844,
    D3DFMT_DXT4 = 0x34545844,
    D3DFMT_DXT5 = 0x35545844,
    D3DFMT_R8G8_B8G8            = 0x47424752,
    D3DFMT_G8R8_G8B8            = 0x42475247,
    D3DFMT_D16_LOCKABLE         = 70,
    D3DFMT_D32                  = 71,
    D3DFMT_D15S1                = 73,
    D3DFMT_D24S8                = 75,
    D3DFMT_D24X8                = 77,
    D3DFMT_D24X4S4              = 79,
    D3DFMT_D16                  = 80,
    D3DFMT_D32F_LOCKABLE        = 82,
    D3DFMT_D24FS8               = 83,
    D3DFMT_L16                  = 81,
    D3DFMT_VERTEXDATA           =100,
    D3DFMT_INDEX16              =101,
    D3DFMT_INDEX32              =102,
    D3DFMT_Q16W16V16U16         =110,
    D3DFMT_MULTI2_ARGB8         = 0x3154454D,
    D3DFMT_R16F                 = 111,
    D3DFMT_G16R16F              = 112,
    D3DFMT_A16B16G16R16F        = 113,
    D3DFMT_R32F                 = 114,
    D3DFMT_G32R32F              = 115,
    D3DFMT_A32B32G32R32F        = 116,
    D3DFMT_CxV8U8               = 117,
    D3DFMT_FORCE_DWORD          =0x7fffffff
}
alias uint D3DSWAPEFFECT; enum : uint
{
    D3DSWAPEFFECT_DISCARD           = 1,
    D3DSWAPEFFECT_FLIP              = 2,
    D3DSWAPEFFECT_COPY              = 3,
    D3DSWAPEFFECT_FORCE_DWORD       = 0x7fffffff
}
alias uint D3DPOOL; enum : uint
{
    D3DPOOL_DEFAULT                 = 0,
    D3DPOOL_MANAGED                 = 1,
    D3DPOOL_SYSTEMMEM               = 2,
    D3DPOOL_SCRATCH                 = 3,
    D3DPOOL_FORCE_DWORD             = 0x7fffffff
}
alias uint D3DBACKBUFFER_TYPE; enum : uint
{
    D3DBACKBUFFER_TYPE_MONO         = 0,
    D3DBACKBUFFER_TYPE_LEFT         = 1,
    D3DBACKBUFFER_TYPE_RIGHT        = 2,
    D3DBACKBUFFER_TYPE_FORCE_DWORD  = 0x7fffffff
}
alias uint D3DRESOURCETYPE; enum : uint
{
    D3DRTYPE_SURFACE                =  1,
    D3DRTYPE_VOLUME                 =  2,
    D3DRTYPE_TEXTURE                =  3,
    D3DRTYPE_VOLUMETEXTURE          =  4,
    D3DRTYPE_CUBETEXTURE            =  5,
    D3DRTYPE_VERTEXBUFFER           =  6,
    D3DRTYPE_INDEXBUFFER            =  7,
    D3DRTYPE_FORCE_DWORD            = 0x7fffffff
}
alias uint D3DCUBEMAP_FACES; enum : uint
{
    D3DCUBEMAP_FACE_POSITIVE_X     = 0,
    D3DCUBEMAP_FACE_NEGATIVE_X     = 1,
    D3DCUBEMAP_FACE_POSITIVE_Y     = 2,
    D3DCUBEMAP_FACE_NEGATIVE_Y     = 3,
    D3DCUBEMAP_FACE_POSITIVE_Z     = 4,
    D3DCUBEMAP_FACE_NEGATIVE_Z     = 5,
    D3DCUBEMAP_FACE_FORCE_DWORD    = 0x7fffffff
}
alias uint D3DDEBUGMONITORTOKENS; enum : uint
{
    D3DDMT_ENABLE            = 0,
    D3DDMT_DISABLE           = 1,
    D3DDMT_FORCE_DWORD     = 0x7fffffff,
}
alias uint D3DQUERYTYPE; enum : uint
{
    D3DQUERYTYPE_VCACHE                 = 4,
    D3DQUERYTYPE_RESOURCEMANAGER        = 5,
    D3DQUERYTYPE_VERTEXSTATS            = 6,
    D3DQUERYTYPE_EVENT                  = 8,
    D3DQUERYTYPE_OCCLUSION              = 9,
}
alias uint D3DXERR; enum : uint
{
    D3DXERR_CANNOTMODIFYINDEXBUFFER     = (1 << 31) | (_FACDD << 16) | 2900,
    D3DXERR_INVALIDMESH                 = (1 << 31) | (_FACDD << 16) | 2901,
    D3DXERR_CANNOTATTRSORT              = (1 << 31) | (_FACDD << 16) | 2902,
    D3DXERR_SKINNINGNOTSUPPORTED        = (1 << 31) | (_FACDD << 16) | 2903,
    D3DXERR_TOOMANYINFLUENCES           = (1 << 31) | (_FACDD << 16) | 2904,
    D3DXERR_INVALIDDATA                 = (1 << 31) | (_FACDD << 16) | 2905,
    D3DXERR_LOADEDMESHASNODATA          = (1 << 31) | (_FACDD << 16) | 2906,
    D3DXERR_DUPLICATENAMEDFRAGMENT      = (1 << 31) | (_FACDD << 16) | 2907
}
alias uint D3DXMESHDATATYPE; enum : uint
{
    D3DXMESHTYPE_MESH      = 0x001,
    D3DXMESHTYPE_PMESH     = 0x002,
    D3DXMESHTYPE_PATCHMESH = 0x003,
    D3DXMESHTYPE_FORCE_DWORD    = 0x7fffffff,
}
alias uint D3DXTRACKFLAG; enum : uint
{
    D3DXTF_LOWPRIORITY            = 0x000,
    D3DXTF_HIGHPRIORITY           = 0x001,
    D3DXTF_FORCE_DWORD    = 0x7fffffff,
}
alias uint D3DXTRANSITIONTYPE; enum : uint
{
    D3DXTRANSITION_LINEAR            = 0x000,
    D3DXTRANSITION_EASEINEASEOUT     = 0x001,
    D3DXTRANSITION_FORCE_DWORD    = 0x7fffffff,
}
alias uint D3DXPATCHMESHTYPE; enum : uint
{
    D3DXPATCHMESH_RECT   = 0x001,
    D3DXPATCHMESH_TRI    = 0x002,
    D3DXPATCHMESH_NPATCH = 0x003,
    D3DXPATCHMESH_FORCE_DWORD    = 0x7fffffff,
}
alias uint D3DXMESH; enum : uint
{
    D3DXMESH_32BIT                  = 0x001,
    D3DXMESH_DONOTCLIP              = 0x002,
    D3DXMESH_POINTS                 = 0x004,
    D3DXMESH_RTPATCHES              = 0x008,
    D3DXMESH_NPATCHES               = 0x4000,
    D3DXMESH_VB_SYSTEMMEM           = 0x010,
    D3DXMESH_VB_MANAGED             = 0x020,
    D3DXMESH_VB_WRITEONLY           = 0x040,
    D3DXMESH_VB_DYNAMIC             = 0x080,
    D3DXMESH_VB_SOFTWAREPROCESSING = 0x8000,
    D3DXMESH_IB_SYSTEMMEM           = 0x100,
    D3DXMESH_IB_MANAGED             = 0x200,
    D3DXMESH_IB_WRITEONLY           = 0x400,
    D3DXMESH_IB_DYNAMIC             = 0x800,
    D3DXMESH_IB_SOFTWAREPROCESSING= 0x10000,
    D3DXMESH_VB_SHARE               = 0x1000,
    D3DXMESH_USEHWONLY              = 0x2000,
    D3DXMESH_SYSTEMMEM              = 0x110,
    D3DXMESH_MANAGED                = 0x220,
    D3DXMESH_WRITEONLY              = 0x440,
    D3DXMESH_DYNAMIC                = 0x880,
    D3DXMESH_SOFTWAREPROCESSING   = 0x18000,
}
alias uint D3DXPATCHMESH; enum : uint
{
    D3DXPATCHMESH_DEFAULT = 000,
}
alias uint D3DXMESHSIMP; enum : uint
{
    D3DXMESHSIMP_VERTEX   = 0x1,
    D3DXMESHSIMP_FACE     = 0x2,
}
alias uint MAX_FVF_DECL_SIZE; enum : uint
{
    MAX_FVF_DECL_SIZE = MAXD3DDECLLENGTH + 1
}
alias uint D3DXEFFECTDEFAULTTYPE; enum : uint
{
    D3DXEDT_STRING = 0x1,
    D3DXEDT_FLOATS = 0x2,
    D3DXEDT_DWORD  = 0x3,
    D3DXEDT_FORCEDWORD = 0x7fffffff
}
alias uint D3DXWELDEPSILONSFLAGS; enum : uint
{
    D3DXWELDEPSILONS_WELDALL             = 0x1,
    D3DXWELDEPSILONS_WELDPARTIALMATCHES  = 0x2,
    D3DXWELDEPSILONS_DONOTREMOVEVERTICES = 0x4,
    D3DXWELDEPSILONS_DONOTSPLIT          = 0x8,
}
alias uint D3DXMESHOPT; enum : uint
{
    D3DXMESHOPT_COMPACT       = 0x01000000,
    D3DXMESHOPT_ATTRSORT      = 0x02000000,
    D3DXMESHOPT_VERTEXCACHE   = 0x04000000,
    D3DXMESHOPT_STRIPREORDER  = 0x08000000,
    D3DXMESHOPT_IGNOREVERTS   = 0x10000000,
    D3DXMESHOPT_DONOTSPLIT    = 0x20000000,
    D3DXMESHOPT_DEVICEINDEPENDENT = 0x00400000,
}
alias uint D3DXREGISTER_SET; enum : uint
{
    D3DXRS_BOOL,
    D3DXRS_INT4,
    D3DXRS_FLOAT4,
    D3DXRS_SAMPLER,
    D3DXRS_FORCE_DWORD = 0x7fffffff
}
alias uint D3DXPARAMETER_CLASS; enum : uint
{
    D3DXPC_SCALAR,
    D3DXPC_VECTOR,
    D3DXPC_MATRIX_ROWS,
    D3DXPC_MATRIX_COLUMNS,
    D3DXPC_OBJECT,
    D3DXPC_STRUCT,
    D3DXPC_FORCE_DWORD = 0x7fffffff
}
alias uint D3DXPARAMETER_TYPE; enum : uint
{
    D3DXPT_VOID,
    D3DXPT_BOOL,
    D3DXPT_INT,
    D3DXPT_FLOAT,
    D3DXPT_STRING,
    D3DXPT_TEXTURE,
    D3DXPT_TEXTURE1D,
    D3DXPT_TEXTURE2D,
    D3DXPT_TEXTURE3D,
    D3DXPT_TEXTURECUBE,
    D3DXPT_SAMPLER,
    D3DXPT_SAMPLER1D,
    D3DXPT_SAMPLER2D,
    D3DXPT_SAMPLER3D,
    D3DXPT_SAMPLERCUBE,
    D3DXPT_PIXELSHADER,
    D3DXPT_VERTEXSHADER,
    D3DXPT_PIXELFRAGMENT,
    D3DXPT_VERTEXFRAGMENT,
    D3DXPT_FORCE_DWORD = 0x7fffffff
}
alias uint D3DXINCLUDE_TYPE; enum : uint
{
    D3DXINC_LOCAL,
    D3DXINC_SYSTEM,
    D3DXINC_FORCE_DWORD = 0x7fffffff
}
alias uint D3DXIMAGE_FILEFORMAT; enum : uint
{
    D3DXIFF_BMP         = 0,
    D3DXIFF_JPG         = 1,
    D3DXIFF_TGA         = 2,
    D3DXIFF_PNG         = 3,
    D3DXIFF_DDS         = 4,
    D3DXIFF_PPM         = 5,
    D3DXIFF_DIB         = 6,
    D3DXIFF_FORCE_DWORD = 0x7fffffff
}


































// すべてのstructをここに

alias DWORD D3DCOLOR;
alias LPCSTR D3DXHANDLE;
alias long LARGE_INTEGER; // signed 64 bits
alias char TCHAR;


struct D3DXSHADER_CONSTANTTABLE
{
    DWORD Size;
    DWORD Creator;
    DWORD Version;
    DWORD Constants;
    DWORD ConstantInfo;
}

// d3d9caps.d
align(4) struct D3DVSHADERCAPS2_0
{
        DWORD Caps;
        INT DynamicFlowControlDepth;
        INT NumTemps;
        INT StaticFlowControlDepth;
}

align(4) struct D3DPSHADERCAPS2_0
{
    DWORD Caps;
    INT DynamicFlowControlDepth;
    INT NumTemps;
    INT StaticFlowControlDepth;
    INT NumInstructionSlots;
}

align(4) struct D3DCAPS9
{
    D3DDEVTYPE  DeviceType;
    UINT        AdapterOrdinal;
    DWORD   Caps;
    DWORD   Caps2;
    DWORD   Caps3;
    DWORD   PresentationIntervals;
    DWORD   CursorCaps;
    DWORD   DevCaps;
    DWORD   PrimitiveMiscCaps;
    DWORD   RasterCaps;
    DWORD   ZCmpCaps;
    DWORD   SrcBlendCaps;
    DWORD   DestBlendCaps;
    DWORD   AlphaCmpCaps;
    DWORD   ShadeCaps;
    DWORD   TextureCaps;
    DWORD   TextureFilterCaps;          // D3DPTFILTERCAPS for IDirect3DTexture9's
    DWORD   CubeTextureFilterCaps;      // D3DPTFILTERCAPS for IDirect3DCubeTexture9's
    DWORD   VolumeTextureFilterCaps;    // D3DPTFILTERCAPS for IDirect3DVolumeTexture9's
    DWORD   TextureAddressCaps;         // D3DPTADDRESSCAPS for IDirect3DTexture9's
    DWORD   VolumeTextureAddressCaps;   // D3DPTADDRESSCAPS for IDirect3DVolumeTexture9's
    DWORD   LineCaps;                   // D3DLINECAPS
    DWORD   MaxTextureWidth, MaxTextureHeight;
    DWORD   MaxVolumeExtent;
    DWORD   MaxTextureRepeat;
    DWORD   MaxTextureAspectRatio;
    DWORD   MaxAnisotropy;
    float   MaxVertexW;
    float   GuardBandLeft;
    float   GuardBandTop;
    float   GuardBandRight;
    float   GuardBandBottom;
    float   ExtentsAdjust;
    DWORD   StencilCaps;
    DWORD   FVFCaps;
    DWORD   TextureOpCaps;
    DWORD   MaxTextureBlendStages;
    DWORD   MaxSimultaneousTextures;
    DWORD   VertexProcessingCaps;
    DWORD   MaxActiveLights;
    DWORD   MaxUserClipPlanes;
    DWORD   MaxVertexBlendMatrices;
    DWORD   MaxVertexBlendMatrixIndex;
    float   MaxPointSize;
    DWORD   MaxPrimitiveCount;          // max number of primitives per DrawPrimitive call
    DWORD   MaxVertexIndex;
    DWORD   MaxStreams;
    DWORD   MaxStreamStride;            // max stride for SetStreamSource
    DWORD   VertexShaderVersion;
    DWORD   MaxVertexShaderConst;       // number of vertex shader constant registers
    DWORD   PixelShaderVersion;
    float   PixelShader1xMaxValue;      // max value storable in registers of ps.1.x shaders
    DWORD   DevCaps2;
    float   MaxNpatchTessellationLevel;
    DWORD   Reserved5;
    UINT    MasterAdapterOrdinal;       // ordinal of master adaptor for adapter group
    UINT    AdapterOrdinalInGroup;      // ordinal inside the adapter group
    UINT    NumberOfAdaptersInGroup;    // number of adapters in this adapter group (only if master)
    DWORD   DeclTypes;                  // Data types, supported in vertex declarations
    DWORD   NumSimultaneousRTs;         // Will be at least 1
    DWORD   StretchRectFilterCaps;      // Filter caps supported by StretchRect
    D3DVSHADERCAPS2_0 VS20Caps;
    D3DPSHADERCAPS2_0 PS20Caps;
    DWORD   VertexTextureFilterCaps;    // D3DPTFILTERCAPS for IDirect3DTexture9's for texture, used in vertex shaders
    DWORD   MaxVShaderInstructionsExecuted; // maximum number of vertex shader instructions that can be executed
    DWORD   MaxPShaderInstructionsExecuted; // maximum number of pixel shader instructions that can be executed
    DWORD   MaxVertexShader30InstructionSlots; 
    DWORD   MaxPixelShader30InstructionSlots;
}

struct D3DXEFFECT_DESC
{
    LPCSTR Creator;                     // Creator string
    UINT Parameters;                    // Number of parameters
    UINT Techniques;                    // Number of techniques
    UINT Functions;                     // Number of function entrypoints
}

struct D3DXPARAMETER_DESC
{
    LPCSTR Name;                        // Parameter name
    LPCSTR Semantic;                    // Parameter semantic
    D3DXPARAMETER_CLASS Class;          // Class
    D3DXPARAMETER_TYPE Type;            // Component type
    UINT Rows;                          // Number of rows
    UINT Columns;                       // Number of columns
    UINT Elements;                      // Number of array elements
    UINT Annotations;                   // Number of annotations
    UINT StructMembers;                 // Number of structure member sub-parameters
    DWORD Flags;                        // D3DX_PARAMETER_* flags
    UINT Bytes;                         // Parameter size, in bytes
}

struct D3DXTECHNIQUE_DESC
{
    LPCSTR Name;                        // Technique name
    UINT Passes;                        // Number of passes
    UINT Annotations;                   // Number of annotations
}

struct D3DXFUNCTION_DESC
{
    LPCSTR Name;                        // Function name
    UINT Annotations;                   // Number of annotations
}

// dxfile.d
struct DXFILELOADRESOURCE
{
    HMODULE hModule;
    LPCTSTR lpName;
    LPCTSTR lpType;
}

struct DXFILELOADMEMORY
{
    LPVOID lpMemory;
    DWORD dSize;
}

// d3dx9tex.d
struct D3DXIMAGE_INFO
{
    UINT                    Width;
    UINT                    Height;
    UINT                    Depth;
    UINT                    MipLevels;
    D3DFORMAT               Format;
    D3DRESOURCETYPE         ResourceType;
    D3DXIMAGE_FILEFORMAT    ImageFileFormat;
}

// d3dx9shader.d
struct D3DXMACRO
{
    LPCSTR Name;
    LPCSTR Definition;
}

struct D3DXSEMANTIC
{
    UINT Usage;
    UINT UsageIndex;
}

struct D3DXPASS_DESC
{
    LPCSTR Name;                        // Pass name
    UINT Annotations;                   // Number of annotations

    DWORD VSVersion;                    // Vertex shader version (0 in case of NULL shader)
    DWORD PSVersion;                    // Pixel shader version (0 in case of NULL shader)

    UINT VSSemanticsUsed;
    D3DXSEMANTIC[MAXD3DDECLLENGTH] VSSemantics;

    UINT PSSemanticsUsed;
    D3DXSEMANTIC[MAXD3DDECLLENGTH] PSSemantics;

    UINT PSSamplersUsed;
    LPCSTR PSSamplers[16];
}

struct D3DXFRAGMENT_DESC
{
    LPCSTR Name;
    DWORD Target;

}

struct D3DXANTTABLE_DESC
{
    LPCSTR Creator;                     // Creator string
    DWORD Version;                      // Shader version
    UINT Constants;                     // Number of constants
}

struct D3DXANT_DESC
{
    LPCSTR Name;                        // Constant name

    D3DXREGISTER_SET RegisterSet;       // Register set
    UINT RegisterIndex;                 // Register index
    UINT RegisterCount;                 // Number of registers occupied

    D3DXPARAMETER_CLASS Class;          // Class
    D3DXPARAMETER_TYPE Type;            // Component type

    UINT Rows;                          // Number of rows
    UINT Columns;                       // Number of columns
    UINT Elements;                      // Number of array elements
    UINT StructMembers;                 // Number of structure member sub-parameters

    UINT Bytes;                         // Data size, in bytes
    LPCVOID DefaultValue;               // Pointer to default value
}

struct D3DXSHADER_ANTTABLE
{
    DWORD Size;             // sizeof(D3DXSHADER_ANTTABLE)
    DWORD Creator;          // LPCSTR offset
    DWORD Version;          // shader version
    DWORD Constants;        // number of constants
    DWORD ConstantInfo;     // D3DXSHADER_ANTINFO[Constants] offset
}

struct D3DXSHADER_ANTINFO
{
    DWORD Name;             // LPCSTR offset
    WORD  RegisterSet;      // D3DXREGISTER_SET
    WORD  RegisterIndex;    // register number
    WORD  RegisterCount;    // number of registers
    WORD  Reserved;         // reserved
    DWORD TypeInfo;         // D3DXSHADER_TYPEINFO offset
    DWORD DefaultValue;     // offset of default value
}

struct D3DXSHADER_TYPEINFO
{
    WORD  Class;            // D3DXPARAMETER_CLASS
    WORD  Type;             // D3DXPARAMETER_TYPE
    WORD  Rows;             // number of rows (matrices)
    WORD  Columns;          // number of columns (vectors and matrices)
    WORD  Elements;         // array dimension
    WORD  StructMembers;    // number of struct members
    DWORD StructMemberInfo; // D3DXSHADER_STRUCTMEMBERINFO[Members] offset

}

struct D3DXSHADER_STRUCTMEMBERINFO
{
    DWORD Name;             // LPCSTR offset
    DWORD TypeInfo;         // D3DXSHADER_TYPEINFO offset

}




// d3dx9mesh.d
struct D3DXINTERSECTINFO
{
    DWORD FaceIndex;                // index of face intersected
    FLOAT U;                        // Barycentric Hit Coordinates    
    FLOAT V;                        // Barycentric Hit Coordinates
    FLOAT Dist;                     // Ray-Intersection Parameter Distance
}

struct D3DXATTRIBUTERANGE
{
    DWORD AttribId;
    DWORD FaceStart;
    DWORD FaceCount;
    DWORD VertexStart;
    DWORD VertexCount;
}

struct D3DXEFFECTDEFAULT
{
    LPSTR                 pParamName;
    D3DXEFFECTDEFAULTTYPE Type;           // type of the data pointed to by pValue
    DWORD                 NumBytes;       // size in bytes of the data pointed to by pValue
    LPVOID                pValue;         // data for the default of the effect
}

struct D3DXEFFECTINSTANCE
{
    LPSTR               pEffectFilename;
    DWORD               NumDefaults;
    D3DXEFFECTDEFAULT* pDefaults;
}

struct D3DXATTRIBUTEWEIGHTS
{
    FLOAT Position;
    FLOAT Boundary;
    FLOAT Normal;
    FLOAT Diffuse;
    FLOAT Specular;
    FLOAT Texcoord[8];
    FLOAT Tangent;
    FLOAT Binormal;
}

struct D3DXWELDEPSILONS
{
    FLOAT Position;                 // NOTE: This does NOT replace the epsilon in GenerateAdjacency
                                            // in general, it should be the same value or greater than the one passed to GeneratedAdjacency
    FLOAT BlendWeights;
    FLOAT Normal;
    FLOAT PSize;
    FLOAT Specular;
    FLOAT Diffuse;
    FLOAT Texcoord[8];
    FLOAT Tangent;
    FLOAT Binormal;
    FLOAT TessFactor;
}

struct D3DXBONECOMBINATION
{
    DWORD AttribId;
    DWORD FaceStart;
    DWORD FaceCount;
    DWORD VertexStart;
    DWORD VertexCount;
    DWORD* BoneId;
}

struct D3DXPATCHINFO
{
    D3DXPATCHMESHTYPE PatchType;
    D3DDEGREETYPE Degree;
    D3DBASISTYPE Basis;
}

struct D3DXTRACK_DESC
{
    DWORD Flags;
    FLOAT Weight;
    FLOAT Speed;
    BOOL  Enable;
    double AnimTime;
}

// d3d9types.d
align(4) struct D3DVECTOR {
    float x;
    float y;
    float z;

    void Set(float x, float y, float z)
    {
        this.x = x;
        this.y = y;
        this.z = z;
    }
}
alias D3DVECTOR D3DXVECTOR3;

struct D3DXKEY_VECTOR3
{
    FLOAT Time;
    D3DXVECTOR3 Value;
}

align(4) struct D3DCOLORVALUE
{
    float r;
    float g;
    float b;
    float a;
}

align(4) struct D3DRECT
{
    LONG x1;
    LONG y1;
    LONG x2;
    LONG y2;
}

align(4) struct D3DMATRIX {
    union {
        align(4) struct {
            float _11, _12, _13, _14;
            float _21, _22, _23, _24;
            float _31, _32, _33, _34;
            float _41, _42, _43, _44;
        };
        float m[4][4];
    };
}
alias D3DMATRIX D3DXMATRIX;

align(4) struct D3DVIEWPORT9
{
    DWORD X;
    DWORD Y;
    DWORD Width;
    DWORD Height;
    float MinZ;
    float MaxZ;
}

align(4) struct D3DCLIPSTATUS9
{
    DWORD ClipUnion;
    DWORD ClipIntersection;
}

align(4) struct D3DMATERIAL9 {
    D3DCOLORVALUE Diffuse = { 0, 0, 0, 0 }; /* Diffuse color RGBA */
    D3DCOLORVALUE Ambient = { 0, 0, 0, 0 }; /* Ambient color RGB */
    D3DCOLORVALUE Specular = { 0, 0, 0, 0 }; /* Specular 'shininess' */
    D3DCOLORVALUE Emissive = { 0, 0, 0, 0 }; /* Emissive color RGB */
    float Power = 0; /* Sharpness if specular highlight */
}

struct D3DXMATERIAL
{
    D3DMATERIAL9  MatD3D;
    LPSTR         pTextureFilename;
}

align(4) struct D3DLIGHT9
{
    D3DLIGHTTYPE Type;
    D3DCOLORVALUE Diffuse = { 0, 0, 0, 0 }; /* Diffuse color of light */
    D3DCOLORVALUE Specular = { 0, 0, 0, 0 }; /* Specular color of light */
    D3DCOLORVALUE Ambient = { 0, 0, 0, 0 }; /* Ambient color of light */
    D3DVECTOR Position = { 0, 0, 0 }; /* Position in world space */
    D3DVECTOR Direction = { 0, 0, 0 }; /* Direction in world space */
    float Range = 0; /* Cutoff range */
    float Falloff = 0; /* Falloff */
    float Attenuation0 = 0; /* Constant attenuation */
    float Attenuation1 = 0; /* Linear attenuation */
    float Attenuation2 = 0; /* Quadratic attenuation */
    float Theta = 0; /* Inner angle of spotlight cone */
    float Phi = 0; /* Outer angle of spotlight cone */
}

align(4) struct D3DVERTEXELEMENT9
{
    WORD    Stream;     // Stream index
    WORD    Offset;     // Offset in the stream in bytes
    BYTE    Type;       // Data type
    BYTE    Method;     // Processing method
    BYTE    Usage;      // Semantics
    BYTE    UsageIndex; // Semantic index
}

align(4) struct D3DDISPLAYMODE
{
    UINT            Width;
    UINT            Height;
    UINT            RefreshRate;
    D3DFORMAT       Format;
}

align(4) struct D3DDEVICE_CREATION_PARAMETERS
{
    UINT            AdapterOrdinal;
    D3DDEVTYPE      DeviceType;
    HWND            hFocusWindow;
    DWORD           BehaviorFlags;
}

align(4) struct D3DPRESENT_PARAMETERS
{
    UINT                BackBufferWidth;
    UINT                BackBufferHeight;
    D3DFORMAT           BackBufferFormat;
    UINT                BackBufferCount;
    D3DMULTISAMPLE_TYPE MultiSampleType;
    DWORD               MultiSampleQuality;
    D3DSWAPEFFECT       SwapEffect;
    HWND                hDeviceWindow;
    BOOL                Windowed;
    BOOL                EnableAutoDepthStencil;
    D3DFORMAT           AutoDepthStencilFormat;
    DWORD               Flags;
    UINT                FullScreen_RefreshRateInHz;
    UINT                PresentationInterval;
}

align(4) struct D3DGAMMARAMP
{
    WORD                red  [256];
    WORD                green[256];
    WORD                blue [256];
}

align(4) struct D3DVERTEXBUFFER_DESC
{
    D3DFORMAT           Format;
    D3DRESOURCETYPE     Type;
    DWORD               Usage;
    D3DPOOL             Pool;
    UINT                Size;

    DWORD               FVF;
}

align(4) struct D3DINDEXBUFFER_DESC
{
    D3DFORMAT           Format;
    D3DRESOURCETYPE     Type;
    DWORD               Usage;
    D3DPOOL             Pool;
    UINT                Size;
}

align(4) struct D3DSURFACE_DESC
{
    D3DFORMAT           Format;
    D3DRESOURCETYPE     Type;
    DWORD               Usage;
    D3DPOOL             Pool;
    D3DMULTISAMPLE_TYPE MultiSampleType;
    DWORD               MultiSampleQuality;
    UINT                Width;
    UINT                Height;
}

align(4) struct D3DVOLUME_DESC
{
    D3DFORMAT           Format;
    D3DRESOURCETYPE     Type;
    DWORD               Usage;
    D3DPOOL             Pool;

    UINT                Width;
    UINT                Height;
    UINT                Depth;
}

align(4) struct D3DLOCKED_RECT
{
    INT                 Pitch;
    void*               pBits;
}

align(4) struct D3DBOX
{
    UINT                Left;
    UINT                Top;
    UINT                Right;
    UINT                Bottom;
    UINT                Front;
    UINT                Back;
}

align(4) struct D3DLOCKED_BOX
{
    INT                 RowPitch;
    INT                 SlicePitch;
    void*               pBits;
}

/* Structures for LockRange */
align(4) struct D3DRANGE
{
    UINT                Offset;
    UINT                Size;
}

align(4) struct D3DRECTPATCH_INFO
{
    UINT                StartVertexOffsetWidth;
    UINT                StartVertexOffsetHeight;
    UINT                Width;
    UINT                Height;
    UINT                Stride;
    D3DBASISTYPE        Basis;
    D3DDEGREETYPE       Degree;
}

align(4) struct D3DTRIPATCH_INFO
{
    UINT                StartVertexOffset;
    UINT                NumVertices;
    D3DBASISTYPE        Basis;
    D3DDEGREETYPE       Degree;
}

/* Adapter Identifier */

align(4) struct D3DADAPTER_IDENTIFIER9
{
    char            Driver[MAX_DEVICE_IDENTIFIER_STRING];
    char            Description[MAX_DEVICE_IDENTIFIER_STRING];
    char            DeviceName[32];         /* Device name for GDI (ex. \\.\DISPLAY1) */

    LARGE_INTEGER   DriverVersion;          /* Defined for 32 bit components */

    DWORD           VendorId;
    DWORD           DeviceId;
    DWORD           SubSysId;
    DWORD           Revision;

    GUID            DeviceIdentifier;

    DWORD           WHQLLevel;
}

align(4) struct D3DRASTER_STATUS
{
    BOOL            InVBlank;
    UINT            ScanLine;
}

align(4) struct D3DRESOURCESTATS
{
// Data collected since last Present()
    BOOL    bThrashing;             /* indicates if thrashing */
    DWORD   ApproxBytesDownloaded;  /* Approximate number of bytes downloaded by resource manager */
    DWORD   NumEvicts;              /* number of objects evicted */
    DWORD   NumVidCreates;          /* number of objects created in video memory */
    DWORD   LastPri;                /* priority of last object evicted */
    DWORD   NumUsed;                /* number of objects set to the device */
    DWORD   NumUsedInVidMem;        /* number of objects set to the device, which are already in video memory */
// Persistent data
    DWORD   WorkingSet;             /* number of objects in video memory */
    DWORD   WorkingSetBytes;        /* number of bytes in video memory */
    DWORD   TotalManaged;           /* total number of managed objects */
    DWORD   TotalBytes;             /* total number of bytes of managed objects */
}


align(4) struct D3DDEVINFO_RESOURCEMANAGER
{
    D3DRESOURCESTATS    stats[D3DRTYPECOUNT];
}

align(4) struct D3DDEVINFO_D3DVERTEXSTATS
{
    DWORD   NumRenderedTriangles;       /* total number of triangles that are not clipped in this frame */
    DWORD   NumExtraClippingTriangles;  /* Number of new triangles generated by clipping */
}

align(4) struct D3DDEVINFO_VCACHE {
    DWORD   Pattern;                    /* bit pattern, return value must be FOUR_CC(舛・ 羨・ 舛・ 践・ */
    DWORD   OptMethod;                  /* optimization method 0 means longest strips, 1 means vertex cache based */
    DWORD   CacheSize;                  /* cache size to optimize for  (only required if type is 1) */
    DWORD   MagicNumber;                /* used to determine when to restart strips (only required if type is 1)*/
}


// d3dx9core.d
struct LOGFONT
{
	LONG lfHeight;
	LONG lfWidth;
	LONG lfEscapement;
	LONG lfOrientation;
	LONG lfWeight;
	ubyte lfItalic;
	ubyte lfUnderline;
	ubyte lfStrikeOut;
	ubyte lfCharSet;
	ubyte lfOutPrecision;
	ubyte lfClipPrecision;
	ubyte lfQuality;
	ubyte lfPitchAndFamily;
	TCHAR[1 + LF_FACESIZE-1] lfFaceName;
}

struct D3DXRTS_DESC
{
    UINT                Width;
    UINT                Height;
    D3DFORMAT           Format;
    BOOL                DepthStencil;
    D3DFORMAT           DepthStencilFormat;
}

struct D3DXRTE_DESC
{
    UINT        Size;
    UINT        MipLevels;
    D3DFORMAT   Format;
    BOOL        DepthStencil;
    D3DFORMAT   DepthStencilFormat;
}

// d3dx9math.d
struct D3DXFLOAT16
{
	WORD value;
}

struct D3DXVECTOR2
{
	FLOAT x, y;
}

struct D3DXVECTOR2_16F
{
	D3DXFLOAT16 x, y;
}

struct D3DXVECTOR3_16F
{
	D3DXFLOAT16 x, y, z;
}

struct D3DXVECTOR4
{
	FLOAT x, y, z, w;
}

struct D3DXVECTOR4_16F
{
	D3DXFLOAT16 x, y, z, w;
}

alias D3DXMATRIX D3DXMATRIXA16;

struct D3DXQUATERNION
{
	FLOAT x, y, z, w;
}

struct D3DXPLANE
{
	FLOAT a, b, c, d;
}

struct D3DXCOLOR
{
	FLOAT r, g, b, a;
}

struct D3DXKEY_QUATERNION
{
    FLOAT Time;
    D3DXQUATERNION Value;
}

struct RGNDATAHEADER
{
    DWORD   dwSize;
    DWORD   iType;
    DWORD   nCount;
    DWORD   nRgnSize;
    RECT    rcBound;
}

struct RGNDATA
{
    RGNDATAHEADER   rdh;
    char            Buffer[1];
}

struct POINTFLOAT
{
  FLOAT  x; 
  FLOAT  y; 
}

struct GLYPHMETRICSFLOAT
{
  FLOAT      gmfBlackBoxX; 
  FLOAT      gmfBlackBoxY; 
  POINTFLOAT gmfptGlyphOrigin; 
  FLOAT      gmfCellIncX; 
  FLOAT      gmfCellIncY; 
}








































alias HANDLE HMONITOR;
alias DWORD DXFILEFORMAT;
alias DWORD DXFILELOADOPTIONS;
alias GUID* REFGUID;
alias GUID* LPGUID;
alias IID* REFIID;
alias void* IStream;

interface ID3DXAllocateHierarchy : IUnknown
{
extern (Windows):
    HRESULT CreateFrame(LPCSTR Name, 
                            D3DXFRAME* *ppNewFrame);
    HRESULT CreateMeshContainer(LPCSTR Name, D3DXMESHDATA* pMeshData, 
                            D3DXMATERIAL* pMaterials, D3DXEFFECTINSTANCE* pEffectInstances, DWORD NumMaterials, 
                            DWORD *pAdjacency, ID3DXSkinInfo pSkinInfo, 
                            D3DXMESHCONTAINER* *ppNewMeshContainer);
    HRESULT DestroyFrame(D3DXFRAME* pFrameToFree);
    HRESULT DestroyMeshContainer(D3DXMESHCONTAINER* pMeshContainerToFree); 
}

interface ID3DXLoadUserData : IUnknown
{
extern (Windows):
    HRESULT LoadTopLevelData(IDirectXFileData pXofChildData);                            
    HRESULT LoadFrameChildData(D3DXFRAME* pFrame, IDirectXFileData pXofChildData);
    HRESULT LoadMeshChildData(D3DXMESHCONTAINER* pMeshContainer, IDirectXFileData pXofChildData);
}

interface ID3DXSaveUserData : IUnknown
{
extern (Windows):
    HRESULT AddFrameChildData(D3DXFRAME* pFrame, 
                            IDirectXFileSaveObject pXofSave, 
                            IDirectXFileData pXofFrameData);
                            
    HRESULT AddMeshChildData(D3DXMESHCONTAINER* pMeshContainer, 
                            IDirectXFileSaveObject pXofSave, 
                            IDirectXFileData pXofMeshData);
                            
    // NOTE: this is called once per Save.  All top level objects should be added using the 
    //    provided interface.  One call adds objects before the frame hierarchy, the other after
    HRESULT AddTopLevelDataObjectsPre(IDirectXFileSaveObject pXofSave); 
    HRESULT AddTopLevelDataObjectsPost(IDirectXFileSaveObject pXofSave);                             

    // callbacks for the user to register and then save templates to the XFile
    HRESULT RegisterTemplates(IDirectXFile pXFileApi);                             
    HRESULT SaveTemplates(IDirectXFileSaveObject pXofSave);                             
}

interface ID3DXInterpolator : IUnknown
{
extern (Windows):
    LPCSTR GetName();
    double GetPeriod();
    HRESULT GetSRT(double Time, D3DXVECTOR3 *pScale, D3DXQUATERNION *pRotate, D3DXVECTOR3 *pTranslate);
    HRESULT GetLastSRT(D3DXVECTOR3 *pScale, D3DXQUATERNION *pRotate, D3DXVECTOR3 *pTranslate);
}

interface ID3DXKeyFrameInterpolator : ID3DXInterpolator
{
extern (Windows):
    UINT GetNumScaleKeys();
    HRESULT GetScaleKeys(D3DXKEY_VECTOR3* pKeys);

    UINT GetNumRotationKeys();
    HRESULT GetRotationKeys(D3DXKEY_QUATERNION* pKeys);

    UINT GetNumTranslationKeys();
    HRESULT GetTranslationKeys(D3DXKEY_VECTOR3* pKeys);

    // the value passed to D3DXCreateKeyFrameInterpolator to scale from the times in D3DXKEY_VECTOR3* to global/anim time.
    double GetSourceTicksPerSecond();
}

interface ID3DXAnimationSet : IUnknown
{
extern (Windows):
    LPCSTR GetName();
    double GetPeriod();

    UINT GetNumInterpolators();
    HRESULT GetInterpolatorByIndex(UINT Index, ID3DXInterpolator *ppInterpolator);
    HRESULT GetInterpolatorByName(LPCSTR pName, ID3DXInterpolator *ppInterpolator);
}

interface ID3DXAnimationController : IUnknown
{
extern (Windows):
  // mixing functionality
    // register outputs of SetTime
    HRESULT RegisterMatrix(LPCSTR Name, D3DXMATRIX *pMatrix);


    // AnimationSets
    UINT GetNumAnimationSets();
    HRESULT GetAnimationSet(DWORD iAnimationSet, ID3DXAnimationSet *ppAnimSet);

    HRESULT RegisterAnimationSet(ID3DXAnimationSet pAnimSet);
    HRESULT UnregisterAnimationSet(ID3DXAnimationSet pAnimSet);


    // Tracks
    UINT GetMaxNumTracks();
    HRESULT GetTrackDesc(DWORD Track, D3DXTRACK_DESC *pDesc);
    HRESULT SetTrackDesc(DWORD Track, D3DXTRACK_DESC *pDesc);
    
    HRESULT GetTrackAnimationSet(DWORD Track, ID3DXAnimationSet *ppAnimSet);
    HRESULT SetTrackAnimationSet(DWORD Track, ID3DXAnimationSet pAnimSet);

    // Individual track field access
    HRESULT SetTrackSpeed(DWORD Track, FLOAT Speed);
    HRESULT SetTrackWeight(DWORD Track, FLOAT Weight);
    HRESULT SetTrackAnimTime(DWORD Track, double AnimTime);
    HRESULT SetTrackEnable(DWORD Track, BOOL Enable);

    // Time
    double GetTime();
    HRESULT SetTime(double Time);

    HRESULT CloneAnimationController(UINT MaxNumMatrices, UINT MaxNumAnimationSets, UINT MaxNumTracks, UINT MaxNumEvents, ID3DXAnimationController *ppAnimController);

    UINT GetMaxNumMatrices();
    UINT GetMaxNumEvents();
    UINT GetMaxNumAnimationSets();

  // Sequencing abilities
    HRESULT KeyTrackSpeed(DWORD Track, FLOAT NewSpeed, double StartTime, double Duration, DWORD Method);
    HRESULT KeyTrackWeight(DWORD Track, FLOAT NewWeight, double StartTime, double Duration, DWORD Method);
    HRESULT KeyTrackAnimTime(DWORD Track, double NewAnimTime, double StartTime);
    HRESULT KeyTrackEnable(DWORD Track, BOOL NewEnable, double StartTime);

    // this functions sets the blend weight to be used to blend high and low priority tracks together.
    //  NOTE: this has no effect unless there are active animations on tracks for a given matrix that have both high and low results
    FLOAT GetPriorityBlend();
    HRESULT SetPriorityBlend(FLOAT BlendWeight);

    HRESULT KeyPriorityBlend(FLOAT NewBlendWeight, double StartTime, double Duration, DWORD Method);
}










interface IDirectXFile : IUnknown
{
	HRESULT CreateEnumObject(LPVOID, DXFILELOADOPTIONS, IDirectXFileEnumObject *);
    HRESULT CreateSaveObject(LPCSTR, DXFILEFORMAT, IDirectXFileSaveObject *);
    HRESULT RegisterTemplates(LPVOID, DWORD);
}

interface IDirectXFileEnumObject : IUnknown
{
	HRESULT GetNextDataObject(IDirectXFileData *);
    HRESULT GetDataObjectById(REFGUID, IDirectXFileData *);
    HRESULT GetDataObjectByName(LPCSTR, IDirectXFileData *);
}

interface IDirectXFileSaveObject : IUnknown
{
    HRESULT SaveTemplates(DWORD,GUID **);
    HRESULT CreateDataObject(REFGUID, LPCSTR,GUID *, DWORD, LPVOID, IDirectXFileData *);
    HRESULT SaveData(IDirectXFileData);
}

interface IDirectXFileObject : IUnknown
{
	HRESULT GetName(LPSTR, LPDWORD);
    HRESULT GetId(LPGUID);
}

interface IDirectXFileData : IDirectXFileObject
{
    HRESULT GetData(LPCSTR, DWORD *, void **);
    HRESULT GetType(GUID **);
    HRESULT GetNextObject(IDirectXFileObject *);
    HRESULT AddDataObject(IDirectXFileData);
    HRESULT AddDataReference(LPCSTR,GUID *);
    HRESULT AddBinaryObject(LPCSTR,GUID *, LPCSTR, LPVOID, DWORD);
}

interface IDirectXFileDataReference : IDirectXFileObject
{
    HRESULT Resolve(IDirectXFileData *);
}

interface IDirectXFileBinary : IDirectXFileObject
{
    HRESULT GetSize(DWORD *);
    HRESULT GetMimeType(LPCSTR *);
    HRESULT Read(LPVOID, DWORD, LPDWORD);
}

interface IDirect3D9 : IUnknown
{
extern (Windows):
    HRESULT RegisterSoftwareDevice(void* pInitializeFunction);
    UINT GetAdapterCount();
    HRESULT GetAdapterIdentifier(UINT Adapter,DWORD Flags,D3DADAPTER_IDENTIFIER9* pIdentifier);
    UINT GetAdapterModeCount(UINT Adapter,D3DFORMAT Format);
    HRESULT EnumAdapterModes(UINT Adapter,D3DFORMAT Format,UINT Mode,D3DDISPLAYMODE* pMode);
    HRESULT GetAdapterDisplayMode(UINT Adapter,D3DDISPLAYMODE* pMode);
    HRESULT CheckDeviceType(UINT iAdapter,D3DDEVTYPE DevType,D3DFORMAT DisplayFormat,D3DFORMAT BackBufferFormat,BOOL bWindowed);
    HRESULT CheckDeviceFormat(UINT Adapter,D3DDEVTYPE DeviceType,D3DFORMAT AdapterFormat,DWORD Usage,D3DRESOURCETYPE RType,D3DFORMAT CheckFormat);
    HRESULT CheckDeviceMultiSampleType(UINT Adapter,D3DDEVTYPE DeviceType,D3DFORMAT SurfaceFormat,BOOL Windowed,D3DMULTISAMPLE_TYPE MultiSampleType,DWORD* pQualityLevels);
    HRESULT CheckDepthStencilMatch(UINT Adapter,D3DDEVTYPE DeviceType,D3DFORMAT AdapterFormat,D3DFORMAT RenderTargetFormat,D3DFORMAT DepthStencilFormat);
    HRESULT CheckDeviceFormatConversion(UINT Adapter,D3DDEVTYPE DeviceType,D3DFORMAT SourceFormat,D3DFORMAT TargetFormat);
    HRESULT GetDeviceCaps(UINT Adapter,D3DDEVTYPE DeviceType,D3DCAPS9* pCaps);
    HMONITOR GetAdapterMonitor(UINT Adapter);
    HRESULT CreateDevice(UINT Adapter,D3DDEVTYPE DeviceType,HWND hFocusWindow,DWORD BehaviorFlags,D3DPRESENT_PARAMETERS* pPresentationParameters,IDirect3DDevice9* ppReturnedDeviceInterface);
}

interface IDirect3DDevice9 : IUnknown
{
extern (Windows):
    HRESULT TestCooperativeLevel();
    UINT GetAvailableTextureMem();
    HRESULT EvictManagedResources();
    HRESULT GetDirect3D(IDirect3D9* ppD3D9);
    HRESULT GetDeviceCaps(D3DCAPS9* pCaps);
    HRESULT GetDisplayMode(UINT iSwapChain,D3DDISPLAYMODE* pMode);
    HRESULT GetCreationParameters(D3DDEVICE_CREATION_PARAMETERS *pParameters);
    HRESULT SetCursorProperties(UINT XHotSpot,UINT YHotSpot,IDirect3DSurface9 pCursorBitmap);
    void SetCursorPosition(int X,int Y,DWORD Flags);
    BOOL ShowCursor(BOOL bShow);
    HRESULT CreateAdditionalSwapChain(D3DPRESENT_PARAMETERS* pPresentationParameters,IDirect3DSwapChain9* pSwapChain);
    HRESULT GetSwapChain(UINT iSwapChain,IDirect3DSwapChain9* pSwapChain);
    UINT GetNumberOfSwapChains();
    HRESULT Reset(D3DPRESENT_PARAMETERS* pPresentationParameters);
    HRESULT Present(RECT* pSourceRect,RECT* pDestRect,HWND hDestWindowOverride,RGNDATA* pDirtyRegion);
    HRESULT GetBackBuffer(UINT iSwapChain,UINT iBackBuffer,D3DBACKBUFFER_TYPE Type,IDirect3DSurface9* ppBackBuffer);
    HRESULT GetRasterStatus(UINT iSwapChain,D3DRASTER_STATUS* pRasterStatus);
    HRESULT SetDialogBoxMode(BOOL bEnableDialogs);
    void SetGammaRamp(UINT iSwapChain,DWORD Flags,D3DGAMMARAMP* pRamp);
    void GetGammaRamp(UINT iSwapChain,D3DGAMMARAMP* pRamp);
    HRESULT CreateTexture(UINT Width,UINT Height,UINT Levels,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DTexture9* ppTexture,HANDLE* pSharedHandle);
    HRESULT CreateVolumeTexture(UINT Width,UINT Height,UINT Depth,UINT Levels,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DVolumeTexture9* ppVolumeTexture,HANDLE* pSharedHandle);
    HRESULT CreateCubeTexture(UINT EdgeLength,UINT Levels,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DCubeTexture9* ppCubeTexture,HANDLE* pSharedHandle);
    HRESULT CreateVertexBuffer(UINT Length,DWORD Usage,DWORD FVF,D3DPOOL Pool,IDirect3DVertexBuffer9* ppVertexBuffer,HANDLE* pSharedHandle);
    HRESULT CreateIndexBuffer(UINT Length,DWORD Usage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DIndexBuffer9* ppIndexBuffer,HANDLE* pSharedHandle);
    HRESULT CreateRenderTarget(UINT Width,UINT Height,D3DFORMAT Format,D3DMULTISAMPLE_TYPE MultiSample,DWORD MultisampleQuality,BOOL Lockable,IDirect3DSurface9* ppSurface,HANDLE* pSharedHandle);
    HRESULT CreateDepthStencilSurface(UINT Width,UINT Height,D3DFORMAT Format,D3DMULTISAMPLE_TYPE MultiSample,DWORD MultisampleQuality,BOOL Discard,IDirect3DSurface9* ppSurface,HANDLE* pSharedHandle);
    HRESULT UpdateSurface(IDirect3DSurface9 pSourceSurface,RECT* pSourceRect,IDirect3DSurface9 pDestinationSurface,POINT* pDestPoint);
    HRESULT UpdateTexture(IDirect3DBaseTexture9 pSourceTexture,IDirect3DBaseTexture9 pDestinationTexture);
    HRESULT GetRenderTargetData(IDirect3DSurface9 pRenderTarget,IDirect3DSurface9 pDestSurface);
    HRESULT GetFrontBufferData(UINT iSwapChain,IDirect3DSurface9 pDestSurface);
    HRESULT StretchRect(IDirect3DSurface9 pSourceSurface,RECT* pSourceRect,IDirect3DSurface9 pDestSurface,RECT* pDestRect,D3DTEXTUREFILTERTYPE Filter);
    HRESULT ColorFill(IDirect3DSurface9 pSurface,RECT* pRect,D3DCOLOR color);
    HRESULT CreateOffscreenPlainSurface(UINT Width,UINT Height,D3DFORMAT Format,D3DPOOL Pool,IDirect3DSurface9* ppSurface,HANDLE* pSharedHandle);
    HRESULT SetRenderTarget(DWORD RenderTargetIndex,IDirect3DSurface9 pRenderTarget);
    HRESULT GetRenderTarget(DWORD RenderTargetIndex,IDirect3DSurface9* ppRenderTarget);
    HRESULT SetDepthStencilSurface(IDirect3DSurface9 pNewZStencil);
    HRESULT GetDepthStencilSurface(IDirect3DSurface9* ppZStencilSurface);
    HRESULT BeginScene();
    HRESULT EndScene();
    HRESULT Clear(DWORD Count,D3DRECT* pRects,DWORD Flags,D3DCOLOR Color,float Z,DWORD Stencil);
    HRESULT SetTransform(D3DTRANSFORMSTATETYPE State,D3DMATRIX* pMatrix);
    HRESULT GetTransform(D3DTRANSFORMSTATETYPE State,D3DMATRIX* pMatrix);
    HRESULT MultiplyTransform(D3DTRANSFORMSTATETYPE,D3DMATRIX*);
    HRESULT SetViewport(D3DVIEWPORT9* pViewport);
    HRESULT GetViewport(D3DVIEWPORT9* pViewport);
    HRESULT SetMaterial(D3DMATERIAL9* pMaterial);
    HRESULT GetMaterial(D3DMATERIAL9* pMaterial);
    HRESULT SetLight(DWORD Index,D3DLIGHT9*);
    HRESULT GetLight(DWORD Index,D3DLIGHT9*);
    HRESULT LightEnable(DWORD Index,BOOL Enable);
    HRESULT GetLightEnable(DWORD Index,BOOL* pEnable);
    HRESULT SetClipPlane(DWORD Index,float* pPlane);
    HRESULT GetClipPlane(DWORD Index,float* pPlane);
    HRESULT SetRenderState(D3DRENDERSTATETYPE State,DWORD Value);
    HRESULT GetRenderState(D3DRENDERSTATETYPE State,DWORD* pValue);
    HRESULT CreateStateBlock(D3DSTATEBLOCKTYPE Type,IDirect3DStateBlock9* ppSB);
    HRESULT BeginStateBlock();
    HRESULT EndStateBlock(IDirect3DStateBlock9* ppSB);
    HRESULT SetClipStatus(D3DCLIPSTATUS9* pClipStatus);
    HRESULT GetClipStatus(D3DCLIPSTATUS9* pClipStatus);
    HRESULT GetTexture(DWORD Stage,IDirect3DBaseTexture9* ppTexture);
    HRESULT SetTexture(DWORD Stage,IDirect3DBaseTexture9 pTexture);
    HRESULT GetTextureStageState(DWORD Stage,D3DTEXTURESTAGESTATETYPE Type,DWORD* pValue);
    HRESULT SetTextureStageState(DWORD Stage,D3DTEXTURESTAGESTATETYPE Type,DWORD Value);
    HRESULT GetSamplerState(DWORD Sampler,D3DSAMPLERSTATETYPE Type,DWORD* pValue);
    HRESULT SetSamplerState(DWORD Sampler,D3DSAMPLERSTATETYPE Type,DWORD Value);
    HRESULT ValidateDevice(DWORD* pNumPasses);
    HRESULT SetPaletteEntries(UINT PaletteNumber,PALETTEENTRY* pEntries);
    HRESULT GetPaletteEntries(UINT PaletteNumber,PALETTEENTRY* pEntries);
    HRESULT SetCurrentTexturePalette(UINT PaletteNumber);
    HRESULT GetCurrentTexturePalette(UINT *PaletteNumber);
    HRESULT SetScissorRect(RECT* pRect);
    HRESULT GetScissorRect(RECT* pRect);
    HRESULT SetSoftwareVertexProcessing(BOOL bSoftware);
    BOOL GetSoftwareVertexProcessing();
    HRESULT SetNPatchMode(float nSegments);
    float GetNPatchMode();
    HRESULT DrawPrimitive(D3DPRIMITIVETYPE PrimitiveType,UINT StartVertex,UINT PrimitiveCount);
    HRESULT DrawIndexedPrimitive(D3DPRIMITIVETYPE,INT BaseVertexIndex,UINT MinVertexIndex,UINT NumVertices,UINT startIndex,UINT primCount);
    HRESULT DrawPrimitiveUP(D3DPRIMITIVETYPE PrimitiveType,UINT PrimitiveCount,void* pVertexStreamZeroData,UINT VertexStreamZeroStride);
    HRESULT DrawIndexedPrimitiveUP(D3DPRIMITIVETYPE PrimitiveType,UINT MinVertexIndex,UINT NumVertices,UINT PrimitiveCount,void* pIndexData,D3DFORMAT IndexDataFormat,void* pVertexStreamZeroData,UINT VertexStreamZeroStride);
    HRESULT ProcessVertices(UINT SrcStartIndex,UINT DestIndex,UINT VertexCount,IDirect3DVertexBuffer9 pDestBuffer,IDirect3DVertexDeclaration9 pVertexDecl,DWORD Flags);
    HRESULT CreateVertexDeclaration(D3DVERTEXELEMENT9* pVertexElements,IDirect3DVertexDeclaration9* ppDecl);
    HRESULT SetVertexDeclaration(IDirect3DVertexDeclaration9 pDecl);
    HRESULT GetVertexDeclaration(IDirect3DVertexDeclaration9* ppDecl);
    HRESULT SetFVF(DWORD FVF);
    HRESULT GetFVF(DWORD* pFVF);
    HRESULT CreateVertexShader(DWORD* pFunction,IDirect3DVertexShader9* ppShader);
    HRESULT SetVertexShader(IDirect3DVertexShader9 pShader);
    HRESULT GetVertexShader(IDirect3DVertexShader9* ppShader);
    HRESULT SetVertexShaderConstantF(UINT StartRegister,float* pConstantData,UINT Vector4fCount);
    HRESULT GetVertexShaderConstantF(UINT StartRegister,float* pConstantData,UINT Vector4fCount);
    HRESULT SetVertexShaderConstantI(UINT StartRegister,int* pConstantData,UINT Vector4iCount);
    HRESULT GetVertexShaderConstantI(UINT StartRegister,int* pConstantData,UINT Vector4iCount);
    HRESULT SetVertexShaderConstantB(UINT StartRegister,BOOL* pConstantData,UINT  BoolCount);
    HRESULT GetVertexShaderConstantB(UINT StartRegister,BOOL* pConstantData,UINT BoolCount);
    HRESULT SetStreamSource(UINT StreamNumber,IDirect3DVertexBuffer9 pStreamData,UINT OffsetInBytes,UINT Stride);
    HRESULT GetStreamSource(UINT StreamNumber,IDirect3DVertexBuffer9* ppStreamData,UINT* OffsetInBytes,UINT* pStride);
    HRESULT SetStreamSourceFreq(UINT StreamNumber,UINT Divider);
    HRESULT GetStreamSourceFreq(UINT StreamNumber,UINT* Divider);
    HRESULT SetIndices(IDirect3DIndexBuffer9 pIndexData);
    HRESULT GetIndices(IDirect3DIndexBuffer9* ppIndexData);
    HRESULT CreatePixelShader(DWORD* pFunction,IDirect3DPixelShader9* ppShader);
    HRESULT SetPixelShader(IDirect3DPixelShader9 pShader);
    HRESULT GetPixelShader(IDirect3DPixelShader9* ppShader);
    HRESULT SetPixelShaderConstantF(UINT StartRegister,float* pConstantData,UINT Vector4fCount);
    HRESULT GetPixelShaderConstantF(UINT StartRegister,float* pConstantData,UINT Vector4fCount);
    HRESULT SetPixelShaderConstantI(UINT StartRegister,int* pConstantData,UINT Vector4iCount);
    HRESULT GetPixelShaderConstantI(UINT StartRegister,int* pConstantData,UINT Vector4iCount);
    HRESULT SetPixelShaderConstantB(UINT StartRegister,BOOL* pConstantData,UINT  BoolCount);
    HRESULT GetPixelShaderConstantB(UINT StartRegister,BOOL* pConstantData,UINT BoolCount);
    HRESULT DrawRectPatch(UINT Handle,float* pNumSegs,D3DRECTPATCH_INFO* pRectPatchInfo);
    HRESULT DrawTriPatch(UINT Handle,float* pNumSegs,D3DTRIPATCH_INFO* pTriPatchInfo);
    HRESULT DeletePatch(UINT Handle);
    HRESULT CreateQuery(D3DQUERYTYPE Type,IDirect3DQuery9* ppQuery);
}

interface IDirect3DStateBlock9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT Capture();
    HRESULT Apply();
}

interface IDirect3DSwapChain9 : IUnknown
{
extern (Windows):
    HRESULT Present(RECT* pSourceRect,RECT* pDestRect,HWND hDestWindowOverride,RGNDATA* pDirtyRegion,DWORD dwFlags);
    HRESULT GetFrontBufferData(IDirect3DSurface9 pDestSurface);
    HRESULT GetBackBuffer(UINT iBackBuffer,D3DBACKBUFFER_TYPE Type,IDirect3DSurface9* ppBackBuffer);
    HRESULT GetRasterStatus(D3DRASTER_STATUS* pRasterStatus);
    HRESULT GetDisplayMode(D3DDISPLAYMODE* pMode);
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetPresentParameters(D3DPRESENT_PARAMETERS* pPresentationParameters);
}

interface IDirect3DResource9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT SetPrivateData(REFGUID refguid,void* pData,DWORD SizeOfData,DWORD Flags);
    HRESULT GetPrivateData(REFGUID refguid,void* pData,DWORD* pSizeOfData);
    HRESULT FreePrivateData(REFGUID refguid);
    DWORD SetPriority(DWORD PriorityNew);
    DWORD GetPriority();
    void PreLoad();
    D3DRESOURCETYPE GetType();
}

interface IDirect3DVertexDeclaration9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetDeclaration(D3DVERTEXELEMENT9*,UINT* pNumElements);
}

interface IDirect3DVertexShader9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetFunction(void*,UINT* pSizeOfData);
}

interface IDirect3DPixelShader9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetFunction(void*,UINT* pSizeOfData);
}

interface IDirect3DBaseTexture9 : IDirect3DResource9
{
extern (Windows):
    DWORD SetLOD(DWORD LODNew);
    DWORD GetLOD();
    DWORD GetLevelCount();
    HRESULT SetAutoGenFilterType(D3DTEXTUREFILTERTYPE FilterType);
    D3DTEXTUREFILTERTYPE GetAutoGenFilterType();
    void GenerateMipSubLevels();
}

interface IDirect3DTexture9 : IDirect3DBaseTexture9
{
extern (Windows):
    HRESULT GetLevelDesc(UINT Level,D3DSURFACE_DESC *pDesc);
    HRESULT GetSurfaceLevel(UINT Level,IDirect3DSurface9* ppSurfaceLevel);
    HRESULT LockRect(UINT Level,D3DLOCKED_RECT* pLockedRect,RECT* pRect,DWORD Flags);
    HRESULT UnlockRect(UINT Level);
    HRESULT AddDirtyRect(RECT* pDirtyRect);
}

interface IDirect3DVolumeTexture9 : IDirect3DBaseTexture9
{
extern (Windows):
    HRESULT GetLevelDesc(UINT Level,D3DVOLUME_DESC *pDesc);
    HRESULT GetVolumeLevel(UINT Level,IDirect3DVolume9* ppVolumeLevel);
    HRESULT LockBox(UINT Level,D3DLOCKED_BOX* pLockedVolume,D3DBOX* pBox,DWORD Flags);
    HRESULT UnlockBox(UINT Level);
    HRESULT AddDirtyBox(D3DBOX* pDirtyBox);
}

interface IDirect3DCubeTexture9 : IDirect3DBaseTexture9
{
extern (Windows):
    HRESULT GetLevelDesc(UINT Level,D3DSURFACE_DESC *pDesc);
    HRESULT GetCubeMapSurface(D3DCUBEMAP_FACES FaceType,UINT Level,IDirect3DSurface9* ppCubeMapSurface);
    HRESULT LockRect(D3DCUBEMAP_FACES FaceType,UINT Level,D3DLOCKED_RECT* pLockedRect,RECT* pRect,DWORD Flags);
    HRESULT UnlockRect(D3DCUBEMAP_FACES FaceType,UINT Level);
    HRESULT AddDirtyRect(D3DCUBEMAP_FACES FaceType,RECT* pDirtyRect);
}

interface IDirect3DVertexBuffer9 : IDirect3DResource9
{
extern (Windows):
    HRESULT Lock(UINT OffsetToLock, UINT SizeToLock, VOID** ppbData, DWORD Flags);
    HRESULT Unlock();
    HRESULT GetDesc(D3DVERTEXBUFFER_DESC *pDesc);
}

interface IDirect3DIndexBuffer9 : IDirect3DResource9
{
extern (Windows):
    HRESULT Lock(UINT OffsetToLock,UINT SizeToLock,void** ppbData,DWORD Flags);
    HRESULT Unlock();
    HRESULT GetDesc(D3DINDEXBUFFER_DESC *pDesc);
}

interface IDirect3DSurface9 : IDirect3DResource9
{
extern (Windows):
    HRESULT GetContainer(REFIID riid,void** ppContainer);
    HRESULT GetDesc(D3DSURFACE_DESC *pDesc);
    HRESULT LockRect(D3DLOCKED_RECT* pLockedRect,RECT* pRect,DWORD Flags);
    HRESULT UnlockRect();
    HRESULT GetDC(HDC *phdc);
    HRESULT ReleaseDC(HDC hdc);
}

interface IDirect3DVolume9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT SetPrivateData(REFGUID refguid,void* pData,DWORD SizeOfData,DWORD Flags);
    HRESULT GetPrivateData(REFGUID refguid,void* pData,DWORD* pSizeOfData);
    HRESULT FreePrivateData(REFGUID refguid);
    HRESULT GetContainer(REFIID riid,void** ppContainer);
    HRESULT GetDesc(D3DVOLUME_DESC *pDesc);
    HRESULT LockBox(D3DLOCKED_BOX * pLockedVolume,D3DBOX* pBox,DWORD Flags);
    HRESULT UnlockBox();
}

interface IDirect3DQuery9 : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    D3DQUERYTYPE GetType();
    DWORD GetDataSize();
    HRESULT Issue(DWORD dwIssueFlags);
    HRESULT GetData(void* pData,DWORD dwSize,DWORD dwGetDataFlags);
}

interface ID3DXEffectPool : IUnknown
{
extern (Windows): // No public methods
}

interface ID3DXBaseEffect : IUnknown
{
extern (Windows):
    HRESULT GetDesc(D3DXEFFECT_DESC* pDesc);
    HRESULT GetParameterDesc(D3DXHANDLE hParameter, D3DXPARAMETER_DESC* pDesc);
    HRESULT GetTechniqueDesc(D3DXHANDLE hTechnique, D3DXTECHNIQUE_DESC* pDesc);
    HRESULT GetPassDesc(D3DXHANDLE hPass, D3DXPASS_DESC* pDesc);
    HRESULT GetFunctionDesc(D3DXHANDLE hShader, D3DXFUNCTION_DESC* pDesc);

    // Handle operations
    D3DXHANDLE GetParameter(D3DXHANDLE hParameter, UINT Index);
    D3DXHANDLE GetParameterByName(D3DXHANDLE hParameter, LPCSTR pName);
    D3DXHANDLE GetParameterBySemantic(D3DXHANDLE hParameter, LPCSTR pSemantic);
    D3DXHANDLE GetParameterElement(D3DXHANDLE hParameter, UINT Index);
    D3DXHANDLE GetTechnique(UINT Index);
    D3DXHANDLE GetTechniqueByName(LPCSTR pName);
    D3DXHANDLE GetPass(D3DXHANDLE hTechnique, UINT Index);
    D3DXHANDLE GetPassByName(D3DXHANDLE hTechnique, LPCSTR pName);
    D3DXHANDLE GetFunction(UINT Index);
    D3DXHANDLE GetFunctionByName(LPCSTR pName);
    D3DXHANDLE GetAnnotation(D3DXHANDLE hObject, UINT Index);
    D3DXHANDLE GetAnnotationByName(D3DXHANDLE hObject, LPCSTR pName);

    // Get/Set Parameters
    HRESULT SetValue(D3DXHANDLE hParameter, LPCVOID pData, UINT Bytes);
    HRESULT GetValue(D3DXHANDLE hParameter, LPVOID pData, UINT Bytes);
    HRESULT SetBool(D3DXHANDLE hParameter, BOOL b);
    HRESULT GetBool(D3DXHANDLE hParameter, BOOL* pb);
    HRESULT SetBoolArray(D3DXHANDLE hParameter,  BOOL* pb, UINT Count);
    HRESULT GetBoolArray(D3DXHANDLE hParameter, BOOL* pb, UINT Count);
    HRESULT SetInt(D3DXHANDLE hParameter, INT n);
    HRESULT GetInt(D3DXHANDLE hParameter, INT* pn);
    HRESULT SetIntArray(D3DXHANDLE hParameter,  INT* pn, UINT Count);
    HRESULT GetIntArray(D3DXHANDLE hParameter, INT* pn, UINT Count);
    HRESULT SetFloat(D3DXHANDLE hParameter, FLOAT f);
    HRESULT GetFloat(D3DXHANDLE hParameter, FLOAT* pf);
    HRESULT SetFloatArray(D3DXHANDLE hParameter,  FLOAT* pf, UINT Count);
    HRESULT GetFloatArray(D3DXHANDLE hParameter, FLOAT* pf, UINT Count);
    HRESULT SetVector(D3DXHANDLE hParameter,  D3DXVECTOR4* pVector);
    HRESULT GetVector(D3DXHANDLE hParameter, D3DXVECTOR4* pVector);
    HRESULT SetVectorArray(D3DXHANDLE hParameter,  D3DXVECTOR4* pVector, UINT Count);
    HRESULT GetVectorArray(D3DXHANDLE hParameter, D3DXVECTOR4* pVector, UINT Count);
    HRESULT SetMatrix(D3DXHANDLE hParameter,  D3DXMATRIX* pMatrix);
    HRESULT GetMatrix(D3DXHANDLE hParameter, D3DXMATRIX* pMatrix);
    HRESULT SetMatrixArray(D3DXHANDLE hParameter,  D3DXMATRIX* pMatrix, UINT Count);
    HRESULT GetMatrixArray(D3DXHANDLE hParameter, D3DXMATRIX* pMatrix, UINT Count);
    HRESULT SetMatrixPointerArray(D3DXHANDLE hParameter,  D3DXMATRIX** ppMatrix, UINT Count);
    HRESULT GetMatrixPointerArray(D3DXHANDLE hParameter, D3DXMATRIX** ppMatrix, UINT Count);
    HRESULT SetMatrixTranspose(D3DXHANDLE hParameter,  D3DXMATRIX* pMatrix);
    HRESULT GetMatrixTranspose(D3DXHANDLE hParameter, D3DXMATRIX* pMatrix);
    HRESULT SetMatrixTransposeArray(D3DXHANDLE hParameter,  D3DXMATRIX* pMatrix, UINT Count);
    HRESULT GetMatrixTransposeArray(D3DXHANDLE hParameter, D3DXMATRIX* pMatrix, UINT Count);
    HRESULT SetMatrixTransposePointerArray(D3DXHANDLE hParameter,  D3DXMATRIX** ppMatrix, UINT Count);
    HRESULT GetMatrixTransposePointerArray(D3DXHANDLE hParameter, D3DXMATRIX** ppMatrix, UINT Count);
    HRESULT SetString(D3DXHANDLE hParameter, LPCSTR pString);
    HRESULT GetString(D3DXHANDLE hParameter, LPCSTR* ppString);
    HRESULT SetTexture(D3DXHANDLE hParameter, IDirect3DBaseTexture9 pTexture);
    HRESULT GetTexture(D3DXHANDLE hParameter, IDirect3DBaseTexture9 *ppTexture);
    HRESULT SetPixelShader(D3DXHANDLE hParameter, IDirect3DPixelShader9 pPShader);
    HRESULT GetPixelShader(D3DXHANDLE hParameter, IDirect3DPixelShader9 *ppPShader);
    HRESULT SetVertexShader(D3DXHANDLE hParameter, IDirect3DVertexShader9 pVShader);
    HRESULT GetVertexShader(D3DXHANDLE hParameter, IDirect3DVertexShader9 *ppVShader);
}

interface ID3DXEffect : ID3DXBaseEffect
{
extern (Windows):
    // Pool
    HRESULT GetPool(ID3DXEffectPool* ppPool);

    // Selecting and setting a technique
    HRESULT SetTechnique(D3DXHANDLE hTechnique);
    D3DXHANDLE GetCurrentTechnique();
    HRESULT ValidateTechnique(D3DXHANDLE hTechnique);
    HRESULT FindNextValidTechnique(D3DXHANDLE hTechnique, D3DXHANDLE *pTechnique);
    BOOL IsParameterUsed(D3DXHANDLE hParameter, D3DXHANDLE hTechnique);

    // Using current technique
    HRESULT Begin(UINT *pPasses, DWORD Flags);
    HRESULT Pass(UINT Pass);
    HRESULT End();

    // Managing D3D Device
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT OnLostDevice();
    HRESULT OnResetDevice();

    // Cloning
    HRESULT CloneEffect(IDirect3DDevice9 pDevice, ID3DXEffect* ppEffect);
}

interface ID3DXEffectCompiler : ID3DXBaseEffect
{
extern (Windows):
    // Parameter sharing, specialization, and information
    HRESULT SetLiteral(D3DXHANDLE hParameter, BOOL Literal);
    HRESULT GetLiteral(D3DXHANDLE hParameter, BOOL *pLiteral);

    // Compilation
    HRESULT CompileEffect(DWORD Flags, ID3DXBuffer* ppEffect, ID3DXBuffer* ppErrorMsgs);

    HRESULT CompileShader(D3DXHANDLE hFunction, LPCSTR pTarget, DWORD Flags,
        ID3DXBuffer* ppShader, ID3DXBuffer* ppErrorMsgs, D3DXSHADER_CONSTANTTABLE* ppConstantTable);
}












interface ID3DXBuffer : IUnknown
{
extern (Windows):
    LPVOID GetBufferPointer();
    DWORD GetBufferSize();
}


interface ID3DXFont : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetLogFont(LOGFONT* pLogFont);

    HRESULT Begin();
    INT DrawTextA(LPCSTR  pString, INT Count, LPRECT pRect, DWORD Format, D3DCOLOR Color);
    INT DrawTextW(LPCWSTR pString, INT Count, LPRECT pRect, DWORD Format, D3DCOLOR Color);
    HRESULT End();

    HRESULT OnLostDevice();
    HRESULT OnResetDevice();
}

interface ID3DXSprite : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);

    HRESULT Begin();

    HRESULT Draw(IDirect3DTexture9  pSrcTexture,
        RECT* pSrcRect, D3DXVECTOR2* pScaling, 
        D3DXVECTOR2* pRotationCenter, FLOAT Rotation, 
        D3DXVECTOR2* pTranslation, D3DCOLOR Color);

    HRESULT DrawTransform(IDirect3DTexture9 pSrcTexture, 
        RECT* pSrcRect, D3DXMATRIX* pTransform, 
        D3DCOLOR Color);

    HRESULT End();

    HRESULT OnLostDevice();
    HRESULT OnResetDevice();
}

interface ID3DXRenderToSurface : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetDesc(D3DXRTS_DESC* pDesc);

    HRESULT BeginScene(IDirect3DSurface9 pSurface, D3DVIEWPORT9* pViewport);
    HRESULT EndScene(DWORD MipFilter);

    HRESULT OnLostDevice();
    HRESULT OnResetDevice();
}

interface ID3DXRenderToEnvMap : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT GetDesc(D3DXRTE_DESC* pDesc);

    HRESULT BeginCube(
        IDirect3DCubeTexture9 pCubeTex);

    HRESULT BeginSphere(
        IDirect3DTexture9 pTex);

    HRESULT BeginHemisphere(
        IDirect3DTexture9 pTexZPos,
        IDirect3DTexture9 pTexZNeg);

    HRESULT BeginParabolic(
        IDirect3DTexture9 pTexZPos,
        IDirect3DTexture9 pTexZNeg);

    HRESULT Face(D3DCUBEMAP_FACES Face, DWORD MipFilter);
    HRESULT End(DWORD MipFilter);

    HRESULT OnLostDevice();
    HRESULT OnResetDevice();
}

interface ID3DXLine : IUnknown
{
extern (Windows):
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);

    HRESULT Begin();

    HRESULT Draw(D3DXVECTOR2 *pVertexList,
        DWORD dwVertexListCount, D3DCOLOR Color);

    HRESULT DrawTransform(D3DXVECTOR3 *pVertexList,
        DWORD dwVertexListCount, D3DXMATRIX* pTransform, 
        D3DCOLOR Color);

    HRESULT SetPattern(DWORD dwPattern);
    DWORD GetPattern();

    HRESULT SetPatternScale(FLOAT fPatternScale);
    FLOAT GetPatternScale();

    HRESULT SetWidth(FLOAT fWidth);
    FLOAT GetWidth();

    HRESULT SetAntialias(BOOL bAntialias);
    BOOL GetAntialias();

    HRESULT SetGLLines(BOOL bGLLines);
    BOOL GetGLLines();

    HRESULT End();

    HRESULT OnLostDevice();
    HRESULT OnResetDevice();
}







interface ID3DXConstantTable : ID3DXBuffer
{
extern (Windows):
    LPVOID GetBufferPointer();
    DWORD GetBufferSize();

    // Descs
    HRESULT GetDesc(D3DXANTTABLE_DESC *pDesc);
    HRESULT GetConstantDesc(D3DXHANDLE hConstant, D3DXANT_DESC *pConstantDesc, UINT *pCount);

    // Handle operations
    D3DXHANDLE GetConstant(D3DXHANDLE hConstant, UINT Index);
    D3DXHANDLE GetConstantByName(D3DXHANDLE hConstant, LPCSTR pName);
    D3DXHANDLE GetConstantElement(D3DXHANDLE hConstant, UINT Index);

    // Set Constants
    HRESULT SetDefaults(IDirect3DDevice9 pDevice);
    HRESULT SetValue(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant, LPCVOID pData, UINT Bytes);
    HRESULT SetBool(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant, BOOL b);
    HRESULT SetBoolArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  BOOL* pb, UINT Count);
    HRESULT SetInt(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant, INT n);
    HRESULT SetIntArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  INT* pn, UINT Count);
    HRESULT SetFloat(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant, FLOAT f);
    HRESULT SetFloatArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  FLOAT* pf, UINT Count);
    HRESULT SetVector(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXVECTOR4* pVector);
    HRESULT SetVectorArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXVECTOR4* pVector, UINT Count);
    HRESULT SetMatrix(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXMATRIX* pMatrix);
    HRESULT SetMatrixArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXMATRIX* pMatrix, UINT Count);
    HRESULT SetMatrixPointerArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXMATRIX** ppMatrix, UINT Count);
    HRESULT SetMatrixTranspose(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXMATRIX* pMatrix);
    HRESULT SetMatrixTransposeArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXMATRIX* pMatrix, UINT Count);
    HRESULT SetMatrixTransposePointerArray(IDirect3DDevice9 pDevice, D3DXHANDLE hConstant,  D3DXMATRIX** ppMatrix, UINT Count);
}

interface ID3DXFragmentLinker : IUnknown
{
extern (Windows):
    // fragment access and information retrieval functions
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    UINT GetNumberOfFragments();

    D3DXHANDLE GetFragmentHandleByIndex(UINT Index);
    D3DXHANDLE GetFragmentHandleByName(LPCSTR Name);
    HRESULT GetFragmentDesc(D3DXHANDLE Name, D3DXFRAGMENT_DESC* FragDesc);

    // add the fragments in the buffer to the linker
    HRESULT AddFragments( DWORD *Fragments);

    // Create a buffer containing the fragments.  Suitable for saving to disk
    HRESULT GetAllFragments(ID3DXBuffer *ppBuffer);
    HRESULT GetFragment(D3DXHANDLE Name, ID3DXBuffer *ppBuffer);

    HRESULT LinkShader(LPCSTR pTarget, DWORD Flags, D3DXHANDLE* rgFragmentHandles, UINT cFragments, ID3DXBuffer *ppBuffer, ID3DXBuffer *ppErrorMsgs);
    HRESULT LinkVertexShader(LPCSTR pTarget, DWORD Flags, D3DXHANDLE* rgFragmentHandles, UINT cFragments, IDirect3DVertexShader9 *pVShader, ID3DXBuffer *ppErrorMsgs);

    HRESULT ClearCache();
}

interface ID3DXInclude
{
    HRESULT Open(D3DXINCLUDE_TYPE IncludeType, LPCSTR pFileName, LPCVOID pParentData, LPCVOID *ppData, UINT *pBytes);
    HRESULT Close(LPCVOID pData);
}

interface ID3DXBaseMesh : IUnknown
{
extern (Windows):
    HRESULT DrawSubset(DWORD AttribId);
    DWORD GetNumFaces();
    DWORD GetNumVertices();
    DWORD GetFVF();
    HRESULT GetDeclaration(D3DVERTEXELEMENT9 Declaration[MAX_FVF_DECL_SIZE]);
    DWORD GetNumBytesPerVertex();
    DWORD GetOptions();
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT CloneMeshFVF(DWORD Options, DWORD FVF, IDirect3DDevice9 pD3DDevice, ID3DXMesh* ppCloneMesh);
    HRESULT CloneMesh(DWORD Options, D3DVERTEXELEMENT9 *pDeclaration, IDirect3DDevice9 pD3DDevice, ID3DXMesh* ppCloneMesh);
    HRESULT GetVertexBuffer(IDirect3DVertexBuffer9* ppVB);
    HRESULT GetIndexBuffer(IDirect3DIndexBuffer9* ppIB);
    HRESULT LockVertexBuffer(DWORD Flags, void** ppData);
    HRESULT UnlockVertexBuffer();
    HRESULT LockIndexBuffer(DWORD Flags, void** ppData);
    HRESULT UnlockIndexBuffer();
    HRESULT GetAttributeTable(D3DXATTRIBUTERANGE *pAttribTable, DWORD* pAttribTableSize);

    HRESULT ConvertPointRepsToAdjacency( DWORD* pPRep, DWORD* pAdjacency);
    HRESULT ConvertAdjacencyToPointReps( DWORD* pAdjacency, DWORD* pPRep);
    HRESULT GenerateAdjacency(FLOAT Epsilon, DWORD* pAdjacency);

    HRESULT UpdateSemantics(D3DVERTEXELEMENT9 Declaration[MAX_FVF_DECL_SIZE]);
}

interface ID3DXMesh : ID3DXBaseMesh
{
extern (Windows):
    HRESULT LockAttributeBuffer(DWORD Flags, DWORD** ppData);
    HRESULT UnlockAttributeBuffer();
    HRESULT Optimize(DWORD Flags,  DWORD* pAdjacencyIn, DWORD* pAdjacencyOut, 
                     DWORD* pFaceRemap, ID3DXBuffer *ppVertexRemap,  
                     ID3DXMesh* ppOptMesh);
    HRESULT OptimizeInplace(DWORD Flags,  DWORD* pAdjacencyIn, DWORD* pAdjacencyOut, 
                     DWORD* pFaceRemap, ID3DXBuffer *ppVertexRemap);
    HRESULT SetAttributeTable(D3DXATTRIBUTERANGE *pAttribTable, DWORD cAttribTableSize);
}

interface ID3DXPMesh : ID3DXBaseMesh
{
extern (Windows):
    HRESULT ClonePMeshFVF(DWORD Options, 
                DWORD FVF, IDirect3DDevice9 pD3D, ID3DXPMesh* ppCloneMesh);
    HRESULT ClonePMesh(DWORD Options, 
                 D3DVERTEXELEMENT9 *pDeclaration, IDirect3DDevice9 pD3D, ID3DXPMesh* ppCloneMesh);
    HRESULT SetNumFaces(DWORD Faces);
    HRESULT SetNumVertices(DWORD Vertices);
    DWORD GetMaxFaces();
    DWORD GetMinFaces();
    DWORD GetMaxVertices();
    DWORD GetMinVertices();
    HRESULT Save(IStream *pStream,  D3DXMATERIAL* pMaterials,  D3DXEFFECTINSTANCE* pEffectInstances, DWORD NumMaterials);

    HRESULT Optimize(DWORD Flags, DWORD* pAdjacencyOut, 
                     DWORD* pFaceRemap, ID3DXBuffer *ppVertexRemap,  
                     ID3DXMesh* ppOptMesh);

    HRESULT OptimizeBaseLOD(DWORD Flags, DWORD* pFaceRemap);
    HRESULT TrimByFaces(DWORD NewFacesMin, DWORD NewFacesMax, DWORD *rgiFaceRemap, DWORD *rgiVertRemap);
    HRESULT TrimByVertices(DWORD NewVerticesMin, DWORD NewVerticesMax, DWORD *rgiFaceRemap, DWORD *rgiVertRemap);

    HRESULT GetAdjacency(DWORD* pAdjacency);

    //  Used to generate the immediate "ancestor" for each vertex when it is removed by a vsplit.  Allows generation of geomorphs
    //     Vertex buffer must be equal to or greater than the maximum number of vertices in the pmesh
    HRESULT GenerateVertexHistory(DWORD* pVertexHistory);
}

interface ID3DXSPMesh : IUnknown
{
extern (Windows):
    DWORD GetNumFaces();
    DWORD GetNumVertices();
    DWORD GetFVF();
    HRESULT GetDeclaration(D3DVERTEXELEMENT9 Declaration[MAX_FVF_DECL_SIZE]);
    DWORD GetOptions();
    HRESULT GetDevice(IDirect3DDevice9* ppDevice);
    HRESULT CloneMeshFVF(DWORD Options, 
                DWORD FVF, IDirect3DDevice9 pD3D, DWORD *pAdjacencyOut, DWORD *pVertexRemapOut, ID3DXMesh* ppCloneMesh);
    HRESULT CloneMesh(DWORD Options, 
                 D3DVERTEXELEMENT9 *pDeclaration, IDirect3DDevice9 pD3DDevice, DWORD *pAdjacencyOut, DWORD *pVertexRemapOut, ID3DXMesh* ppCloneMesh);
    HRESULT ClonePMeshFVF(DWORD Options, 
                DWORD FVF, IDirect3DDevice9 pD3D, DWORD *pVertexRemapOut, FLOAT *pErrorsByFace, ID3DXPMesh* ppCloneMesh);
    HRESULT ClonePMesh(DWORD Options, 
                 D3DVERTEXELEMENT9 *pDeclaration, IDirect3DDevice9 pD3D, DWORD *pVertexRemapOut, FLOAT *pErrorsbyFace, ID3DXPMesh* ppCloneMesh);
    HRESULT ReduceFaces(DWORD Faces);
    HRESULT ReduceVertices(DWORD Vertices);
    DWORD GetMaxFaces();
    DWORD GetMaxVertices();
    HRESULT GetVertexAttributeWeights(D3DXATTRIBUTEWEIGHTS* pVertexAttributeWeights);
    HRESULT GetVertexWeights(FLOAT *pVertexWeights);
}

interface ID3DXPatchMesh : IUnknown
{
extern (Windows):
    // Return creation parameters
    DWORD GetNumPatches();
    DWORD GetNumVertices();
    HRESULT GetDeclaration(D3DVERTEXELEMENT9*);
    DWORD GetControlVerticesPerPatch();
    DWORD GetOptions();
    HRESULT GetDevice(IDirect3DDevice9 *ppDevice);
    HRESULT GetPatchInfo(D3DXPATCHINFO* PatchInfo);

    // Control mesh access    
    HRESULT GetVertexBuffer(IDirect3DVertexBuffer9* ppVB);
    HRESULT GetIndexBuffer(IDirect3DIndexBuffer9* ppIB);
    HRESULT LockVertexBuffer(DWORD flags, void** ppData);
    HRESULT UnlockVertexBuffer();
    HRESULT LockIndexBuffer(DWORD flags, void** ppData);
    HRESULT UnlockIndexBuffer();
    HRESULT LockAttributeBuffer(DWORD flags, DWORD** ppData);
    HRESULT UnlockAttributeBuffer();

    // This function returns the size of the tessellated mesh given a tessellation level.
    // This assumes uniform tessellation. For adaptive tessellation the Adaptive parameter must
    // be set to TRUE and TessellationLevel should be the max tessellation.
    // This will result in the max mesh size necessary for adaptive tessellation.    
    HRESULT GetTessSize(FLOAT fTessLevel,DWORD Adapative, DWORD *NumTriangles,DWORD *NumVertices);
    
    //GenerateAdjacency determines which patches are adjacent with provided tolerance
    //this information is used internally to optimize tessellation
    HRESULT GenerateAdjacency(FLOAT Tolerance);
    
    //CloneMesh Creates a new patchmesh with the specified decl, and converts the vertex buffer
    //to the new decl. Entries in the new decl which are new are set to 0. If the current mesh
    //has adjacency, the new mesh will also have adjacency
    HRESULT CloneMesh(DWORD Options,  D3DVERTEXELEMENT9 *pDecl, ID3DXPatchMesh *pMesh);
    
    // Optimizes the patchmesh for efficient tessellation. This function is designed
    // to perform one time optimization for patch meshes that need to be tessellated
    // repeatedly by calling the Tessellate() method. The optimization performed is
    // independent of the actual tessellation level used.
    // Currently Flags is unused.
    // If vertices are changed, Optimize must be called again
    HRESULT Optimize(DWORD flags);

    //gets and sets displacement parameters
    //displacement maps can only be 2D textures MIP-MAPPING is ignored for non adapative tessellation
    HRESULT SetDisplaceParam(IDirect3DBaseTexture9 Texture,
                              D3DTEXTUREFILTERTYPE MinFilter,
                              D3DTEXTUREFILTERTYPE MagFilter,
                              D3DTEXTUREFILTERTYPE MipFilter,
                              D3DTEXTUREADDRESS Wrap,
                              DWORD dwLODBias);

    HRESULT GetDisplaceParam(IDirect3DBaseTexture9 *Texture,
                                D3DTEXTUREFILTERTYPE *MinFilter,
                                D3DTEXTUREFILTERTYPE *MagFilter,
                                D3DTEXTUREFILTERTYPE *MipFilter,
                                D3DTEXTUREADDRESS *Wrap,
                                DWORD *dwLODBias);
        
    // Performs the uniform tessellation based on the tessellation level. 
    // This function will perform more efficiently if the patch mesh has been optimized using the Optimize() call.
    HRESULT Tessellate(FLOAT fTessLevel,ID3DXMesh pMesh);

    // Performs adaptive tessellation based on the Z based adaptive tessellation criterion.
    // pTrans specifies a 4D vector that is dotted with the vertices to get the per vertex
    // adaptive tessellation amount. Each edge is tessellated to the average of the criterion
    // at the 2 vertices it connects.
    // MaxTessLevel specifies the upper limit for adaptive tesselation.
    // This function will perform more efficiently if the patch mesh has been optimized using the Optimize() call.
    HRESULT TessellateAdaptive(D3DXVECTOR4 *pTrans,DWORD dwMaxTessLevel, DWORD dwMinTessLevel,ID3DXMesh pMesh);
}

interface ID3DXSkinInfo : IUnknown
{
extern (Windows):
    // Specify the which vertices do each bones influence and by how much
    HRESULT SetBoneInfluence(DWORD bone, DWORD numInfluences,  DWORD* vertices,  FLOAT* weights);
    DWORD GetNumBoneInfluences(DWORD bone);
    HRESULT GetBoneInfluence(DWORD bone, DWORD* vertices, FLOAT* weights);
    HRESULT GetMaxVertexInfluences(DWORD* maxVertexInfluences);
    DWORD GetNumBones();

    // This gets the max face influences based on a triangle mesh with the specified index buffer
    HRESULT GetMaxFaceInfluences(IDirect3DIndexBuffer9 pIB, DWORD NumFaces, DWORD* maxFaceInfluences);
    
    // Set min bone influence. Bone influences that are smaller than this are ignored
    HRESULT SetMinBoneInfluence(FLOAT MinInfl);
    // Get min bone influence. 
    FLOAT GetMinBoneInfluence();
    
    // Bone names are returned by D3DXLoadSkinMeshFromXof. They are not used by any other method of this object
    HRESULT SetBoneName(DWORD Bone, LPCSTR pName); // pName is copied to an internal string buffer
    LPCSTR GetBoneName(DWORD Bone); // A pointer to an internal string buffer is returned. Do not free this.
    
    // Bone offset matrices are returned by D3DXLoadSkinMeshFromXof. They are not used by any other method of this object
    HRESULT SetBoneOffsetMatrix(DWORD Bone, D3DXMATRIX* pBoneTransform); // pBoneTransform is copied to an internal buffer
    D3DXMATRIX* GetBoneOffsetMatrix(DWORD Bone); // A pointer to an internal matrix is returned. Do not free this.
    
    // Clone a skin info object
    HRESULT Clone(ID3DXSkinInfo* ppSkinInfo);
    
    // Update bone influence information to match vertices after they are reordered. This should be called 
    // if the target vertex buffer has been reordered externally.
    HRESULT Remap(DWORD NumVertices, DWORD* pVertexRemap);

    // These methods enable the modification of the vertex layout of the vertices that will be skinned
    HRESULT SetFVF(DWORD FVF);
    HRESULT SetDeclaration( D3DVERTEXELEMENT9 *pDeclaration);
    DWORD GetFVF();
    HRESULT GetDeclaration(D3DVERTEXELEMENT9 Declaration[MAX_FVF_DECL_SIZE]);

    // Apply SW skinning based on current pose matrices to the target vertices.
    HRESULT UpdateSkinnedMesh(
         D3DXMATRIX* pBoneTransforms, 
         D3DXMATRIX* pBoneInvTransposeTransforms, 
        PVOID pVerticesSrc, 
        PVOID pVerticesDst);

    // Takes a mesh and returns a new mesh with per vertex blend weights and a bone combination
    // table that describes which bones affect which subsets of the mesh
    HRESULT ConvertToBlendedMesh(
        ID3DXMesh pMesh,
        DWORD Options, 
         DWORD *pAdjacencyIn, 
        LPDWORD pAdjacencyOut,
        DWORD* pFaceRemap, 
        ID3DXBuffer *ppVertexRemap, 
        DWORD* pMaxFaceInfl,
        DWORD* pNumBoneCombinations, 
        ID3DXBuffer* ppBoneCombinationTable, 
        ID3DXMesh* ppMesh);

    // Takes a mesh and returns a new mesh with per vertex blend weights and indices 
    // and a bone combination table that describes which bones palettes affect which subsets of the mesh
    HRESULT ConvertToIndexedBlendedMesh(
        ID3DXMesh pMesh,
        DWORD Options, 
        DWORD paletteSize, 
         DWORD *pAdjacencyIn, 
        LPDWORD pAdjacencyOut, 
        DWORD* pFaceRemap, 
        ID3DXBuffer *ppVertexRemap, 
        DWORD* pMaxVertexInfl,
        DWORD* pNumBoneCombinations, 
        ID3DXBuffer* ppBoneCombinationTable, 
        ID3DXMesh* ppMesh);
}


interface ID3DXMatrixStack : IUnknown
{
extern (Windows):
    // Pops the top of the stack, returns the current top
    // *after* popping the top.
    HRESULT Pop();

    // Pushes the stack by one, duplicating the current matrix.
    HRESULT Push();

    // Loads identity in the current matrix.
    HRESULT LoadIdentity();

    // Loads the given matrix into the current matrix
    HRESULT LoadMatrix(D3DXMATRIX* pM );

    // Right-Multiplies the given matrix to the current matrix.
    // (transformation is about the current world origin)
    HRESULT MultMatrix(D3DXMATRIX* pM );

    // Left-Multiplies the given matrix to the current matrix
    // (transformation is about the local origin of the object)
    HRESULT MultMatrixLocal(D3DXMATRIX* pM );

    // Right multiply the current matrix with the computed rotation
    // matrix, counterclockwise about the given axis with the given angle.
    // (rotation is about the current world origin)
    HRESULT RotateAxis
        (D3DXVECTOR3* pV, FLOAT Angle);

    // Left multiply the current matrix with the computed rotation
    // matrix, counterclockwise about the given axis with the given angle.
    // (rotation is about the local origin of the object)
    HRESULT RotateAxisLocal
        (D3DXVECTOR3* pV, FLOAT Angle);

    // Right multiply the current matrix with the computed rotation
    // matrix. All angles are counterclockwise. (rotation is about the
    // current world origin)

    // The rotation is composed of a yaw around the Y axis, a pitch around
    // the X axis, and a roll around the Z axis.
    HRESULT RotateYawPitchRoll
        (FLOAT Yaw, FLOAT Pitch, FLOAT Roll);

    // Left multiply the current matrix with the computed rotation
    // matrix. All angles are counterclockwise. (rotation is about the
    // local origin of the object)

    // The rotation is composed of a yaw around the Y axis, a pitch around
    // the X axis, and a roll around the Z axis.
    HRESULT RotateYawPitchRollLocal
        (FLOAT Yaw, FLOAT Pitch, FLOAT Roll);

    // Right multiply the current matrix with the computed scale
    // matrix. (transformation is about the current world origin)
    HRESULT Scale(FLOAT x, FLOAT y, FLOAT z);

    // Left multiply the current matrix with the computed scale
    // matrix. (transformation is about the local origin of the object)
    HRESULT ScaleLocal(FLOAT x, FLOAT y, FLOAT z);

    // Right multiply the current matrix with the computed translation
    // matrix. (transformation is about the current world origin)
    HRESULT Translate(FLOAT x, FLOAT y, FLOAT z );

    // Left multiply the current matrix with the computed translation
    // matrix. (transformation is about the local origin of the object)
    HRESULT TranslateLocal(FLOAT x, FLOAT y, FLOAT z);

    // Obtain the current matrix at the top of the stack
    D3DXMATRIX* GetTop();
}










////////////////////////////////////////////////
struct D3DXMESHDATA
{
    D3DXMESHDATATYPE Type;

    // current mesh data interface
    union
    {
        ID3DXMesh              pMesh;
        ID3DXPMesh             pPMesh;
        ID3DXPatchMesh         pPatchMesh;
    };
}

struct D3DXMESHCONTAINER
{
    LPSTR                   Name;

    D3DXMESHDATA            MeshData;

    D3DXMATERIAL*          pMaterials;
    D3DXEFFECTINSTANCE*    pEffects;
    DWORD                   NumMaterials;
    DWORD                  *pAdjacency;

    ID3DXSkinInfo          pSkinInfo;

    D3DXMESHCONTAINER* pNextMeshContainer;
}

struct D3DXFRAME
{
    LPSTR                   Name;
    D3DXMATRIX              TransformationMatrix;

    D3DXMESHCONTAINER*     pMeshContainer;

    D3DXFRAME* pFrameSibling;
    D3DXFRAME* pFrameFirstChild;
}







































// すべての関数をここに

//alias long LARGE_INTEGER; // signed 64 bits

// D3DCOLOR is equivalent to D3DFMT_A8R8G8B8
//alias DWORD D3DCOLOR;

int MAKEFOURCC(BYTE ch0, BYTE ch1, BYTE ch2, BYTE ch3) { return ch0 | (ch1 << 8) | (ch2 << 16) | (ch3 << 24); }

HRESULT MAKE_HRESULT(int sev, int fac, int code) { return (sev << 31) | (fac << 16) | code; }
HRESULT MAKE_DDHRESULT(int code) { return MAKE_HRESULT( 1, _FACDD, code ); }
HRESULT MAKE_D3DHRESULT(int code) { return MAKE_HRESULT(1, _FACD3D, code); }
int MAKE_D3DSTATUS(int code) { return MAKE_HRESULT(0, _FACD3D, code); }



int D3DFVF_TEXCOORDSIZE3(int CoordIndex) { return (D3DFVF_TEXTUREFORMAT3 << (CoordIndex*2 + 16)); }
int D3DFVF_TEXCOORDSIZE2(int CoordIndex) { return (D3DFVF_TEXTUREFORMAT2); }
int D3DFVF_TEXCOORDSIZE4(int CoordIndex) { return (D3DFVF_TEXTUREFORMAT4 << (CoordIndex*2 + 16)); }
int D3DFVF_TEXCOORDSIZE1(int CoordIndex) { return (D3DFVF_TEXTUREFORMAT1 << (CoordIndex*2 + 16)); }
int D3DPS_END() { return 0x0000FFFF; }
int D3DVS_END() { return 0x0000FFFF; }
int D3DSHADER_VERSION_MAJOR(int _Version) { return (((_Version)>>8)&0xFF); }
int D3DSHADER_VERSION_MINOR(int _Version) { return (((_Version)>>0)&0xFF); }
int D3DSHADER_COMMENT(int _DWordSize)
{
	return ((((_DWordSize)<<D3DSI_COMMENTSIZE_SHIFT)&D3DSI_COMMENTSIZE_MASK)| D3DSIO_COMMENT);
}

D3DCOLOR D3DCOLOR_ARGB(ubyte a, ubyte r, ubyte g, ubyte b) { return (a << 24) | (r << 16) | (g << 8) | b; }
D3DCOLOR D3DCOLOR_RGBA(ubyte r, ubyte g, ubyte b, ubyte a) { return D3DCOLOR_ARGB(a, r, g, b); }
D3DCOLOR D3DCOLOR_XRGB(ubyte r, ubyte g, ubyte b) { return D3DCOLOR_ARGB(255, r, g, b); }
D3DCOLOR D3DCOLOR_COLORVALUE(float r, float g, float b, float a)
{
	return D3DCOLOR_RGBA(cast(ubyte)(r * 255), cast(ubyte)(g * 255), cast(ubyte)(b * 255), cast(ubyte)(a * 255));
}
D3DCOLOR D3DCOLOR_XYUV(ubyte y, ubyte u, ubyte v) { return D3DCOLOR_ARGB(0xff,y,u,v); }
D3DCOLOR D3DCOLOR_AYUV(ubyte a, ubyte y, ubyte u, ubyte v) { return D3DCOLOR_ARGB(a,y,u,v); }
ubyte[] D3DDECL_END()
{
	const ubyte[] result = [ 0xFF,0,D3DDECLTYPE_UNUSED,0,0,0 ];
	return result;
}
/////////////////////////////////////////////////////////////////////////////////////////////////////// TODO
//#define D3DSINCOSCONST1 -1.5500992e-006f, -2.1701389e-005f,  0.0026041667f, 0.00026041668f
//#define D3DSINCOSCONST2 -0.020833334f, -0.12500000f, 1.0f, 0.50000000f
///////////////////////////////////////////////////////////////////////////////////////////////////////
uint D3DPS_VERSION(int _Major, int _Minor) { return (0xFFFF0000|((_Major)<<8)|(_Minor)); }
uint D3DVS_VERSION(int _Major, int _Minor) { return (0xFFFE0000|((_Major)<<8)|(_Minor)); }







extern (Windows) BOOL function (UINT D3DSdkVersion, UINT D3DXSdkVersion) D3DXCheckVersion;

extern (Windows) UINT function (IDirect3DDevice9 pDevice) D3DXGetDriverLevel;

extern (Windows) HRESULT function(
        IDirect3DDevice9   pDevice,
        HFONT               hFont,
        ID3DXFont*         ppFont) D3DXCreateFont;

extern (Windows) HRESULT function(
        IDirect3DDevice9   pDevice,
        LOGFONT*      pLogFont,
        ID3DXFont*         ppFont) D3DXCreateFontIndirect;

extern (Windows) HRESULT function(
        IDirect3DDevice9   pDevice,
        ID3DXSprite*       ppSprite) D3DXCreateSprite;

extern (Windows) HRESULT function(
        IDirect3DDevice9       pDevice,
        UINT                    Width,
        UINT                    Height,
        D3DFORMAT               Format,
        BOOL                    DepthStencil,
        D3DFORMAT               DepthStencilFormat,
        ID3DXRenderToSurface*  ppRenderToSurface) D3DXCreateRenderToSurface;

extern (Windows) HRESULT function(
        IDirect3DDevice9       pDevice,
        UINT                    Size,
        UINT                    MipLevels,
        D3DFORMAT               Format,
        BOOL                    DepthStencil,
        D3DFORMAT               DepthStencilFormat,
        ID3DXRenderToEnvMap*   ppRenderToEnvMap) D3DXCreateRenderToEnvMap;
        
extern (Windows) HRESULT function(IDirect3DDevice9 pDevice, ID3DXLine* ppLine) D3DXCreateLine;




extern (Windows) HRESULT function(IDirectXFile* lplpDirectXFile) DirectXFileCreate;


extern (Windows)
{
	HRESULT function(ID3DXEffectPool* ppPool) D3DXCreateEffectPool;
	HRESULT function(
        IDirect3DDevice9               pDevice,
        LPCSTR                          pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectPool                pPool,
        ID3DXEffect*                   ppEffect,
        ID3DXBuffer*                   ppCompilationErrors) D3DXCreateEffectFromFileA;
	HRESULT function(
        IDirect3DDevice9               pDevice,
        LPCWSTR                         pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectPool                pPool,
        ID3DXEffect*                   ppEffect,
        ID3DXBuffer*                   ppCompilationErrors) D3DXCreateEffectFromFileW;
	HRESULT function(
        IDirect3DDevice9               pDevice,
        HMODULE                         hSrcModule,
        LPCSTR                          pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectPool                pPool,
        ID3DXEffect*                   ppEffect,
        ID3DXBuffer*                   ppCompilationErrors) D3DXCreateEffectFromResourceA;
	HRESULT function(
        IDirect3DDevice9               pDevice,
        HMODULE                         hSrcModule,
        LPCWSTR                         pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectPool                pPool,
        ID3DXEffect*                   ppEffect,
        ID3DXBuffer*                   ppCompilationErrors) D3DXCreateEffectFromResourceW;
	HRESULT function(
        IDirect3DDevice9               pDevice,
        LPCVOID                         pSrcData,
        UINT                            SrcDataLen,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectPool                pPool,
        ID3DXEffect*                   ppEffect,
        ID3DXBuffer*                   ppCompilationErrors) D3DXCreateEffect;
	HRESULT function(
        LPCSTR                          pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectCompiler*           ppCompiler,
        ID3DXBuffer*                   ppParseErrors) D3DXCreateEffectCompilerFromFileA;
	HRESULT function(
        LPCWSTR                         pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectCompiler*           ppCompiler,
        ID3DXBuffer*                   ppParseErrors) D3DXCreateEffectCompilerFromFileW;
	HRESULT function(
        HMODULE                         hSrcModule,
        LPCSTR                          pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectCompiler*           ppCompiler,
        ID3DXBuffer*                   ppParseErrors) D3DXCreateEffectCompilerFromResourceA;
	HRESULT function(
        HMODULE                         hSrcModule,
        LPCWSTR                         pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectCompiler*           ppCompiler,
        ID3DXBuffer*                   ppParseErrors) D3DXCreateEffectCompilerFromResourceW;
	HRESULT function(
        LPCSTR                          pSrcData,
        UINT                            SrcDataLen,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXEffectCompiler*           ppCompiler,
        ID3DXBuffer*                   ppParseErrors) D3DXCreateEffectCompiler;
}

extern (Windows):
HRESULT function(
        IDirect3DDevice9   pDevice,
        FLOAT               Length, 
        UINT                Sides, 
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency) D3DXCreatePolygon;

HRESULT function(
        IDirect3DDevice9   pDevice, 
        FLOAT               Width,
        FLOAT               Height,
        FLOAT               Depth,
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency) D3DXCreateBox;

HRESULT function(
        IDirect3DDevice9   pDevice,
        FLOAT               Radius1, 
        FLOAT               Radius2, 
        FLOAT               Length, 
        UINT                Slices, 
        UINT                Stacks,   
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency) D3DXCreateCylinder;

HRESULT function(
        IDirect3DDevice9  pDevice, 
        FLOAT              Radius, 
        UINT               Slices, 
        UINT               Stacks,
        ID3DXMesh*        ppMesh,
        ID3DXBuffer*      ppAdjacency) D3DXCreateSphere;

HRESULT function(
        IDirect3DDevice9   pDevice,
        FLOAT               InnerRadius,
        FLOAT               OuterRadius, 
        UINT                Sides,
        UINT                Rings, 
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency) D3DXCreateTorus;

HRESULT function(
        IDirect3DDevice9   pDevice,
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency) D3DXCreateTeapot;

HRESULT function(
        IDirect3DDevice9   pDevice,
        HDC                 hDC,
        LPCSTR              pText,
        FLOAT               Deviation,
        FLOAT               Extrusion,
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency,
        GLYPHMETRICSFLOAT* pGlyphMetrics) D3DXCreateTextA;

HRESULT function(
        IDirect3DDevice9   pDevice,
        HDC                 hDC,
        LPCWSTR             pText,
        FLOAT               Deviation,
        FLOAT               Extrusion,
        ID3DXMesh*         ppMesh,
        ID3DXBuffer*       ppAdjacency,
        GLYPHMETRICSFLOAT* pGlyphMetrics) D3DXCreateTextW;




HRESULT function(
        LPCSTR Filename,
        DWORD MeshOptions,
        IDirect3DDevice9 pD3DDevice,
        ID3DXAllocateHierarchy pAlloc,
        ID3DXLoadUserData pUserDataLoader, 
        D3DXFRAME** ppFrameHierarchy,
        ID3DXAnimationController *ppAnimController) D3DXLoadMeshHierarchyFromXA;

HRESULT function(
        LPCWSTR Filename,
        DWORD MeshOptions,
        IDirect3DDevice9 pD3DDevice,
        ID3DXAllocateHierarchy pAlloc,
        ID3DXLoadUserData pUserDataLoader, 
        D3DXFRAME* *ppFrameHierarchy,
        ID3DXAnimationController *ppAnimController) D3DXLoadMeshHierarchyFromXW;

HRESULT function(
        LPCVOID Memory,
        DWORD SizeOfMemory,
        DWORD MeshOptions,
        IDirect3DDevice9 pD3DDevice,
        ID3DXAllocateHierarchy pAlloc,
        ID3DXLoadUserData pUserDataLoader, 
        D3DXFRAME* *ppFrameHierarchy,
        ID3DXAnimationController *ppAnimController) D3DXLoadMeshHierarchyFromXInMemory;

HRESULT function(
        LPCSTR Filename,
        DWORD XFormat,
        D3DXFRAME* pFrameRoot, 
        ID3DXAnimationController pAnimMixer,
        ID3DXSaveUserData pUserDataSaver) D3DXSaveMeshHierarchyToFileA;

HRESULT function(
        LPCWSTR Filename,
        DWORD XFormat,
        D3DXFRAME* pFrameRoot, 
        ID3DXAnimationController pAnimMixer,
        ID3DXSaveUserData pUserDataSaver) D3DXSaveMeshHierarchyToFileW;

HRESULT function(
        D3DXFRAME* pFrameRoot,
        ID3DXAllocateHierarchy pAlloc) D3DXFrameDestroy;

HRESULT function(
        D3DXFRAME* pFrameParent,
        D3DXFRAME* pFrameChild) D3DXFrameAppendChild;

D3DXFRAME* function(
        D3DXFRAME* pFrameRoot,
        LPCSTR Name) D3DXFrameFind;

HRESULT function(
        D3DXFRAME* pFrameRoot,
        ID3DXAnimationController pAnimMixer) D3DXFrameRegisterNamedMatrices;

UINT function(
        D3DXFRAME* pFrameRoot) D3DXFrameNumNamedMatrices;

HRESULT function(
        D3DXFRAME* pFrameRoot, 
        D3DXVECTOR3* pObjectCenter, 
        FLOAT *pObjectRadius) D3DXFrameCalculateBoundingSphere;

HRESULT function(LPCSTR Name, 
                            D3DXKEY_VECTOR3*    ScaleKeys,     UINT NumScaleKeys,
                            D3DXKEY_QUATERNION* RotationKeys,  UINT NumRotationKeys,
                            D3DXKEY_VECTOR3*    TranslateKeys, UINT NumTranslateKeys,
                            double ScaleInputTimeBy, ID3DXKeyFrameInterpolator *ppNewInterpolator) D3DXCreateKeyFrameInterpolator;

HRESULT function(LPCSTR Name, 
                            ID3DXInterpolator *ppInterpolators, UINT NumInterpolators,
                            ID3DXAnimationSet *ppAnimSet) D3DXCreateAnimationSet;

HRESULT function(UINT MaxNumMatrices, UINT MaxNumAnimationSets, UINT MaxNumTracks, UINT MaxNumEvents,
                                 ID3DXAnimationController *ppAnimController) D3DXCreateAnimationController;
                                 





HRESULT D3DXTX_VERSION(int _Major, int _Minor)
{
	return ((cast(char)'T' << 24) | (cast(char)'X' << 16) | ((_Major) << 8) | (_Minor));
}






HRESULT function(
        LPCSTR                          pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXAssembleShaderFromFileA;

HRESULT function(
        LPCWSTR                         pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXAssembleShaderFromFileW;

HRESULT function(
        HMODULE                         hSrcModule,
        LPCSTR                          pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXAssembleShaderFromResourceA;

HRESULT function(
        HMODULE                         hSrcModule,
        LPCWSTR                         pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXAssembleShaderFromResourceW;

HRESULT function(
        LPCSTR                          pSrcData,
        UINT                            SrcDataLen,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXAssembleShader;

HRESULT function(
        LPCSTR                          pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        LPCSTR                          pFunctionName,
        LPCSTR                          pTarget,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs,
        D3DXSHADER_CONSTANTTABLE**            ppConstantTable) D3DXCompileShaderFromFileA;

HRESULT function(
        LPCWSTR                         pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        LPCSTR                          pFunctionName,
        LPCSTR                          pTarget,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs,
        D3DXSHADER_CONSTANTTABLE**            ppConstantTable) D3DXCompileShaderFromFileW;

HRESULT function(
        HMODULE                         hSrcModule,
        LPCSTR                          pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        LPCSTR                          pFunctionName,
        LPCSTR                          pTarget,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs,
        D3DXSHADER_CONSTANTTABLE**            ppConstantTable) D3DXCompileShaderFromResourceA;

HRESULT function(
        HMODULE                         hSrcModule,
        LPCWSTR                         pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        LPCSTR                          pFunctionName,
        LPCSTR                          pTarget,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs,
        D3DXSHADER_CONSTANTTABLE**            ppConstantTable) D3DXCompileShaderFromResourceW;

HRESULT function(
        LPCSTR                          pSrcData,
        UINT                            SrcDataLen,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        LPCSTR                          pFunctionName,
        LPCSTR                          pTarget,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs,
        D3DXSHADER_CONSTANTTABLE**            ppConstantTable) D3DXCompileShader;

HRESULT function(
         DWORD*                    pFunction,
        DWORD                           FourCC,
        LPCVOID*                        ppData,
        UINT*                           pSizeInBytes) D3DXFindShaderComment;

HRESULT function(
         DWORD*                    pFunction,
        D3DXSEMANTIC*                   pSemantics,
        UINT*                           pCount) D3DXGetShaderInputSemantics;

HRESULT function(
         DWORD*                    pFunction,
        D3DXSEMANTIC*                   pSemantics,
        UINT*                           pCount) D3DXGetShaderOutputSemantics;

HRESULT function(
         DWORD*                    pFunction,
        LPCSTR*                         pSamplers,
        UINT*                           pCount) D3DXGetShaderSamplers;

HRESULT function(
         DWORD*                    pFunction,
        D3DXSHADER_CONSTANTTABLE**            ppConstantTable) D3DXGetShaderConstantTable;

HRESULT function(
         DWORD*                    pFunction,
        ID3DXBuffer*                   ppDebugInfo) D3DXGetShaderDebugInfo;

HRESULT function(
        LPCSTR                          pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXGatherFragmentsFromFileA;

HRESULT function(
        LPCWSTR                         pSrcFile,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXGatherFragmentsFromFileW;

HRESULT function(
        HMODULE                         hSrcModule,
        LPCSTR                          pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXGatherFragmentsFromResourceA;

HRESULT function(
        HMODULE                         hSrcModule,
        LPCWSTR                         pSrcResource,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXGatherFragmentsFromResourceW;

HRESULT function(
        LPCSTR                          pSrcData,
        UINT                            SrcDataLen,
         D3DXMACRO*                pDefines,
        ID3DXInclude                   pInclude,
        DWORD                           Flags,
        ID3DXBuffer*                   ppShader,
        ID3DXBuffer*                   ppErrorMsgs) D3DXGatherFragments;

HRESULT function(
        IDirect3DDevice9               pDevice,
        UINT                            ShaderCacheSize,
        ID3DXFragmentLinker*           ppFragmentLinker) D3DXCreateFragmentLinker;


void function(D3DXVECTOR4 *pOut, D3DXVECTOR2 *pTexCoord, D3DXVECTOR2 *pTexelSize, LPVOID pData) LPD3DXFILL2D;
void function(D3DXVECTOR4 *pOut, D3DXVECTOR3 *pTexCoord, D3DXVECTOR3 *pTexelSize, LPVOID pData) LPD3DXFILL3D;





HRESULT function(
        LPCSTR                    pSrcFile,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXGetImageInfoFromFileA;

HRESULT function(
        LPCWSTR                   pSrcFile,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXGetImageInfoFromFileW;

HRESULT function(
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXGetImageInfoFromResourceA;

HRESULT function(
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXGetImageInfoFromResourceW;

HRESULT function(
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXGetImageInfoFromFileInMemory;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        LPCSTR                    pSrcFile,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadSurfaceFromFileA;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        LPCWSTR                   pSrcFile,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadSurfaceFromFileW;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadSurfaceFromResourceA;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadSurfaceFromResourceW;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadSurfaceFromFileInMemory;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        IDirect3DSurface9        pSrcSurface,
         PALETTEENTRY*       pSrcPalette,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey) D3DXLoadSurfaceFromSurface;

HRESULT function(
        IDirect3DSurface9        pDestSurface,
         PALETTEENTRY*       pDestPalette,
         RECT*               pDestRect,
        LPCVOID                   pSrcMemory,
        D3DFORMAT                 SrcFormat,
        UINT                      SrcPitch,
         PALETTEENTRY*       pSrcPalette,
         RECT*               pSrcRect,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey) D3DXLoadSurfaceFromMemory;

HRESULT function(
        LPCSTR                    pDestFile,
        D3DXIMAGE_FILEFORMAT      DestFormat,
        IDirect3DSurface9        pSrcSurface,
         PALETTEENTRY*       pSrcPalette,
         RECT*               pSrcRect) D3DXSaveSurfaceToFileA;

HRESULT function(
        LPCWSTR                   pDestFile,
        D3DXIMAGE_FILEFORMAT      DestFormat,
        IDirect3DSurface9        pSrcSurface,
         PALETTEENTRY*       pSrcPalette,
         RECT*               pSrcRect) D3DXSaveSurfaceToFileW;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        LPCSTR                    pSrcFile,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadVolumeFromFileA;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        LPCWSTR                   pSrcFile,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadVolumeFromFileW;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadVolumeFromResourceA;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadVolumeFromResourceW;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo) D3DXLoadVolumeFromFileInMemory;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        IDirect3DVolume9         pSrcVolume,
         PALETTEENTRY*       pSrcPalette,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey) D3DXLoadVolumeFromVolume;

HRESULT function(
        IDirect3DVolume9         pDestVolume,
         PALETTEENTRY*       pDestPalette,
         D3DBOX*             pDestBox,
        LPCVOID                   pSrcMemory,
        D3DFORMAT                 SrcFormat,
        UINT                      SrcRowPitch,
        UINT                      SrcSlicePitch,
         PALETTEENTRY*       pSrcPalette,
         D3DBOX*             pSrcBox,
        DWORD                     Filter,
        D3DCOLOR                  ColorKey) D3DXLoadVolumeFromMemory;

HRESULT function(
        LPCSTR                    pDestFile,
        D3DXIMAGE_FILEFORMAT      DestFormat,
        IDirect3DVolume9         pSrcVolume,
         PALETTEENTRY*       pSrcPalette,
         D3DBOX*             pSrcBox) D3DXSaveVolumeToFileA;

HRESULT function(
        LPCWSTR                   pDestFile,
        D3DXIMAGE_FILEFORMAT      DestFormat,
        IDirect3DVolume9         pSrcVolume,
         PALETTEENTRY*       pSrcPalette,
         D3DBOX*             pSrcBox) D3DXSaveVolumeToFileW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        UINT*                     pWidth,
        UINT*                     pHeight,
        UINT*                     pNumMipLevels,
        DWORD                     Usage,
        D3DFORMAT*                pFormat,
        D3DPOOL                   Pool) D3DXCheckTextureRequirements;

HRESULT function(
        IDirect3DDevice9         pDevice,
        UINT*                     pSize,
        UINT*                     pNumMipLevels,
        DWORD                     Usage,
        D3DFORMAT*                pFormat,
        D3DPOOL                   Pool) D3DXCheckCubeTextureRequirements;

HRESULT function(
        IDirect3DDevice9         pDevice,
        UINT*                     pWidth,
        UINT*                     pHeight,
        UINT*                     pDepth,
        UINT*                     pNumMipLevels,
        DWORD                     Usage,
        D3DFORMAT*                pFormat,
        D3DPOOL                   Pool) D3DXCheckVolumeTextureRequirements;

HRESULT function(
        IDirect3DDevice9         pDevice,
        UINT                      Width,
        UINT                      Height,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        IDirect3DTexture9*       ppTexture) D3DXCreateTexture;

HRESULT function(
        IDirect3DDevice9         pDevice,
        UINT                      Size,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTexture;

HRESULT function(
        IDirect3DDevice9         pDevice,
        UINT                      Width,
        UINT                      Height,
        UINT                      Depth,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTexture;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCSTR                    pSrcFile,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromFileA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCWSTR                   pSrcFile,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromFileW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCSTR                    pSrcFile,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromFileA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCWSTR                   pSrcFile,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromFileW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCSTR                    pSrcFile,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromFileA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCWSTR                   pSrcFile,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromFileW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromResourceA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromResourceW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromResourceA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromResourceW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromResourceA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromResourceW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCSTR                    pSrcFile,
        UINT                      Width,
        UINT                      Height,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromFileExA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCWSTR                   pSrcFile,
        UINT                      Width,
        UINT                      Height,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromFileExW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCSTR                    pSrcFile,
        UINT                      Size,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromFileExA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCWSTR                   pSrcFile,
        UINT                      Size,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromFileExW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCSTR                    pSrcFile,
        UINT                      Width,
        UINT                      Height,
        UINT                      Depth,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromFileExA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCWSTR                   pSrcFile,
        UINT                      Width,
        UINT                      Height,
        UINT                      Depth,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromFileExW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        UINT                      Width,
        UINT                      Height,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromResourceExA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        UINT                      Width,
        UINT                      Height,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromResourceExW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        UINT                      Size,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromResourceExA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        UINT                      Size,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromResourceExW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCSTR                    pSrcResource,
        UINT                      Width,
        UINT                      Height,
        UINT                      Depth,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromResourceExA;

HRESULT function(
        IDirect3DDevice9         pDevice,
        HMODULE                   hSrcModule,
        LPCWSTR                   pSrcResource,
        UINT                      Width,
        UINT                      Height,
        UINT                      Depth,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromResourceExW;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromFileInMemory;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromFileInMemory;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromFileInMemory;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        UINT                      Width,
        UINT                      Height,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DTexture9*       ppTexture) D3DXCreateTextureFromFileInMemoryEx;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        UINT                      Size,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DCubeTexture9*   ppCubeTexture) D3DXCreateCubeTextureFromFileInMemoryEx;

HRESULT function(
        IDirect3DDevice9         pDevice,
        LPCVOID                   pSrcData,
        UINT                      SrcDataSize,
        UINT                      Width,
        UINT                      Height,
        UINT                      Depth,
        UINT                      MipLevels,
        DWORD                     Usage,
        D3DFORMAT                 Format,
        D3DPOOL                   Pool,
        DWORD                     Filter,
        DWORD                     MipFilter,
        D3DCOLOR                  ColorKey,
        D3DXIMAGE_INFO*           pSrcInfo,
        PALETTEENTRY*             pPalette,
        IDirect3DVolumeTexture9* ppVolumeTexture) D3DXCreateVolumeTextureFromFileInMemoryEx;

HRESULT function(
        LPCSTR                    pDestFile,
        D3DXIMAGE_FILEFORMAT      DestFormat,
        IDirect3DBaseTexture9    pSrcTexture,
         PALETTEENTRY*       pSrcPalette) D3DXSaveTextureToFileA;

HRESULT function(
        LPCWSTR                   pDestFile,
        D3DXIMAGE_FILEFORMAT      DestFormat,
        IDirect3DBaseTexture9    pSrcTexture,
         PALETTEENTRY*       pSrcPalette) D3DXSaveTextureToFileW;

HRESULT function(
        IDirect3DBaseTexture9    pBaseTexture,
         PALETTEENTRY*       pPalette,
        UINT                      SrcLevel,
        DWORD                     Filter) D3DXFilterTexture;

HRESULT function(
        IDirect3DTexture9        pTexture,
        void function(D3DXVECTOR4 *pOut, D3DXVECTOR2 *pTexCoord, D3DXVECTOR2 *pTexelSize, LPVOID pData) pFunction,
        LPVOID                    pData) D3DXFillTexture;

HRESULT function(
        IDirect3DCubeTexture9    pCubeTexture,
        void function(D3DXVECTOR4 *pOut, D3DXVECTOR3 *pTexCoord, D3DXVECTOR3 *pTexelSize, LPVOID pData) pFunction,
        LPVOID                    pData) D3DXFillCubeTexture;

HRESULT function(
        IDirect3DVolumeTexture9  pVolumeTexture,
        void function(D3DXVECTOR4 *pOut, D3DXVECTOR3 *pTexCoord, D3DXVECTOR3 *pTexelSize, LPVOID pData) pFunction,
        LPVOID                    pData) D3DXFillVolumeTexture;

HRESULT function(
        IDirect3DTexture9        pTexture,
         DWORD*              pFunction,
         D3DXVECTOR4*        pConstants,
        UINT                      Constants) D3DXFillTextureTX;


HRESULT function(
        IDirect3DCubeTexture9    pCubeTexture,
         DWORD*              pFunction,
         D3DXVECTOR4*        pConstants,
        UINT                      Constants) D3DXFillCubeTextureTX;
                                                
HRESULT function(
        IDirect3DVolumeTexture9  pVolumeTexture,
         DWORD*              pFunction,
         D3DXVECTOR4*        pConstants,
        UINT                      Constants) D3DXFillVolumeTextureTX;

HRESULT function(
        IDirect3DTexture9        pTexture,
        IDirect3DTexture9        pSrcTexture,
         PALETTEENTRY*       pSrcPalette,
        DWORD                     Flags,
        DWORD                     Channel,
        FLOAT                     Amplitude) D3DXComputeNormalMap;




HRESULT function(
        DWORD NumFaces, 
        DWORD NumVertices, 
        DWORD Options, 
         D3DVERTEXELEMENT9 *pDeclaration, 
        IDirect3DDevice9 pD3D, 
        ID3DXMesh* ppMesh) D3DXCreateMesh;

HRESULT function(
        DWORD NumFaces, 
        DWORD NumVertices, 
        DWORD Options, 
        DWORD FVF, 
        IDirect3DDevice9 pD3D, 
        ID3DXMesh* ppMesh) D3DXCreateMeshFVF;

HRESULT function(
        ID3DXMesh pMesh, 
         DWORD* pAdjacency, 
         D3DXATTRIBUTEWEIGHTS *pVertexAttributeWeights,
         FLOAT *pVertexWeights,
        ID3DXSPMesh* ppSMesh) D3DXCreateSPMesh;

HRESULT function(
    ID3DXMesh pMeshIn,
     DWORD* pAdjacencyIn,
    ID3DXMesh* ppMeshOut,
    DWORD* pAdjacencyOut,
    ID3DXBuffer* ppErrorsAndWarnings) D3DXCleanMesh;

HRESULT function(
    ID3DXMesh pMeshIn,
     DWORD* pAdjacency,
    ID3DXBuffer* ppErrorsAndWarnings) D3DXValidMesh;

HRESULT function(
        ID3DXMesh pMesh, 
         DWORD* pAdjacency, 
         D3DXATTRIBUTEWEIGHTS *pVertexAttributeWeights,
         FLOAT *pVertexWeights,
        DWORD MinValue, 
        DWORD Options, 
        ID3DXPMesh* ppPMesh) D3DXGeneratePMesh;

HRESULT function(
        ID3DXMesh pMesh, 
         DWORD* pAdjacency, 
         D3DXATTRIBUTEWEIGHTS *pVertexAttributeWeights,
         FLOAT *pVertexWeights,
        DWORD MinValue, 
        DWORD Options, 
        ID3DXMesh* ppMesh) D3DXSimplifyMesh;

HRESULT function(
        D3DXVECTOR3* pFirstPosition,       // pointer to first position
        DWORD NumVertices, 
        DWORD dwStride,                     // count in bytes to subsequent position vectors
        D3DXVECTOR3 *pCenter, 
        FLOAT *pRadius) D3DXComputeBoundingSphere;

HRESULT function(
        D3DXVECTOR3* pFirstPosition,       // pointer to first position
        DWORD NumVertices, 
        DWORD dwStride,                     // count in bytes to subsequent position vectors
        D3DXVECTOR3 *pMin, 
        D3DXVECTOR3 *pMax) D3DXComputeBoundingBox;

HRESULT function(
        ID3DXBaseMesh pMesh,
         DWORD *pAdjacency) D3DXComputeNormals;

HRESULT function(
        DWORD NumBytes, 
        ID3DXBuffer *ppBuffer) D3DXCreateBuffer;

HRESULT function(
        LPCSTR pFilename, 
        DWORD Options, 
        IDirect3DDevice9 pD3D, 
        ID3DXBuffer *ppAdjacency,
        ID3DXBuffer *ppMaterials, 
        ID3DXBuffer *ppEffectInstances, 
        DWORD *pNumMaterials,
        ID3DXMesh *ppMesh) D3DXLoadMeshFromXA;

HRESULT function(
        LPCWSTR pFilename, 
        DWORD Options, 
        IDirect3DDevice9 pD3D, 
        ID3DXBuffer *ppAdjacency,
        ID3DXBuffer *ppMaterials, 
        ID3DXBuffer *ppEffectInstances, 
        DWORD *pNumMaterials,
        ID3DXMesh *ppMesh) D3DXLoadMeshFromXW;

HRESULT function(
        LPCVOID Memory,
        DWORD SizeOfMemory,
        DWORD Options, 
        IDirect3DDevice9 pD3D, 
        ID3DXBuffer *ppAdjacency,
        ID3DXBuffer *ppMaterials, 
        ID3DXBuffer *ppEffectInstances, 
        DWORD *pNumMaterials,
        ID3DXMesh *ppMesh) D3DXLoadMeshFromXInMemory;

HRESULT function(
        HMODULE Module,
        LPCSTR Name,
        LPCSTR Type,
        DWORD Options, 
        IDirect3DDevice9 pD3D, 
        ID3DXBuffer *ppAdjacency,
        ID3DXBuffer *ppMaterials, 
        ID3DXBuffer *ppEffectInstances, 
        DWORD *pNumMaterials,
        ID3DXMesh *ppMesh) D3DXLoadMeshFromXResource;

HRESULT function(
        LPCSTR pFilename,
        ID3DXMesh pMesh,
         DWORD* pAdjacency,
         D3DXMATERIAL* pMaterials,
         D3DXEFFECTINSTANCE* pEffectInstances, 
        DWORD NumMaterials,
        DWORD Format
        ) D3DXSaveMeshToXA;

HRESULT function(
        LPCWSTR pFilename,
        ID3DXMesh pMesh,
         DWORD* pAdjacency,
         D3DXMATERIAL* pMaterials,
         D3DXEFFECTINSTANCE* pEffectInstances, 
        DWORD NumMaterials,
        DWORD Format
        ) D3DXSaveMeshToXW;

HRESULT function(
        IStream *pStream, 
        DWORD Options,
        IDirect3DDevice9 pD3DDevice, 
        ID3DXBuffer *ppMaterials,
        ID3DXBuffer *ppEffectInstances, 
        DWORD* pNumMaterials,
        ID3DXPMesh *ppPMesh) D3DXCreatePMeshFromStream;

HRESULT function(
        DWORD NumVertices,
         D3DVERTEXELEMENT9 *pDeclaration, 
        DWORD NumBones,
        ID3DXSkinInfo* ppSkinInfo) D3DXCreateSkinInfo;

HRESULT function(
        DWORD NumVertices,
        DWORD FVF,
        DWORD NumBones,
        ID3DXSkinInfo* ppSkinInfo) D3DXCreateSkinInfoFVF;

HRESULT function(
        IDirectXFileData pXofObjMesh, 
        DWORD Options, 
        IDirect3DDevice9 pD3DDevice, 
        ID3DXBuffer *ppAdjacency,
        ID3DXBuffer *ppMaterials, 
        ID3DXBuffer *ppEffectInstances, 
        DWORD *pNumMaterials,
        ID3DXMesh *ppMesh) D3DXLoadMeshFromXof;

HRESULT function(
        IDirectXFileData pxofobjMesh, 
        DWORD Options,
        IDirect3DDevice9 pD3D,
        ID3DXBuffer* ppAdjacency,
        ID3DXBuffer* ppMaterials,
        ID3DXBuffer *ppEffectInstances, 
        DWORD *pMatOut,
        ID3DXSkinInfo* ppSkinInfo,
        ID3DXMesh* ppMesh) D3DXLoadSkinMeshFromXof;

HRESULT function(
        ID3DXBaseMesh pMesh,
        DWORD NumBoneCombinations,
        D3DXBONECOMBINATION* pBoneCombinationTable,
        ID3DXSkinInfo* ppSkinInfo) D3DXCreateSkinInfoFromBlendedMesh;
        
HRESULT function(
        ID3DXMesh pMeshIn,             
         DWORD* pAdjacencyIn,             
        FLOAT NumSegs,                    
        BOOL  QuadraticInterpNormals,     // if false use linear intrep for normals, if true use quadratic
        ID3DXMesh *ppMeshOut,
        ID3DXBuffer *ppAdjacencyOut) D3DXTessellateNPatches;

HRESULT function(
        D3DVERTEXELEMENT9 *pOutput,
        D3DVERTEXELEMENT9 *pInput) D3DXGenerateOutputDecl;

HRESULT function(
        IDirectXFileData pXofObjMesh,
        DWORD Options,
        IDirect3DDevice9 pDevice,
        ID3DXBuffer *ppMaterials,
        ID3DXBuffer *ppEffectInstances, 
        PDWORD pNumMaterials,
        ID3DXPatchMesh *ppMesh) D3DXLoadPatchMeshFromXof;

HRESULT function(
         FLOAT *pfNumSegs, //segments for each edge (4)
        DWORD *pdwTriangles,    //output number of triangles
        DWORD *pdwVertices) D3DXRectPatchSize;    //output number of vertices
 
HRESULT function(
         FLOAT *pfNumSegs, //segments for each edge (3)    
        DWORD *pdwTriangles,    //output number of triangles
        DWORD *pdwVertices) D3DXTriPatchSize;    //output number of vertices

HRESULT function(
        IDirect3DVertexBuffer9 pVB,
         FLOAT *pNumSegs,
         D3DVERTEXELEMENT9 *pdwInDecl,
         D3DRECTPATCH_INFO *pRectPatchInfo,
        ID3DXMesh pMesh) D3DXTessellateRectPatch;

HRESULT function(
      IDirect3DVertexBuffer9 pVB,
       FLOAT *pNumSegs,
       D3DVERTEXELEMENT9 *pInDecl,
       D3DTRIPATCH_INFO *pTriPatchInfo,
      ID3DXMesh pMesh) D3DXTessellateTriPatch;

HRESULT function(
        ID3DXMesh pMeshSysMem,
        ID3DXPatchMesh *pPatchMesh) D3DXCreateNPatchMesh;

HRESULT function(
        D3DXPATCHINFO* pInfo, //patch type
        DWORD dwNumPatches,    //number of patches
        DWORD dwNumVertices,   //number of control vertices
        DWORD dwOptions,       //options 
         D3DVERTEXELEMENT9 *pDecl, //format of control vertices
        IDirect3DDevice9 pDevice, 
        ID3DXPatchMesh *pPatchMesh) D3DXCreatePatchMesh;

HRESULT function(ID3DXPatchMesh pMesh,
                        DWORD *dwcDegenerateVertices,
                        DWORD *dwcDegeneratePatches,
                        ID3DXBuffer *ppErrorsAndWarnings) D3DXValidPatchMesh;

UINT function(DWORD FVF) D3DXGetFVFVertexSize;

UINT function( D3DVERTEXELEMENT9 *pDecl,DWORD Stream) D3DXGetDeclVertexSize;

UINT function( D3DVERTEXELEMENT9 *pDecl) D3DXGetDeclLength;

HRESULT function(
        DWORD FVF,
        D3DVERTEXELEMENT9 pDeclarator[MAX_FVF_DECL_SIZE]) D3DXDeclaratorFromFVF;

HRESULT function(
         D3DVERTEXELEMENT9 *pDeclarator,
        DWORD *pFVF) D3DXFVFFromDeclarator;

HRESULT function(
         ID3DXMesh pMesh,         
        DWORD Flags,
         D3DXWELDEPSILONS *pEpsilons,                 
         DWORD *pAdjacencyIn, 
        DWORD *pAdjacencyOut,
        DWORD *pFaceRemap, 
        ID3DXBuffer *ppVertexRemap) D3DXWeldVertices;

HRESULT function(
        ID3DXBaseMesh pMesh,
         D3DXVECTOR3 *pRayPos,
         D3DXVECTOR3 *pRayDir, 
        BOOL    *pHit,              // True if any faces were intersected
        DWORD   *pFaceIndex,        // index of closest face intersected
        FLOAT   *pU,                // Barycentric Hit Coordinates    
        FLOAT   *pV,                // Barycentric Hit Coordinates
        FLOAT   *pDist,             // Ray-Intersection Parameter Distance
        ID3DXBuffer *ppAllHits,    // Array of D3DXINTERSECTINFOs for all hits (not just closest) 
        DWORD   *pCountOfHits) D3DXIntersect;     // Number of entries in AllHits array

HRESULT function(
        ID3DXBaseMesh pMesh,
        DWORD AttribId,
         D3DXVECTOR3 *pRayPos,
         D3DXVECTOR3 *pRayDir, 
        BOOL    *pHit,              // True if any faces were intersected
        DWORD   *pFaceIndex,        // index of closest face intersected
        FLOAT   *pU,                // Barycentric Hit Coordinates    
        FLOAT   *pV,                // Barycentric Hit Coordinates
        FLOAT   *pDist,             // Ray-Intersection Parameter Distance
        ID3DXBuffer *ppAllHits,    // Array of D3DXINTERSECTINFOs for all hits (not just closest) 
        DWORD   *pCountOfHits) D3DXIntersectSubset;     // Number of entries in AllHits array

HRESULT function(
     ID3DXMesh pMeshIn,         
     DWORD *pAdjacencyIn, 
     DWORD MaxSize,
     DWORD Options,
    DWORD *pMeshesOut,
    ID3DXBuffer *ppMeshArrayOut,
    ID3DXBuffer *ppAdjacencyArrayOut,
    ID3DXBuffer *ppFaceRemapArrayOut,
    ID3DXBuffer *ppVertRemapArrayOut
    ) D3DXSplitMesh;

BOOL function(
     D3DXVECTOR3 *p0,           // Triangle vertex 0 position
     D3DXVECTOR3 *p1,           // Triangle vertex 1 position
     D3DXVECTOR3 *p2,           // Triangle vertex 2 position
     D3DXVECTOR3 *pRayPos,      // Ray origin
     D3DXVECTOR3 *pRayDir,      // Ray direction
    FLOAT *pU,                       // Barycentric Hit Coordinates
    FLOAT *pV,                       // Barycentric Hit Coordinates
    FLOAT *pDist) D3DXIntersectTri;                   // Ray-Intersection Parameter Distance

BOOL function(
         D3DXVECTOR3 *pCenter,
        FLOAT Radius,
         D3DXVECTOR3 *pRayPosition,
         D3DXVECTOR3 *pRayDirection) D3DXSphereBoundProbe;

BOOL function(
         D3DXVECTOR3 *pMin, 
         D3DXVECTOR3 *pMax,
         D3DXVECTOR3 *pRayPosition,
         D3DXVECTOR3 *pRayDirection) D3DXBoxBoundProbe;

HRESULT function(ID3DXMesh Mesh,
                                 DWORD TexStage,
                                 DWORD TangentIndex,
                                 DWORD BinormIndex,
                                 DWORD Wrap,
                                 DWORD *Adjacency) D3DXComputeTangent;

HRESULT function
(
    ID3DXBaseMesh MeshIn,
    DWORD AttribId,
    DWORD IBOptions,
    IDirect3DIndexBuffer9 *ppIndexBuffer,
    DWORD *pNumIndices
) D3DXConvertMeshSubsetToSingleStrip;

HRESULT function(
    ID3DXBaseMesh MeshIn,
    DWORD AttribId,
    DWORD IBOptions,
    IDirect3DIndexBuffer9 *ppIndexBuffer,
    DWORD *pNumIndices,
    ID3DXBuffer *ppStripLengths,
    DWORD *pNumStrips
) D3DXConvertMeshSubsetToStrips;





FLOAT D3DXToRadian(FLOAT degree) { return (degree * (D3DX_PI / 180.0f)); }
FLOAT D3DXToDegree(FLOAT radian) { return (radian * (180.0f / D3DX_PI)); }


extern (Windows) D3DXFLOAT16* function( D3DXFLOAT16 *pOut, FLOAT *pIn, UINT n ) D3DXFloat32To16Array;
extern (Windows) FLOAT* function( FLOAT *pOut, D3DXFLOAT16 *pIn, UINT n ) D3DXFloat16To32Array;



FLOAT D3DXVec2Length( D3DXVECTOR2 *pV )
{
    return sqrt(pV.x * pV.x + pV.y * pV.y);
}

FLOAT D3DXVec2LengthSq( D3DXVECTOR2 *pV )
{
    return pV.x * pV.x + pV.y * pV.y;
}

FLOAT D3DXVec2Dot( D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2 )
{
    return pV1.x * pV2.x + pV1.y * pV2.y;
}

FLOAT D3DXVec2CCW( D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2 )
{
    return pV1.x * pV2.y - pV1.y * pV2.x;
}

D3DXVECTOR2* D3DXVec2Add( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2 )
{
    pOut.x = pV1.x + pV2.x;
    pOut.y = pV1.y + pV2.y;
    return pOut;
}

D3DXVECTOR2* D3DXVec2Subtract( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2 )
{
    pOut.x = pV1.x - pV2.x;
    pOut.y = pV1.y - pV2.y;
    return pOut;
}

D3DXVECTOR2* D3DXVec2Minimize( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2 )
{
    pOut.x = pV1.x < pV2.x ? pV1.x : pV2.x;
    pOut.y = pV1.y < pV2.y ? pV1.y : pV2.y;
    return pOut;
}

D3DXVECTOR2* D3DXVec2Maximize( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2 )
{
    pOut.x = pV1.x > pV2.x ? pV1.x : pV2.x;
    pOut.y = pV1.y > pV2.y ? pV1.y : pV2.y;
    return pOut;
}

D3DXVECTOR2* D3DXVec2Scale( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV, FLOAT s )
{
    pOut.x = pV.x * s;
    pOut.y = pV.y * s;
    return pOut;
}

D3DXVECTOR2* D3DXVec2Lerp( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2,
      FLOAT s )
{
    pOut.x = pV1.x + s * (pV2.x - pV1.x);
    pOut.y = pV1.y + s * (pV2.y - pV1.y);
    return pOut;
}



D3DXVECTOR2* function( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV ) D3DXVec2Normalize;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pT1, D3DXVECTOR2 *pV2, D3DXVECTOR2 *pT2, FLOAT s ) D3DXVec2Hermite;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV0, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2, D3DXVECTOR2 *pV3, FLOAT s ) D3DXVec2CatmullRom;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV1, D3DXVECTOR2 *pV2, D3DXVECTOR2 *pV3, FLOAT f, FLOAT g) D3DXVec2BaryCentric;
D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR2 *pV, D3DXMATRIX *pM ) D3DXVec2Transform;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV, D3DXMATRIX *pM ) D3DXVec2TransformCoord;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, D3DXVECTOR2 *pV, D3DXMATRIX *pM ) D3DXVec2TransformNormal;
D3DXVECTOR4* function( D3DXVECTOR4 *pOut, UINT OutStride, D3DXVECTOR2 *pV, UINT VStride, D3DXMATRIX *pM, UINT n) D3DXVec2TransformArray;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, UINT OutStride, D3DXVECTOR2 *pV, UINT VStride, D3DXMATRIX *pM, UINT n ) D3DXVec2TransformCoordArray;
D3DXVECTOR2* function( D3DXVECTOR2 *pOut, UINT OutStride, D3DXVECTOR2 *pV, UINT VStride, D3DXMATRIX *pM, UINT n ) D3DXVec2TransformNormalArray;



   
FLOAT D3DXVec3Length( D3DXVECTOR3 *pV )
{
    return sqrt(pV.x * pV.x + pV.y * pV.y + pV.z * pV.z);
}

FLOAT D3DXVec3LengthSq( D3DXVECTOR3 *pV )
{
    return pV.x * pV.x + pV.y * pV.y + pV.z * pV.z;
}

FLOAT D3DXVec3Dot( D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2 )
{
    return pV1.x * pV2.x + pV1.y * pV2.y + pV1.z * pV2.z;
}

D3DXVECTOR3* D3DXVec3Cross( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2 )
{
    D3DXVECTOR3 v;

    v.x = pV1.y * pV2.z - pV1.z * pV2.y;
    v.y = pV1.z * pV2.x - pV1.x * pV2.z;
    v.z = pV1.x * pV2.y - pV1.y * pV2.x;

    *pOut = v;
    return pOut;
}

D3DXVECTOR3* D3DXVec3Add( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2 )
{
    pOut.x = pV1.x + pV2.x;
    pOut.y = pV1.y + pV2.y;
    pOut.z = pV1.z + pV2.z;
    return pOut;
}

D3DXVECTOR3* D3DXVec3Subtract( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2 )
{
    pOut.x = pV1.x - pV2.x;
    pOut.y = pV1.y - pV2.y;
    pOut.z = pV1.z - pV2.z;
    return pOut;
}

D3DXVECTOR3* D3DXVec3Minimize( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2 )
{
    pOut.x = pV1.x < pV2.x ? pV1.x : pV2.x;
    pOut.y = pV1.y < pV2.y ? pV1.y : pV2.y;
    pOut.z = pV1.z < pV2.z ? pV1.z : pV2.z;
    return pOut;
}

D3DXVECTOR3* D3DXVec3Maximize( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2 )
{
    pOut.x = pV1.x > pV2.x ? pV1.x : pV2.x;
    pOut.y = pV1.y > pV2.y ? pV1.y : pV2.y;
    pOut.z = pV1.z > pV2.z ? pV1.z : pV2.z;
    return pOut;
}

D3DXVECTOR3* D3DXVec3Scale( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV, FLOAT s)
{
    pOut.x = pV.x * s;
    pOut.y = pV.y * s;
    pOut.z = pV.z * s;
    return pOut;
}

D3DXVECTOR3* D3DXVec3Lerp( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2, FLOAT s )
{
    pOut.x = pV1.x + s * (pV2.x - pV1.x);
    pOut.y = pV1.y + s * (pV2.y - pV1.y);
    pOut.z = pV1.z + s * (pV2.z - pV1.z);
    return pOut;
}



FLOAT D3DXVec4Length( D3DXVECTOR4 *pV )
{
    return sqrt(pV.x * pV.x + pV.y * pV.y + pV.z * pV.z + pV.w * pV.w);
}

FLOAT D3DXVec4LengthSq( D3DXVECTOR4 *pV )
{
    return pV.x * pV.x + pV.y * pV.y + pV.z * pV.z + pV.w * pV.w;
}

FLOAT D3DXVec4Dot( D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2 )
{
    return pV1.x * pV2.x + pV1.y * pV2.y + pV1.z * pV2.z + pV1.w * pV2.w;
}

D3DXVECTOR4* D3DXVec4Add( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2)
{
    pOut.x = pV1.x + pV2.x;
    pOut.y = pV1.y + pV2.y;
    pOut.z = pV1.z + pV2.z;
    pOut.w = pV1.w + pV2.w;
    return pOut;
}

D3DXVECTOR4* D3DXVec4Subtract( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2)
{
    pOut.x = pV1.x - pV2.x;
    pOut.y = pV1.y - pV2.y;
    pOut.z = pV1.z - pV2.z;
    pOut.w = pV1.w - pV2.w;
    return pOut;
}

D3DXVECTOR4* D3DXVec4Minimize( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2)
{
    pOut.x = pV1.x < pV2.x ? pV1.x : pV2.x;
    pOut.y = pV1.y < pV2.y ? pV1.y : pV2.y;
    pOut.z = pV1.z < pV2.z ? pV1.z : pV2.z;
    pOut.w = pV1.w < pV2.w ? pV1.w : pV2.w;
    return pOut;
}

D3DXVECTOR4* D3DXVec4Maximize( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2)
{
    pOut.x = pV1.x > pV2.x ? pV1.x : pV2.x;
    pOut.y = pV1.y > pV2.y ? pV1.y : pV2.y;
    pOut.z = pV1.z > pV2.z ? pV1.z : pV2.z;
    pOut.w = pV1.w > pV2.w ? pV1.w : pV2.w;
    return pOut;
}

D3DXVECTOR4* D3DXVec4Scale( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV, FLOAT s)
{
    pOut.x = pV.x * s;
    pOut.y = pV.y * s;
    pOut.z = pV.z * s;
    pOut.w = pV.w * s;
    return pOut;
}

D3DXVECTOR4* D3DXVec4Lerp( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2, FLOAT s )
{
    pOut.x = pV1.x + s * (pV2.x - pV1.x);
    pOut.y = pV1.y + s * (pV2.y - pV1.y);
    pOut.z = pV1.z + s * (pV2.z - pV1.z);
    pOut.w = pV1.w + s * (pV2.w - pV1.w);
    return pOut;
}


D3DXMATRIX* D3DXMatrixIdentity( D3DXMATRIX *pOut )
{
    pOut.m[0][1] = pOut.m[0][2] = pOut.m[0][3] =
    pOut.m[1][0] = pOut.m[1][2] = pOut.m[1][3] =
    pOut.m[2][0] = pOut.m[2][1] = pOut.m[2][3] =
    pOut.m[3][0] = pOut.m[3][1] = pOut.m[3][2] = 0.0f;

    pOut.m[0][0] = pOut.m[1][1] = pOut.m[2][2] = pOut.m[3][3] = 1.0f;
    return pOut;
}


BOOL D3DXMatrixIsIdentity( D3DXMATRIX *pM )
{
    return pM.m[0][0] == 1.0f && pM.m[0][1] == 0.0f && pM.m[0][2] == 0.0f && pM.m[0][3] == 0.0f &&
           pM.m[1][0] == 0.0f && pM.m[1][1] == 1.0f && pM.m[1][2] == 0.0f && pM.m[1][3] == 0.0f &&
           pM.m[2][0] == 0.0f && pM.m[2][1] == 0.0f && pM.m[2][2] == 1.0f && pM.m[2][3] == 0.0f &&
           pM.m[3][0] == 0.0f && pM.m[3][1] == 0.0f && pM.m[3][2] == 0.0f && pM.m[3][3] == 1.0f;
}



FLOAT D3DXQuaternionLength( D3DXQUATERNION *pQ )
{
    return sqrt(pQ.x * pQ.x + pQ.y * pQ.y + pQ.z * pQ.z + pQ.w * pQ.w);
}

FLOAT D3DXQuaternionLengthSq( D3DXQUATERNION *pQ )
{
    return pQ.x * pQ.x + pQ.y * pQ.y + pQ.z * pQ.z + pQ.w * pQ.w;
}

FLOAT D3DXQuaternionDot( D3DXQUATERNION *pQ1, D3DXQUATERNION *pQ2 )
{
    return pQ1.x * pQ2.x + pQ1.y * pQ2.y + pQ1.z * pQ2.z + pQ1.w * pQ2.w;
}


D3DXQUATERNION* D3DXQuaternionIdentity( D3DXQUATERNION *pOut )
{
    pOut.x = pOut.y = pOut.z = 0.0f;
    pOut.w = 1.0f;
    return pOut;
}

BOOL D3DXQuaternionIsIdentity( D3DXQUATERNION *pQ )
{
    return pQ.x == 0.0f && pQ.y == 0.0f && pQ.z == 0.0f && pQ.w == 1.0f;
}


D3DXQUATERNION* D3DXQuaternionConjugate( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ )
{
    pOut.x = -pQ.x;
    pOut.y = -pQ.y;
    pOut.z = -pQ.z;
    pOut.w =  pQ.w;
    return pOut;
}



FLOAT D3DXPlaneDot( D3DXPLANE *pP, D3DXVECTOR4 *pV)
{
    return pP.a * pV.x + pP.b * pV.y + pP.c * pV.z + pP.d * pV.w;
}

FLOAT D3DXPlaneDotCoord( D3DXPLANE *pP, D3DXVECTOR3 *pV)
{
    return pP.a * pV.x + pP.b * pV.y + pP.c * pV.z + pP.d;
}

FLOAT D3DXPlaneDotNormal( D3DXPLANE *pP, D3DXVECTOR3 *pV)
{
    return pP.a * pV.x + pP.b * pV.y + pP.c * pV.z;
}




D3DXCOLOR* D3DXColorNegative(D3DXCOLOR *pOut, D3DXCOLOR *pC)
{
    pOut.r = 1.0f - pC.r;
    pOut.g = 1.0f - pC.g;
    pOut.b = 1.0f - pC.b;
    pOut.a = pC.a;
    return pOut;
}

D3DXCOLOR* D3DXColorAdd(D3DXCOLOR *pOut, D3DXCOLOR *pC1, D3DXCOLOR *pC2)
{
    pOut.r = pC1.r + pC2.r;
    pOut.g = pC1.g + pC2.g;
    pOut.b = pC1.b + pC2.b;
    pOut.a = pC1.a + pC2.a;
    return pOut;
}

D3DXCOLOR* D3DXColorSubtract(D3DXCOLOR *pOut, D3DXCOLOR *pC1, D3DXCOLOR *pC2)
{
    pOut.r = pC1.r - pC2.r;
    pOut.g = pC1.g - pC2.g;
    pOut.b = pC1.b - pC2.b;
    pOut.a = pC1.a - pC2.a;
    return pOut;
}

D3DXCOLOR* D3DXColorScale(D3DXCOLOR *pOut, D3DXCOLOR *pC, FLOAT s)
{
    pOut.r = pC.r * s;
    pOut.g = pC.g * s;
    pOut.b = pC.b * s;
    pOut.a = pC.a * s;
    return pOut;
}

D3DXCOLOR* D3DXColorModulate(D3DXCOLOR *pOut, D3DXCOLOR *pC1, D3DXCOLOR *pC2)
{
    pOut.r = pC1.r * pC2.r;
    pOut.g = pC1.g * pC2.g;
    pOut.b = pC1.b * pC2.b;
    pOut.a = pC1.a * pC2.a;
    return pOut;
}

D3DXCOLOR* D3DXColorLerp(D3DXCOLOR *pOut, D3DXCOLOR *pC1, D3DXCOLOR *pC2, FLOAT s)
{
    pOut.r = pC1.r + s * (pC2.r - pC1.r);
    pOut.g = pC1.g + s * (pC2.g - pC1.g);
    pOut.b = pC1.b + s * (pC2.b - pC1.b);
    pOut.a = pC1.a + s * (pC2.a - pC1.a);
    return pOut;
}



D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV ) D3DXVec3Normalize;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pT1,
      D3DXVECTOR3 *pV2, D3DXVECTOR3 *pT2, FLOAT s ) D3DXVec3Hermite;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV0, D3DXVECTOR3 *pV1,
      D3DXVECTOR3 *pV2, D3DXVECTOR3 *pV3, FLOAT s ) D3DXVec3CatmullRom;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2,
      D3DXVECTOR3 *pV3, FLOAT f, FLOAT g) D3DXVec3BaryCentric;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR3 *pV, D3DXMATRIX *pM ) D3DXVec3Transform;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV, D3DXMATRIX *pM ) D3DXVec3TransformCoord;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV, D3DXMATRIX *pM ) D3DXVec3TransformNormal;
    
D3DXVECTOR4* function( D3DXVECTOR4 *pOut, UINT OutStride, D3DXVECTOR3 *pV, UINT VStride, D3DXMATRIX *pM, UINT n ) D3DXVec3TransformArray;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, UINT OutStride, D3DXVECTOR3 *pV, UINT VStride, D3DXMATRIX *pM, UINT n ) D3DXVec3TransformCoordArray;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, UINT OutStride, D3DXVECTOR3 *pV, UINT VStride, D3DXMATRIX *pM, UINT n ) D3DXVec3TransformNormalArray;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV, D3DVIEWPORT9 *pViewport,
      D3DXMATRIX *pProjection, D3DXMATRIX *pView, D3DXMATRIX *pWorld) D3DXVec3Project;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXVECTOR3 *pV, D3DVIEWPORT9 *pViewport,
      D3DXMATRIX *pProjection, D3DXMATRIX *pView, D3DXMATRIX *pWorld) D3DXVec3Unproject;
      
D3DXVECTOR3* function( D3DXVECTOR3 *pOut, UINT OutStride,D3DXVECTOR3 *pV, UINT VStride,D3DVIEWPORT9 *pViewport,
      D3DXMATRIX *pProjection, D3DXMATRIX *pView, D3DXMATRIX *pWorld, UINT n) D3DXVec3ProjectArray;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, UINT OutStride, D3DXVECTOR3 *pV, UINT VStride, D3DVIEWPORT9 *pViewport,
      D3DXMATRIX *pProjection, D3DXMATRIX *pView, D3DXMATRIX *pWorld, UINT n) D3DXVec3UnprojectArray;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2,
      D3DXVECTOR4 *pV3) D3DXVec4Cross;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV ) D3DXVec4Normalize;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pT1,
      D3DXVECTOR4 *pV2, D3DXVECTOR4 *pT2, FLOAT s ) D3DXVec4Hermite;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV0, D3DXVECTOR4 *pV1,
      D3DXVECTOR4 *pV2, D3DXVECTOR4 *pV3, FLOAT s ) D3DXVec4CatmullRom;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV1, D3DXVECTOR4 *pV2,
      D3DXVECTOR4 *pV3, FLOAT f, FLOAT g) D3DXVec4BaryCentric;

D3DXVECTOR4* function( D3DXVECTOR4 *pOut, D3DXVECTOR4 *pV, D3DXMATRIX *pM ) D3DXVec4Transform;
    
D3DXVECTOR4* function( D3DXVECTOR4 *pOut, UINT OutStride, D3DXVECTOR4 *pV, UINT VStride, D3DXMATRIX *pM, UINT n ) D3DXVec4TransformArray;

FLOAT function( D3DXMATRIX *pM ) D3DXMatrixDeterminant;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXMATRIX *pM ) D3DXMatrixTranspose;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXMATRIX *pM1, D3DXMATRIX *pM2 ) D3DXMatrixMultiply;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXMATRIX *pM1, D3DXMATRIX *pM2 ) D3DXMatrixMultiplyTranspose;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT *pDeterminant, D3DXMATRIX *pM ) D3DXMatrixInverse;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT sx, FLOAT sy, FLOAT sz ) D3DXMatrixScaling;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT x, FLOAT y, FLOAT z ) D3DXMatrixTranslation;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT Angle ) D3DXMatrixRotationX;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT Angle ) D3DXMatrixRotationY;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT Angle ) D3DXMatrixRotationZ;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXVECTOR3 *pV, FLOAT Angle ) D3DXMatrixRotationAxis;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXQUATERNION *pQ) D3DXMatrixRotationQuaternion;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT Yaw, FLOAT Pitch, FLOAT Roll ) D3DXMatrixRotationYawPitchRoll;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXVECTOR3 *pScalingCenter,
      D3DXQUATERNION *pScalingRotation, D3DXVECTOR3 *pScaling,
      D3DXVECTOR3 *pRotationCenter, D3DXQUATERNION *pRotation,
      D3DXVECTOR3 *pTranslation) D3DXMatrixTransformation;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT Scaling, D3DXVECTOR3 *pRotationCenter,
      D3DXQUATERNION *pRotation, D3DXVECTOR3 *pTranslation) D3DXMatrixAffineTransformation;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXVECTOR3 *pEye, D3DXVECTOR3 *pAt,
      D3DXVECTOR3 *pUp ) D3DXMatrixLookAtRH;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXVECTOR3 *pEye, D3DXVECTOR3 *pAt,
      D3DXVECTOR3 *pUp ) D3DXMatrixLookAtLH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT w, FLOAT h, FLOAT zn, FLOAT zf ) D3DXMatrixPerspectiveRH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT w, FLOAT h, FLOAT zn, FLOAT zf ) D3DXMatrixPerspectiveLH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT fovy, FLOAT Aspect, FLOAT zn, FLOAT zf ) D3DXMatrixPerspectiveFovRH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT fovy, FLOAT Aspect, FLOAT zn, FLOAT zf ) D3DXMatrixPerspectiveFovLH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT l, FLOAT r, FLOAT b, FLOAT t, FLOAT zn,
      FLOAT zf ) D3DXMatrixPerspectiveOffCenterRH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT l, FLOAT r, FLOAT b, FLOAT t, FLOAT zn,
      FLOAT zf ) D3DXMatrixPerspectiveOffCenterLH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT w, FLOAT h, FLOAT zn, FLOAT zf ) D3DXMatrixOrthoRH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT w, FLOAT h, FLOAT zn, FLOAT zf ) D3DXMatrixOrthoLH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT l, FLOAT r, FLOAT b, FLOAT t, FLOAT zn,
      FLOAT zf ) D3DXMatrixOrthoOffCenterRH;

D3DXMATRIX* function( D3DXMATRIX *pOut, FLOAT l, FLOAT r, FLOAT b, FLOAT t, FLOAT zn,
      FLOAT zf ) D3DXMatrixOrthoOffCenterLH;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXVECTOR4 *pLight,
      D3DXPLANE *pPlane ) D3DXMatrixShadow;

D3DXMATRIX* function( D3DXMATRIX *pOut, D3DXPLANE *pPlane ) D3DXMatrixReflect;

void function( D3DXQUATERNION *pQ, D3DXVECTOR3 *pAxis, FLOAT *pAngle ) D3DXQuaternionToAxisAngle;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXMATRIX *pM) D3DXQuaternionRotationMatrix;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXVECTOR3 *pV, FLOAT Angle ) D3DXQuaternionRotationAxis;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, FLOAT Yaw, FLOAT Pitch, FLOAT Roll ) D3DXQuaternionRotationYawPitchRoll;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ1,
      D3DXQUATERNION *pQ2 ) D3DXQuaternionMultiply;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ ) D3DXQuaternionNormalize;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ ) D3DXQuaternionInverse;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ ) D3DXQuaternionLn;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ ) D3DXQuaternionExp;
      
D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ1,
      D3DXQUATERNION *pQ2, FLOAT t ) D3DXQuaternionSlerp;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ1,
      D3DXQUATERNION *pA, D3DXQUATERNION *pB,
      D3DXQUATERNION *pC, FLOAT t ) D3DXQuaternionSquad;

void function( D3DXQUATERNION *pAOut, D3DXQUATERNION *pBOut, D3DXQUATERNION *pCOut,
      D3DXQUATERNION *pQ0, D3DXQUATERNION *pQ1, 
      D3DXQUATERNION *pQ2, D3DXQUATERNION *pQ3 ) D3DXQuaternionSquadSetup;

D3DXQUATERNION* function( D3DXQUATERNION *pOut, D3DXQUATERNION *pQ1,
      D3DXQUATERNION *pQ2, D3DXQUATERNION *pQ3,
      FLOAT f, FLOAT g ) D3DXQuaternionBaryCentric;

D3DXPLANE* function( D3DXPLANE *pOut, D3DXPLANE *pP) D3DXPlaneNormalize;

D3DXVECTOR3* function( D3DXVECTOR3 *pOut, D3DXPLANE *pP, D3DXVECTOR3 *pV1,
      D3DXVECTOR3 *pV2) D3DXPlaneIntersectLine;

D3DXPLANE* function( D3DXPLANE *pOut, D3DXVECTOR3 *pPoint, D3DXVECTOR3 *pNormal) D3DXPlaneFromPointNormal;

D3DXPLANE* function( D3DXPLANE *pOut, D3DXVECTOR3 *pV1, D3DXVECTOR3 *pV2,
      D3DXVECTOR3 *pV3) D3DXPlaneFromPoints;

D3DXPLANE* function( D3DXPLANE *pOut, D3DXPLANE *pP, D3DXMATRIX *pM ) D3DXPlaneTransform;
    
D3DXPLANE* function( D3DXPLANE *pOut, UINT OutStride, D3DXPLANE *pP, UINT PStride, D3DXMATRIX *pM, UINT n ) D3DXPlaneTransformArray;

D3DXCOLOR* function(D3DXCOLOR *pOut, D3DXCOLOR *pC, FLOAT s) D3DXColorAdjustSaturation;

D3DXCOLOR* function(D3DXCOLOR *pOut, D3DXCOLOR *pC, FLOAT c) D3DXColorAdjustContrast;

FLOAT function(FLOAT CosTheta, FLOAT RefractionIndex) D3DXFresnelTerm;

HRESULT function(DWORD Flags, ID3DXMatrixStack* ppStack) D3DXCreateMatrixStack;

extern(Windows) char* DXGetErrorDescription9A(HRESULT hr);
alias DXGetErrorDescription9A DXGetErrorDescription9;





































//private extern (Windows) IDirect3D9 function (UINT SDKVersion) Direct3DCreate9Func;
extern (Windows) IDirect3D9 function (UINT SDKVersion) Direct3DCreate9;

/*void Direct3DInit()
{
    static bit initialized;
    static bit erroneous;
    static HANDLE library;
    const char[] libraryName = "d3d9.dll";

    void load(void** pointer, char[] name)
    {
        *pointer = GetProcAddress(library, toStringz(name));
        if (*pointer === null)
        {
            erroneous = true;
            throw new Error("Couldn't initialize Direct3D because '" ~ name ~ "' does not exist in '" ~ libraryName ~ "'");
        }
    }

    if (initialized)
        return;
    if (erroneous)
        throw new Error("Direct3D could not be initialized.");

    if ((library = LoadLibraryA(libraryName)) === cast(HANDLE) 0)
    {
        erroneous = true;
        throw new Error("Couldn't initialize Direct3D because '" ~ libraryName ~ "' could not be loaded.");
    }

    load(cast(void**) &Direct3DCreate9Func, "Direct3DCreate9");
    initialized = true;
}*/

/*
IDirect3D9 Direct3DCreate9(UINT SDKVersion)
{
	//Direct3DInit();
	IDirect3D9 id3d=Direct3DCreate9(SDKVersion);
    if(!id3d) throw new Error("Could not create D3D device");
    return id3d;
}*/
